@@ -1,7 +1,7 @@
 /*  audio_call.c
  *
  *
- *  Copyright (C) 2014 Toxic All Rights Reserved.
+ *  Copyright (C) 2024 Toxic All Rights Reserved.
  *
  *  This file is part of Toxic.
  *
@@ -67,66 +67,72 @@ void on_call_state(ToxAV *av, uint32_t friend_number, uint32_t state, void *user
 void on_audio_receive_frame(ToxAV *av, uint32_t friend_number, int16_t const *pcm, size_t sample_count,
                             uint8_t channels, uint32_t sampling_rate, void *user_data);
 
-void callback_recv_invite(Tox *m, uint32_t friend_number);
-void callback_recv_ringing(uint32_t friend_number);
-void callback_recv_starting(uint32_t friend_number);
-void callback_recv_ending(uint32_t friend_number);
-void callback_call_started(uint32_t friend_number);
-void callback_call_canceled(uint32_t friend_number);
-void callback_call_rejected(uint32_t friend_number);
-void callback_call_ended(uint32_t friend_number);
+void callback_recv_invite(Toxic *toxic, uint32_t friend_number);
+void callback_recv_ringing(Toxic *toxic, uint32_t friend_number);
+void callback_recv_starting(Toxic *toxic, uint32_t friend_number);
+void callback_call_started(Toxic *toxic, uint32_t friend_number);
+void callback_call_canceled(Toxic *toxic, uint32_t friend_number);
+void callback_call_rejected(Toxic *toxic, uint32_t friend_number);
+void callback_call_ended(Toxic *toxic, uint32_t friend_number);
 
 void write_device_callback(uint32_t friend_number, const int16_t *PCM, uint16_t sample_count, uint8_t channels,
                            uint32_t sample_rate);
 
 void audio_bit_rate_callback(ToxAV *av, uint32_t friend_number, uint32_t audio_bit_rate, void *user_data);
 
-static void print_err(ToxWindow *self, const char *error_str)
+static void print_err(ToxWindow *self, const Client_Config *c_config, const char *error_str)
 {
-    line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, "%s", error_str);
+    line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "%s", error_str);
 }
 
-ToxAV *init_audio(ToxWindow *self, Tox *tox)
+ToxAV *init_audio(Toxic *toxic)
 {
+    if (toxic == NULL) {
+        return NULL;
+    }
+
+    ToxWindow *self = toxic->home_window;
+    const Client_Config *c_config = toxic->c_config;
+
     Toxav_Err_New error;
     CallControl.audio_errors = ae_None;
-    CallControl.prompt = self;
 
-    CallControl.av = toxav_new(tox, &error);
+    toxic->av = toxav_new(toxic->tox, &error);
 
     CallControl.audio_enabled = true;
     CallControl.default_audio_bit_rate = 64;
     CallControl.audio_sample_rate = 48000;
     CallControl.audio_frame_duration = 20;
-    CallControl.audio_channels = user_settings->chat_audio_channels;
+    CallControl.audio_channels = c_config->chat_audio_channels;
 
     CallControl.video_enabled = false;
     CallControl.default_video_bit_rate = 0;
     CallControl.video_frame_duration = 0;
 
-    if (!CallControl.av) {
+    if (toxic->av == NULL) {
         CallControl.audio_errors |= ae_StartingCoreAudio;
-        line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, "Failed to init ToxAV");
-
+        line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "Failed to init ToxAV");
         return NULL;
     }
 
     if (init_devices() == de_InternalError) {
-        line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, "Failed to init devices");
-        toxav_kill(CallControl.av);
-
-        return CallControl.av = NULL;
+        line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "Failed to init devices");
+        toxav_kill(toxic->av);
+        toxic->av = NULL;
+        return NULL;
     }
 
-    toxav_callback_call(CallControl.av, on_call, tox);
-    toxav_callback_call_state(CallControl.av, on_call_state, NULL);
-    toxav_callback_audio_receive_frame(CallControl.av, on_audio_receive_frame, NULL);
-    toxav_callback_audio_bit_rate(CallControl.av, audio_bit_rate_callback, NULL);
+    toxav_callback_call(toxic->av, on_call, (void *) toxic);
+    toxav_callback_call_state(toxic->av, on_call_state, (void *) toxic);
+    toxav_callback_audio_receive_frame(toxic->av, on_audio_receive_frame, NULL);
+    toxav_callback_audio_bit_rate(toxic->av, audio_bit_rate_callback, NULL);
 
-    return CallControl.av;
+    CallControl.av = toxic->av;  // TODO: get rid of this
+
+    return toxic->av;
 }
 
-void read_device_callback(const int16_t *captured, uint32_t size, void *data)
+static void read_device_callback(const int16_t *captured, uint32_t size, void *data)
 {
     UNUSED_VAR(size);
 
@@ -190,41 +196,52 @@ static bool cancel_call(Call *call)
     return true;
 }
 
-static int start_transmission(ToxWindow *self, Call *call)
+static int start_transmission(ToxWindow *self, Toxic *toxic, Call *call)
 {
-    if (!self || !CallControl.av) {
-        line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, "Failed to prepare audio transmission");
+    if (self == NULL || toxic == NULL) {
+        return -1;
+    }
+
+    const Client_Config *c_config = toxic->c_config;
+
+    if (toxic->av == NULL) {
+        line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "Failed to prepare audio transmission");
         return -1;
     }
 
     DeviceError error = open_input_device(&call->in_idx, read_device_callback, &self->num,
-                                          CallControl.audio_sample_rate, CallControl.audio_frame_duration, CallControl.audio_channels);
+                                          CallControl.audio_sample_rate, CallControl.audio_frame_duration,
+                                          CallControl.audio_channels, c_config->VAD_threshold);
 
     if (error != de_None) {
         if (error == de_FailedStart) {
-            line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, "Failed to start audio input device");
+            line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "Failed to start audio input device");
         }
 
         if (error == de_InternalError) {
-            line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, "Internal error with opening audio input device");
+            line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0,
+                          "Internal error with opening audio input device");
         }
     }
 
     if (open_output_device(&call->out_idx,
-                           CallControl.audio_sample_rate, CallControl.audio_frame_duration, CallControl.audio_channels) != de_None) {
-        line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, "Failed to open audio output device!");
+                           CallControl.audio_sample_rate,
+                           CallControl.audio_frame_duration,
+                           CallControl.audio_channels,
+                           c_config->VAD_threshold) != de_None) {
+        line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "Failed to open audio output device!");
     }
 
     return 0;
 }
 
-static void start_call(ToxWindow *self, Call *call)
+static void start_call(ToxWindow *self, Toxic *toxic, Call *call)
 {
     if (call->status != cs_Pending) {
         return;
     }
 
-    if (start_transmission(self, call) != 0) {
+    if (start_transmission(self, toxic, call) != 0) {
         return;
     }
 
@@ -237,7 +254,7 @@ static void start_call(ToxWindow *self, Call *call)
     }
 
     if (call->video_bit_rate) {
-        start_video_transmission(self, CallControl.av, call);
+        start_video_transmission(self, toxic, call);
     }
 
 #endif
@@ -272,14 +289,14 @@ static int stop_transmission(Call *call, uint32_t friend_number)
     return 0;
 }
 
-void terminate_audio(void)
+void terminate_audio(ToxAV *av)
 {
     for (int i = 0; i < CallControl.max_calls; ++i) {
         stop_transmission(&CallControl.calls[i], i);
     }
 
-    if (CallControl.av) {
-        toxav_kill(CallControl.av);
+    if (av) {
+        toxav_kill(av);
     }
 
     terminate_devices();
@@ -300,7 +317,7 @@ void on_call(ToxAV *av, uint32_t friend_number, bool audio_enabled, bool video_e
 {
     UNUSED_VAR(av);
 
-    Tox *m = (Tox *) user_data;
+    Toxic *toxic = (Toxic *) user_data;
 
     Call *call = &CallControl.calls[friend_number];
     init_call(call);
@@ -315,13 +332,18 @@ void on_call(ToxAV *av, uint32_t friend_number, bool audio_enabled, bool video_e
         call->state |= TOXAV_FRIEND_CALL_STATE_SENDING_V;
     }
 
-    callback_recv_invite(m, friend_number);
+    callback_recv_invite(toxic, friend_number);
 }
 
 void on_call_state(ToxAV *av, uint32_t friend_number, uint32_t state, void *user_data)
 {
+    Toxic *toxic = (Toxic *) user_data;
+
+    if (toxic == NULL) {
+        return;
+    }
+
     UNUSED_VAR(av);
-    UNUSED_VAR(user_data);
 
     Call *call = &CallControl.calls[friend_number];
 
@@ -335,12 +357,13 @@ void on_call_state(ToxAV *av, uint32_t friend_number, uint32_t state, void *user
         case TOXAV_FRIEND_CALL_STATE_ERROR:
         case TOXAV_FRIEND_CALL_STATE_FINISHED:
             if (state == TOXAV_FRIEND_CALL_STATE_ERROR) {
-                line_info_add(CallControl.prompt, false, NULL, NULL, SYS_MSG, 0, 0, "ToxAV callstate error!");
+                line_info_add(toxic->home_window, toxic->c_config, false, NULL, NULL, SYS_MSG, 0, 0,
+                              "ToxAV callstate error!");
             }
 
             if (call->status == cs_Pending) {
                 cancel_call(call);
-                callback_call_rejected(friend_number);
+                callback_call_rejected(toxic, friend_number);
             } else {
 
 #ifdef VIDEO
@@ -349,7 +372,7 @@ void on_call_state(ToxAV *av, uint32_t friend_number, uint32_t state, void *user
 #endif /* VIDEO */
 
                 stop_transmission(call, friend_number);
-                callback_call_ended(friend_number);
+                callback_call_ended(toxic, friend_number);
             }
 
             break;
@@ -357,7 +380,7 @@ void on_call_state(ToxAV *av, uint32_t friend_number, uint32_t state, void *user
         default:
             if (call->status == cs_Pending) {
                 /* Start answered call */
-                callback_call_started(friend_number);
+                callback_call_started(toxic, friend_number);
             }
 
 #ifdef VIDEO
@@ -394,8 +417,12 @@ void audio_bit_rate_callback(ToxAV *av, uint32_t friend_number, uint32_t audio_b
     toxav_audio_set_bit_rate(av, friend_number, audio_bit_rate, NULL);
 }
 
-void callback_recv_invite(Tox *m, uint32_t friend_number)
+void callback_recv_invite(Toxic *toxic, uint32_t friend_number)
 {
+    if (toxic == NULL) {
+        return;
+    }
+
     if (friend_number >= Friends.max_idx) {
         return;
     }
@@ -405,88 +432,77 @@ void callback_recv_invite(Tox *m, uint32_t friend_number)
             return;
         }
 
-        Friends.list[friend_number].chatwin = add_window(m, new_chat(m, Friends.list[friend_number].num));
+        Friends.list[friend_number].chatwin = add_window(toxic, new_chat(toxic->tox, Friends.list[friend_number].num));
     }
 
     const Call *call = &CallControl.calls[friend_number];
 
     for (uint8_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
         if (windows[i] != NULL && windows[i]->onInvite != NULL && windows[i]->num == friend_number) {
-            windows[i]->onInvite(windows[i], CallControl.av, friend_number, call->state);
+            windows[i]->onInvite(windows[i], toxic, friend_number, call->state);
         }
     }
 }
-void callback_recv_ringing(uint32_t friend_number)
+void callback_recv_ringing(Toxic *toxic, uint32_t friend_number)
 {
     const Call *call = &CallControl.calls[friend_number];
 
     for (uint8_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
         if (windows[i] != NULL && windows[i]->onRinging != NULL && windows[i]->num == friend_number) {
-            windows[i]->onRinging(windows[i], CallControl.av, friend_number, call->state);
+            windows[i]->onRinging(windows[i], toxic, friend_number, call->state);
         }
     }
 }
-void callback_recv_starting(uint32_t friend_number)
+void callback_recv_starting(Toxic *toxic, uint32_t friend_number)
 {
     Call *call = &CallControl.calls[friend_number];
 
     for (uint8_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
         if (windows[i] != NULL && windows[i]->onStarting != NULL && windows[i]->num == friend_number) {
-            windows[i]->onStarting(windows[i], CallControl.av, friend_number, call->state);
-
-            start_call(windows[i], call);
+            windows[i]->onStarting(windows[i], toxic, friend_number, call->state);
+            start_call(windows[i], toxic, call);
         }
     }
 }
-void callback_recv_ending(uint32_t friend_number)
-{
-    const Call *call = &CallControl.calls[friend_number];
-
-    for (uint8_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
-        if (windows[i] != NULL && windows[i]->onEnding != NULL && windows[i]->num == friend_number) {
-            windows[i]->onEnding(windows[i], CallControl.av, friend_number, call->state);
-        }
-    }
-}
-void callback_call_started(uint32_t friend_number)
+void callback_call_started(Toxic *toxic, uint32_t friend_number)
 {
     Call *call = &CallControl.calls[friend_number];
 
     for (uint8_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
         if (windows[i] != NULL && windows[i]->onStart != NULL && windows[i]->num == friend_number) {
-            windows[i]->onStart(windows[i], CallControl.av, friend_number, call->state);
+            windows[i]->onStart(windows[i], toxic, friend_number, call->state);
 
-            start_call(windows[i], call);
+            start_call(windows[i], toxic, call);
         }
     }
 }
-void callback_call_canceled(uint32_t friend_number)
+void callback_call_canceled(Toxic *toxic, uint32_t friend_number)
 {
     const Call *call = &CallControl.calls[friend_number];
 
     for (uint8_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
         if (windows[i] != NULL && windows[i]->onCancel != NULL && windows[i]->num == friend_number) {
-            windows[i]->onCancel(windows[i], CallControl.av, friend_number, call->state);
+            windows[i]->onCancel(windows[i], toxic, friend_number, call->state);
         }
     }
 }
-void callback_call_rejected(uint32_t friend_number)
+void callback_call_rejected(Toxic *toxic, uint32_t friend_number)
 {
     const Call *call = &CallControl.calls[friend_number];
 
     for (uint8_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
         if (windows[i] != NULL && windows[i]->onReject != NULL && windows[i]->num == friend_number) {
-            windows[i]->onReject(windows[i], CallControl.av, friend_number, call->state);
+            windows[i]->onReject(windows[i], toxic, friend_number, call->state);
         }
     }
 }
-void callback_call_ended(uint32_t friend_number)
+void callback_call_ended(Toxic *toxic, uint32_t friend_number)
 {
     const Call *call = &CallControl.calls[friend_number];
 
     for (uint8_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
         if (windows[i] != NULL && windows[i]->onEnd != NULL && windows[i]->num == friend_number) {
-            windows[i]->onEnd(windows[i], CallControl.av, friend_number, call->state);
+            windows[i]->onEnd(windows[i], toxic, friend_number, call->state);
         }
     }
 }
@@ -499,153 +515,178 @@ void callback_call_ended(uint32_t friend_number)
 /*
  * Commands from chat_commands.h
  */
-void cmd_call(WINDOW *window, ToxWindow *self, Tox *m, int argc, char (*argv)[MAX_STR_SIZE])
+void cmd_call(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, char (*argv)[MAX_STR_SIZE])
 {
     UNUSED_VAR(window);
-    UNUSED_VAR(m);
     UNUSED_VAR(argv);
 
-    if (argc != 0) {
-        print_err(self, "Unknown arguments.");
+    if (toxic == NULL || self == NULL) {
         return;
     }
 
-    if (!CallControl.av) {
-        print_err(self, "ToxAV not supported!");
+    const Client_Config *c_config = toxic->c_config;
+
+    if (argc != 0) {
+        print_err(self, c_config, "Unknown arguments.");
+        return;
+    }
+
+    if (toxic->av == NULL) {
+        print_err(self, c_config, "ToxAV not supported!");
         return;
     }
 
     if (!self->stb->connection) {
-        print_err(self, "Friend is offline.");
+        print_err(self, c_config, "Friend is offline.");
         return;
     }
 
     Call *call = &CallControl.calls[self->num];
 
     if (call->status != cs_None) {
-        print_err(self, "Already calling.");
+        print_err(self, c_config, "Already calling.");
         return;
     }
 
     init_call(call);
 
-    place_call(self);
+    place_call(self, toxic);
 }
 
-void cmd_answer(WINDOW *window, ToxWindow *self, Tox *m, int argc, char (*argv)[MAX_STR_SIZE])
+void cmd_answer(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, char (*argv)[MAX_STR_SIZE])
 {
     UNUSED_VAR(window);
-    UNUSED_VAR(m);
     UNUSED_VAR(argv);
 
+    if (toxic == NULL || self == NULL) {
+        return;
+    }
+
+    const Client_Config *c_config = toxic->c_config;
+
     Toxav_Err_Answer error;
 
     if (argc != 0) {
-        print_err(self, "Unknown arguments.");
+        print_err(self, c_config, "Unknown arguments.");
         return;
     }
 
-    if (!CallControl.av) {
-        print_err(self, "Audio not supported!");
+    if (toxic->av == NULL) {
+        print_err(self, c_config, "Audio not supported!");
         return;
     }
 
     Call *call = &CallControl.calls[self->num];
 
     if (call->status != cs_Pending) {
-        print_err(self, "No incoming call!");
+        print_err(self, c_config, "No incoming call!");
         return;
     }
 
-    toxav_answer(CallControl.av, self->num, call->audio_bit_rate, call->video_bit_rate, &error);
+    toxav_answer(toxic->av, self->num, call->audio_bit_rate, call->video_bit_rate, &error);
 
     if (error != TOXAV_ERR_ANSWER_OK) {
         if (error == TOXAV_ERR_ANSWER_FRIEND_NOT_CALLING) {
-            print_err(self, "No incoming call!");
+            print_err(self, c_config, "No incoming call!");
         } else if (error == TOXAV_ERR_ANSWER_CODEC_INITIALIZATION) {
-            print_err(self, "Failed to initialize codecs!");
+            print_err(self, c_config, "Failed to initialize codecs!");
         } else if (error == TOXAV_ERR_ANSWER_FRIEND_NOT_FOUND) {
-            print_err(self, "Friend not found!");
+            print_err(self, c_config, "Friend not found!");
         } else if (error == TOXAV_ERR_ANSWER_INVALID_BIT_RATE) {
-            print_err(self, "Invalid bit rate!");
+            print_err(self, c_config, "Invalid bit rate!");
         } else {
-            print_err(self, "Internal error!");
+            print_err(self, c_config, "Internal error!");
         }
 
         return;
     }
 
     /* Callback will print status... */
-    callback_recv_starting(self->num);
+    callback_recv_starting(toxic, self->num);
 }
 
-void cmd_reject(WINDOW *window, ToxWindow *self, Tox *m, int argc, char (*argv)[MAX_STR_SIZE])
+void cmd_reject(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, char (*argv)[MAX_STR_SIZE])
 {
     UNUSED_VAR(window);
-    UNUSED_VAR(m);
     UNUSED_VAR(argv);
 
-    if (argc != 0) {
-        print_err(self, "Unknown arguments.");
+    if (toxic == NULL || self == NULL) {
         return;
     }
 
-    if (!CallControl.av) {
-        print_err(self, "Audio not supported!");
+    const Client_Config *c_config = toxic->c_config;
+
+    if (argc != 0) {
+        print_err(self, c_config, "Unknown arguments.");
+        return;
+    }
+
+    if (toxic->av == NULL) {
+        print_err(self, c_config, "Audio not supported!");
         return;
     }
 
     Call *call = &CallControl.calls[self->num];
 
     if (call->status != cs_Pending) {
-        print_err(self, "No incoming call!");
+        print_err(self, c_config, "No incoming call!");
         return;
     }
 
     /* Manually send a cancel call control because call hasn't started */
-    toxav_call_control(CallControl.av, self->num, TOXAV_CALL_CONTROL_CANCEL, NULL);
+    toxav_call_control(toxic->av, self->num, TOXAV_CALL_CONTROL_CANCEL, NULL);
     cancel_call(call);
 
     /* Callback will print status... */
-    callback_call_rejected(self->num);
+    callback_call_rejected(toxic, self->num);
 }
 
-void cmd_hangup(WINDOW *window, ToxWindow *self, Tox *m, int argc, char (*argv)[MAX_STR_SIZE])
+void cmd_hangup(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, char (*argv)[MAX_STR_SIZE])
 {
     UNUSED_VAR(window);
-    UNUSED_VAR(m);
     UNUSED_VAR(argv);
 
-    if (!CallControl.av) {
-        print_err(self, "Audio not supported!");
+    if (toxic == NULL || self == NULL) {
+        return;
+    }
+
+    const Client_Config *c_config = toxic->c_config;
+
+    if (toxic->av == NULL) {
+        print_err(self, c_config, "Audio not supported!");
         return;
     }
 
     if (argc != 0) {
-        print_err(self, "Unknown arguments.");
+        print_err(self, c_config, "Unknown arguments.");
         return;
     }
 
     Call *call = &CallControl.calls[self->num];
 
     if (call->status == cs_None) {
-        print_err(self, "Not in a call.");
+        print_err(self, c_config, "Not in a call.");
         return;
     }
 
-    stop_current_call(self);
+    stop_current_call(self, toxic);
 }
 
-void cmd_list_devices(WINDOW *window, ToxWindow *self, Tox *m, int argc, char (*argv)[MAX_STR_SIZE])
+void cmd_list_devices(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, char (*argv)[MAX_STR_SIZE])
 {
     UNUSED_VAR(window);
-    UNUSED_VAR(m);
+
+    if (toxic == NULL || self == NULL) {
+        return;
+    }
+
+    const Client_Config *c_config = toxic->c_config;
 
     if (argc != 1) {
         if (argc < 1) {
-            print_err(self, "Type must be specified!");
+            print_err(self, c_config, "Type must be specified!");
         } else {
-            print_err(self, "Only one argument allowed!");
+            print_err(self, c_config, "Only one argument allowed!");
         }
 
         return;
@@ -662,29 +703,34 @@ void cmd_list_devices(WINDOW *window, ToxWindow *self, Tox *m, int argc, char (*
     }
 
     else {
-        line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, "Invalid type: %s", argv[1]);
+        line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "Invalid type: %s", argv[1]);
         return;
     }
 
     // Refresh device list.
     get_al_device_names();
 
-    print_al_devices(self, type);
+    print_al_devices(self, c_config, type);
 }
 
 /* This changes primary device only */
-void cmd_change_device(WINDOW *window, ToxWindow *self, Tox *m, int argc, char (*argv)[MAX_STR_SIZE])
+void cmd_change_device(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, char (*argv)[MAX_STR_SIZE])
 {
     UNUSED_VAR(window);
-    UNUSED_VAR(m);
+
+    if (toxic == NULL || self == NULL) {
+        return;
+    }
+
+    const Client_Config *c_config = toxic->c_config;
 
     if (argc != 2) {
         if (argc < 1) {
-            print_err(self, "Type must be specified!");
+            print_err(self, c_config, "Type must be specified!");
         } else if (argc < 2) {
-            print_err(self, "Must have id!");
+            print_err(self, c_config, "Must have id!");
         } else {
-            print_err(self, "Only two arguments allowed!");
+            print_err(self, c_config, "Only two arguments allowed!");
         }
 
         return;
@@ -701,7 +747,7 @@ void cmd_change_device(WINDOW *window, ToxWindow *self, Tox *m, int argc, char (
     }
 
     else {
-        line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, "Invalid type: %s", argv[1]);
+        line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "Invalid type: %s", argv[1]);
         return;
     }
 
@@ -710,23 +756,28 @@ void cmd_change_device(WINDOW *window, ToxWindow *self, Tox *m, int argc, char (
     long int selection = strtol(argv[2], &end, 10);
 
     if (*end) {
-        print_err(self, "Invalid input");
+        print_err(self, c_config, "Invalid input");
         return;
     }
 
     if (set_al_device(type, selection) == de_InvalidSelection) {
-        print_err(self, "Invalid selection!");
+        print_err(self, c_config, "Invalid selection!");
         return;
     }
 }
 
-void cmd_mute(WINDOW *window, ToxWindow *self, Tox *m, int argc, char (*argv)[MAX_STR_SIZE])
+void cmd_mute(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, char (*argv)[MAX_STR_SIZE])
 {
     UNUSED_VAR(window);
-    UNUSED_VAR(m);
+
+    if (toxic == NULL || self == NULL) {
+        return;
+    }
+
+    const Client_Config *c_config = toxic->c_config;
 
     if (argc != 1) {
-        print_err(self, "Specify type: \"/mute in\" or \"/mute out\".");
+        print_err(self, c_config, "Specify type: \"/mute in\" or \"/mute out\".");
         return;
     }
 
@@ -741,7 +792,7 @@ void cmd_mute(WINDOW *window, ToxWindow *self, Tox *m, int argc, char (*argv)[MA
     }
 
     else {
-        line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, "Invalid type: %s", argv[1]);
+        line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "Invalid type: %s", argv[1]);
         return;
     }
 
@@ -762,16 +813,21 @@ void cmd_mute(WINDOW *window, ToxWindow *self, Tox *m, int argc, char (*argv)[MA
     return;
 }
 
-void cmd_sense(WINDOW *window, ToxWindow *self, Tox *m, int argc, char (*argv)[MAX_STR_SIZE])
+void cmd_sense(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, char (*argv)[MAX_STR_SIZE])
 {
     UNUSED_VAR(window);
-    UNUSED_VAR(m);
+
+    if (toxic == NULL || self == NULL) {
+        return;
+    }
+
+    const Client_Config *c_config = toxic->c_config;
 
     if (argc != 1) {
         if (argc < 1) {
-            print_err(self, "Must have value!");
+            print_err(self, c_config, "Must have value!");
         } else {
-            print_err(self, "Only two arguments allowed!");
+            print_err(self, c_config, "Only two arguments allowed!");
         }
 
         return;
@@ -781,7 +837,7 @@ void cmd_sense(WINDOW *window, ToxWindow *self, Tox *m, int argc, char (*argv)[M
     float value = strtof(argv[1], &end);
 
     if (*end) {
-        print_err(self, "Invalid input");
+        print_err(self, c_config, "Invalid input");
         return;
     }
 
@@ -796,26 +852,31 @@ void cmd_sense(WINDOW *window, ToxWindow *self, Tox *m, int argc, char (*argv)[M
     return;
 }
 
-void cmd_bitrate(WINDOW *window, ToxWindow *self, Tox *m, int argc, char (*argv)[MAX_STR_SIZE])
+void cmd_bitrate(WINDOW *window, ToxWindow *self, Toxic *toxic, int argc, char (*argv)[MAX_STR_SIZE])
 {
     UNUSED_VAR(window);
-    UNUSED_VAR(m);
+
+    if (toxic == NULL || self == NULL) {
+        return;
+    }
+
+    const Client_Config *c_config = toxic->c_config;
 
     Call *call = &CallControl.calls[self->num];
 
     if (call->status != cs_Active) {
-        print_err(self, "Must be in a call");
+        print_err(self, c_config, "Must be in a call");
         return;
     }
 
     if (argc == 0) {
-        line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0,
+        line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0,
                       "Current audio encoding bitrate: %u", call->audio_bit_rate);
         return;
     }
 
     if (argc > 1) {
-        print_err(self, "Too many arguments.");
+        print_err(self, c_config, "Too many arguments.");
         return;
     }
 
@@ -823,24 +884,24 @@ void cmd_bitrate(WINDOW *window, ToxWindow *self, Tox *m, int argc, char (*argv)
     const long int bit_rate = strtol(argv[1], &end, 10);
 
     if (*end || bit_rate < 0 || bit_rate > UINT32_MAX) {
-        print_err(self, "Invalid input");
+        print_err(self, c_config, "Invalid input");
         return;
     }
 
     Toxav_Err_Bit_Rate_Set error;
-    toxav_audio_set_bit_rate(CallControl.av, self->num, bit_rate, &error);
+    toxav_audio_set_bit_rate(toxic->av, self->num, bit_rate, &error);
 
     if (error != TOXAV_ERR_BIT_RATE_SET_OK) {
         if (error == TOXAV_ERR_BIT_RATE_SET_SYNC) {
-            print_err(self, "Synchronization error occured");
+            print_err(self, c_config, "Synchronization error occured");
         } else if (error == TOXAV_ERR_BIT_RATE_SET_INVALID_BIT_RATE) {
-            print_err(self, "Invalid audio bit rate value (valid is 6-510)");
+            print_err(self, c_config, "Invalid audio bit rate value (valid is 6-510)");
         } else if (error == TOXAV_ERR_BIT_RATE_SET_FRIEND_NOT_FOUND) {
-            print_err(self, "Friend not found");
+            print_err(self, c_config, "Friend not found");
         } else if (error == TOXAV_ERR_BIT_RATE_SET_FRIEND_NOT_IN_CALL) {
-            print_err(self, "Friend is not in the call");
+            print_err(self, c_config, "Friend is not in the call");
         } else {
-            print_err(self, "Unknown error");
+            print_err(self, c_config, "Unknown error");
         }
 
         return;
@@ -851,7 +912,7 @@ void cmd_bitrate(WINDOW *window, ToxWindow *self, Tox *m, int argc, char (*argv)
     return;
 }
 
-void place_call(ToxWindow *self)
+void place_call(ToxWindow *self, Toxic *toxic)
 {
     Call *call = &CallControl.calls[self->num];
 
@@ -859,39 +920,41 @@ void place_call(ToxWindow *self)
         return;
     }
 
+    const Client_Config *c_config = toxic->c_config;
+
     Toxav_Err_Call error;
 
-    toxav_call(CallControl.av, self->num, call->audio_bit_rate, call->video_bit_rate, &error);
+    toxav_call(toxic->av, self->num, call->audio_bit_rate, call->video_bit_rate, &error);
 
     if (error != TOXAV_ERR_CALL_OK) {
         if (error == TOXAV_ERR_CALL_FRIEND_ALREADY_IN_CALL) {
-            print_err(self, "Already in a call!");
+            print_err(self, c_config, "Already in a call!");
         } else if (error == TOXAV_ERR_CALL_MALLOC) {
-            print_err(self, "Memory allocation issue");
+            print_err(self, c_config, "Memory allocation issue");
         } else if (error == TOXAV_ERR_CALL_FRIEND_NOT_FOUND) {
-            print_err(self, "Friend number invalid");
+            print_err(self, c_config, "Friend number invalid");
         } else if (error == TOXAV_ERR_CALL_FRIEND_NOT_CONNECTED) {
-            print_err(self, "Friend is valid but not currently connected");
+            print_err(self, c_config, "Friend is valid but not currently connected");
         } else {
-            print_err(self, "Internal error!");
+            print_err(self, c_config, "Internal error!");
         }
 
         cancel_call(call);
         return;
     }
 
-    callback_recv_ringing(self->num);
+    callback_recv_ringing(toxic, self->num);
 }
 
-void stop_current_call(ToxWindow *self)
+void stop_current_call(ToxWindow *self, Toxic *toxic)
 {
     Call *call = &CallControl.calls[self->num];
 
     if (call->status == cs_Pending) {
-        toxav_call_control(CallControl.av, self->num, TOXAV_CALL_CONTROL_CANCEL, NULL);
+        toxav_call_control(toxic->av, self->num, TOXAV_CALL_CONTROL_CANCEL, NULL);
 
         cancel_call(call);
-        callback_call_canceled(self->num);
+        callback_call_canceled(toxic, self->num);
     } else {
 
 #ifdef VIDEO
@@ -900,7 +963,7 @@ void stop_current_call(ToxWindow *self)
 #endif /* VIDEO */
 
         stop_transmission(call, self->num);
-        callback_call_ended(self->num);
+        callback_call_ended(toxic, self->num);
     }
 }
 
