@@ -1,7 +1,7 @@
 /*  audio_call.h
  *
  *
- *  Copyright (C) 2014 Toxic All Rights Reserved.
+ *  Copyright (C) 2024 Toxic All Rights Reserved.
  *
  *  This file is part of Toxic.
  *
@@ -74,7 +74,6 @@ struct CallControl {
 #endif /* VIDEO */
 
     ToxAV *av;
-    ToxWindow *prompt;
 
     Call *calls;
     uint32_t max_calls;
@@ -95,13 +94,13 @@ struct CallControl {
 extern struct CallControl CallControl;
 
 /* You will have to pass pointer to first member of 'windows' declared in windows.c */
-ToxAV *init_audio(ToxWindow *self, Tox *tox);
-void terminate_audio(void);
+ToxAV *init_audio(Toxic *toxic);
+void terminate_audio(ToxAV *av);
 
 bool init_call(Call *call);
 
-void place_call(ToxWindow *self);
-void stop_current_call(ToxWindow *self);
+void place_call(ToxWindow *self, Toxic *toxic);
+void stop_current_call(ToxWindow *self, Toxic *toxic);
 
 void init_friend_AV(uint32_t index);
 void del_friend_AV(uint32_t index);
