@@ -1,7 +1,7 @@
 /*  autocomplete.h
  *
  *
- *  Copyright (C) 2014 Toxic All Rights Reserved.
+ *  Copyright (C) 2024 Toxic All Rights Reserved.
  *
  *  This file is part of Toxic.
  *
@@ -23,6 +23,7 @@
 #ifndef AUTOCOMPLETE_H
 #define AUTOCOMPLETE_H
 
+#include "toxic.h"
 #include "windows.h"
 
 /*
@@ -39,7 +40,7 @@
  *
  * Note: This function should not be called directly. Use complete_line() and complete_path() instead.
  */
-int complete_line(ToxWindow *self, const char **list, size_t n_items);
+int complete_line(ToxWindow *self, Toxic *toxic, const char *const *list, size_t n_items);
 
 /* Attempts to match /command "<incomplete-dir>" line to matching directories.
  * If there is only one match the line is auto-completed.
@@ -47,6 +48,6 @@ int complete_line(ToxWindow *self, const char **list, size_t n_items);
  * Returns the diff between old len and new len of ctx->line on success.
  * Returns -1 if no matches or more than one match.
  */
-int dir_match(ToxWindow *self, Tox *m, const wchar_t *line, const wchar_t *cmd);
+int dir_match(ToxWindow *self, Toxic *toxic, const wchar_t *line, const wchar_t *cmd);
 
 #endif /* AUTOCOMPLETE_H */
