@@ -1,7 +1,7 @@
 /*  bootstrap.c
  *
  *
- *  Copyright (C) 2016 Toxic All Rights Reserved.
+ *  Copyright (C) 2024 Toxic All Rights Reserved.
  *
  *  This file is part of Toxic.
  *
@@ -20,6 +20,8 @@
  *
  */
 
+#include "bootstrap.h"
+
 #include <arpa/inet.h>
 #include <limits.h>
 #include <netinet/in.h>
@@ -36,11 +38,10 @@
 #include "line_info.h"
 #include "misc_tools.h"
 #include "prompt.h"
+#include "run_options.h"
 #include "settings.h"
 #include "windows.h"
 
-extern struct user_settings *user_settings;
-
 /* URL that we get the JSON encoded nodes list from. */
 #define NODES_LIST_URL "https://nodes.tox.chat/json"
 
@@ -148,9 +149,9 @@ static bool node_is_offline(unsigned long long int last_ping)
  * This will be the case if the file is empty, has an invalid format,
  * or if the file is older than the given timeout.
  */
-static bool nodeslist_needs_update(const char *nodes_path)
+static bool nodeslist_needs_update(const char *nodes_path, int update_frequency)
 {
-    if (user_settings->nodeslist_update_freq <= 0) {
+    if (update_frequency <= 0) {
         return false;
     }
 
@@ -183,7 +184,7 @@ static bool nodeslist_needs_update(const char *nodes_path)
     pthread_mutex_unlock(&thread_data.lock);
 
     pthread_mutex_lock(&Winthread.lock);
-    bool is_timeout = timed_out(last_scan, user_settings->nodeslist_update_freq * 24 * 60 * 60);
+    bool is_timeout = timed_out(last_scan, update_frequency * 24 * 60 * 60);
     pthread_mutex_unlock(&Winthread.lock);
 
     if (is_timeout) {
@@ -198,7 +199,7 @@ static bool nodeslist_needs_update(const char *nodes_path)
  * Return 0 on success.
  * Return -1 on failure.
  */
-static int curl_fetch_nodes_JSON(struct Recv_Curl_Data *recv_data)
+static int curl_fetch_nodes_JSON(const Run_Options *run_opts, struct Recv_Curl_Data *recv_data)
 {
     CURL *c_handle = curl_easy_init();
 
@@ -254,7 +255,7 @@ static int curl_fetch_nodes_JSON(struct Recv_Curl_Data *recv_data)
         goto on_exit;
     }
 
-    int proxy_ret = set_curl_proxy(c_handle, arg_opts.proxy_address, arg_opts.proxy_port, arg_opts.proxy_type);
+    int proxy_ret = set_curl_proxy(c_handle, run_opts->proxy_address, run_opts->proxy_port, run_opts->proxy_type);
 
     if (proxy_ret != 0) {
         fprintf(stderr, "set_curl_proxy() failed with error %d\n", proxy_ret);
@@ -314,9 +315,9 @@ on_exit:
  * Return -4 if data could not be written to disk.
  * Return -5 if memory allocation fails.
  */
-static int update_DHT_nodeslist(const char *nodes_path)
+static int update_DHT_nodeslist(const Run_Options *run_opts, const char *nodes_path, int update_frequency)
 {
-    if (!nodeslist_needs_update(nodes_path)) {
+    if (!nodeslist_needs_update(nodes_path, update_frequency)) {
         return 0;
     }
 
@@ -333,7 +334,7 @@ static int update_DHT_nodeslist(const char *nodes_path)
         return -5;
     }
 
-    if (curl_fetch_nodes_JSON(recv_data) == -1) {
+    if (curl_fetch_nodes_JSON(run_opts, recv_data) == -1) {
         free(recv_data);
         fclose(fp);
         return -2;
@@ -357,12 +358,12 @@ static int update_DHT_nodeslist(const char *nodes_path)
     return 1;
 }
 
-static void get_nodeslist_path(char *buf, size_t buf_size)
+static void get_nodeslist_path(const Run_Options *run_opts, char *buf, size_t buf_size)
 {
     char *config_dir = NULL;
 
-    if (arg_opts.nodes_path[0]) {
-        snprintf(buf, buf_size, "%s", arg_opts.nodes_path);
+    if (!string_is_empty(run_opts->nodes_path)) {
+        snprintf(buf, buf_size, "%s", run_opts->nodes_path);
     } else if ((config_dir = get_user_config_dir()) != NULL) {
         snprintf(buf, buf_size, "%s%s%s", config_dir, CONFIGDIR, DEFAULT_NODES_FILENAME);
         free(config_dir);
@@ -511,12 +512,17 @@ static int extract_node(const char *line, struct Node *node)
 }
 
 /* Loads the DHT nodeslist to memory from json encoded nodes file. */
-void *load_nodeslist_thread(void *data)
+static void *load_nodeslist_thread(void *data)
 {
-    UNUSED_VAR(data);
+
+    const Toxic *toxic = (Toxic *) data;
+
+    if (toxic == NULL) {
+        goto on_exit;
+    }
 
     char nodes_path[PATH_MAX];
-    get_nodeslist_path(nodes_path, sizeof(nodes_path));
+    get_nodeslist_path(toxic->run_opts, nodes_path, sizeof(nodes_path));
 
     FILE *fp = NULL;
 
@@ -530,7 +536,9 @@ void *load_nodeslist_thread(void *data)
         goto on_exit;
     }
 
-    int update_err = update_DHT_nodeslist(nodes_path);
+    const Client_Config *c_config = toxic->c_config;
+
+    const int update_err = update_DHT_nodeslist(toxic->run_opts, nodes_path, c_config->nodeslist_update_freq);
 
     if (update_err < 0) {
         fprintf(stderr, "update_DHT_nodeslist() failed with error %d\n", update_err);
@@ -584,7 +592,7 @@ void *load_nodeslist_thread(void *data)
 on_exit:
     thread_data.active = false;
     pthread_attr_destroy(&thread_data.attr);
-    pthread_exit(0);
+    pthread_exit(NULL);
 }
 
 /* Creates a new thread that will load the DHT nodeslist to memory
@@ -598,7 +606,7 @@ on_exit:
  * Return -4 if pthread fails to set detached state.
  * Return -5 if thread creation fails.
  */
-int load_DHT_nodeslist(void)
+int load_DHT_nodeslist(const Toxic *toxic)
 {
     if (thread_data.active) {
         return -1;
@@ -618,7 +626,7 @@ int load_DHT_nodeslist(void)
 
     thread_data.active = true;
 
-    if (pthread_create(&thread_data.tid, &thread_data.attr, load_nodeslist_thread, NULL) != 0) {
+    if (pthread_create(&thread_data.tid, &thread_data.attr, load_nodeslist_thread, (void *) toxic) != 0) {
         thread_data.active = false;
         return -5;
     }
@@ -627,7 +635,7 @@ int load_DHT_nodeslist(void)
 }
 
 /* Connects to NUM_BOOTSTRAP_NODES random DHT nodes listed in the DHTnodes file. */
-static void DHT_bootstrap(Tox *m)
+static void DHT_bootstrap(Tox *tox)
 {
     pthread_mutex_lock(&thread_data.lock);
     size_t num_nodes = Nodes.count;
@@ -650,13 +658,13 @@ static void DHT_bootstrap(Tox *m)
         }
 
         Tox_Err_Bootstrap err;
-        tox_bootstrap(m, addr, node->port, (uint8_t *) node->key, &err);
+        tox_bootstrap(tox, addr, node->port, (uint8_t *) node->key, &err);
 
         if (err != TOX_ERR_BOOTSTRAP_OK) {
             fprintf(stderr, "Failed to bootstrap %s:%d\n", addr, node->port);
         }
 
-        tox_add_tcp_relay(m, addr, node->port, (uint8_t *) node->key, &err);
+        tox_add_tcp_relay(tox, addr, node->port, (uint8_t *) node->key, &err);
 
         if (err != TOX_ERR_BOOTSTRAP_OK) {
             fprintf(stderr, "Failed to add TCP relay %s:%d\n", addr, node->port);
@@ -667,13 +675,17 @@ static void DHT_bootstrap(Tox *m)
 }
 
 /* Manages connection to the Tox DHT network. */
-void do_tox_connection(Tox *m)
+void do_tox_connection(Toxic *toxic)
 {
-    static time_t last_bootstrap_time = 0;
-    bool connected = prompt_selfConnectionStatus() != TOX_CONNECTION_NONE;
+    if (toxic == NULL) {
+        return;
+    }
+
+    static time_t last_bootstrap_time = 0;  // TODO: Put this in Toxic
+    const bool connected = prompt_selfConnectionStatus(toxic) != TOX_CONNECTION_NONE;
 
     if (!connected && timed_out(last_bootstrap_time, TRY_BOOTSTRAP_INTERVAL)) {
-        DHT_bootstrap(m);
+        DHT_bootstrap(toxic->tox);
         last_bootstrap_time = get_unix_time();
     }
 }
