@@ -1,7 +1,7 @@
 /*  bootstrap.h
  *
  *
- *  Copyright (C) 2016 Toxic All Rights Reserved.
+ *  Copyright (C) 2024 Toxic All Rights Reserved.
  *
  *  This file is part of Toxic.
  *
@@ -23,8 +23,10 @@
 #ifndef BOOTSTRAP_H
 #define BOOTSTRAP_H
 
+#include "toxic.h"
+
 /* Manages connection to the Tox DHT network. */
-void do_tox_connection(Tox *m);
+void do_tox_connection(Toxic *toxic);
 
 /* Creates a new thread that will load the DHT nodeslist to memory
  * from json encoded nodes file obtained at NODES_LIST_URL. Only one
@@ -37,6 +39,6 @@ void do_tox_connection(Tox *m);
  * Return -4 if pthread fails to set detached state.
  * Return -5 if thread creation fails.
  */
-int load_DHT_nodeslist(void);
+int load_DHT_nodeslist(const Toxic *toxic);
 
 #endif /* BOOTSTRAP_H */
