@@ -1,7 +1,7 @@
 /*  chat.h
  *
  *
- *  Copyright (C) 2014 Toxic All Rights Reserved.
+ *  Copyright (C) 2024 Toxic All Rights Reserved.
  *
  *  This file is part of Toxic.
  *
@@ -28,8 +28,8 @@
 
 /* set CTRL to -1 if we don't want to send a control signal.
    set msg to NULL if we don't want to display a message */
-void chat_close_file_receiver(Tox *m, int filenum, int friendnum, int CTRL);
-void kill_chat_window(ToxWindow *self, Tox *m);
-ToxWindow *new_chat(Tox *m, int32_t friendnum);
+void chat_close_file_receiver(Tox *tox, int filenum, int friendnum, int CTRL);
+void kill_chat_window(ToxWindow *self, Toxic *tox);
+ToxWindow *new_chat(Tox *tox, uint32_t friendnum);
 
 #endif /* end of include guard: CHAT_H */
