@@ -1,7 +1,7 @@
 /*  conference.c
  *
  *
- *  Copyright (C) 2014 Toxic All Rights Reserved.
+ *  Copyright (C) 2024 Toxic All Rights Reserved.
  *
  *  This file is part of Toxic.
  *
@@ -24,6 +24,7 @@
 #define _GNU_SOURCE    /* needed for strcasestr() and wcswidth() */
 #endif
 
+#include <assert.h>
 #include <inttypes.h>
 #include <math.h>
 #include <stdlib.h>
@@ -56,7 +57,6 @@
 #include "log.h"
 #include "misc_tools.h"
 #include "notify.h"
-#include "prompt.h"
 #include "settings.h"
 #include "toxic.h"
 #include "toxic_strings.h"
@@ -68,9 +68,10 @@
 static ConferenceChat conferences[MAX_CONFERENCE_NUM];
 static int max_conference_index = 0;
 
-extern struct user_settings *user_settings;
 extern struct Winthread Winthread;
 
+static_assert(TOX_CONFERENCE_ID_SIZE == TOX_PUBLIC_KEY_SIZE, "TOX_CONFERENCE_ID_SIZE != TOX_PUBLIC_KEY_SIZE");
+
 /* Array of conference command names used for tab completion. */
 static const char *conference_cmd_list[] = {
     "/accept",
@@ -79,8 +80,10 @@ static const char *conference_cmd_list[] = {
     "/audio",
 #endif
     "/avatar",
+    "/chatid",
     "/clear",
     "/close",
+    "/color",
     "/connect",
     "/decline",
     "/exit",
@@ -139,7 +142,7 @@ void conference_set_title(ToxWindow *self, uint32_t conferencesnum, const char *
     set_window_title(self, title, length);
 }
 
-static void kill_conference_window(ToxWindow *self)
+static void kill_conference_window(ToxWindow *self, const Client_Config *c_config)
 {
     if (self == NULL) {
         return;
@@ -159,39 +162,48 @@ static void kill_conference_window(ToxWindow *self)
 
     free(self->help);
     kill_notifs(self->active_box);
-    del_window(self);
+    del_window(self, c_config);
 }
 
-static void init_conference_logging(ToxWindow *self, Tox *m, uint32_t conferencenum)
+static void init_conference_logging(ToxWindow *self, Toxic *toxic, uint32_t conferencenum)
 {
-    ChatContext *ctx = self->chatwin;
-
-    char my_id[TOX_ADDRESS_SIZE];
-    tox_self_get_address(m, (uint8_t *) my_id);
-
-    char conference_id[TOX_CONFERENCE_ID_SIZE];
-    tox_conference_get_id(m, conferencenum, (uint8_t *) conference_id);
-
-    if (log_init(ctx->log, conferences[self->num].title, my_id, conference_id, LOG_TYPE_CHAT) != 0) {
-        line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, "Warning: Log failed to initialize.");
+    if (toxic == NULL || self == NULL) {
         return;
     }
 
-    if (load_chat_history(self, ctx->log) != 0) {
-        line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, "Failed to load chat history.");
+    Tox *tox = toxic->tox;
+    const Client_Config *c_config = toxic->c_config;
+
+    ChatContext *ctx = self->chatwin;
+
+    char my_id[TOX_ADDRESS_SIZE];
+    tox_self_get_address(tox, (uint8_t *) my_id);
+
+    char conference_id[TOX_CONFERENCE_ID_SIZE];
+    tox_conference_get_id(tox, conferencenum, (uint8_t *) conference_id);
+
+    if (log_init(ctx->log, c_config, conferences[self->num].title, my_id, conference_id, LOG_TYPE_CHAT) != 0) {
+        line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "Warning: Log failed to initialize.");
+        return;
     }
 
-    if (user_settings->autolog == AUTOLOG_ON) {
+    if (load_chat_history(ctx->log, self, c_config) != 0) {
+        line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "Failed to load chat history.");
+    }
+
+    if (c_config->autolog == AUTOLOG_ON) {
         if (log_enable(ctx->log) != 0) {
-            line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, "Failed to enable chat log.");
+            line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "Failed to enable chat log.");
         }
     }
-
-    execute(ctx->history, self, m, "/log", GLOBAL_COMMAND_MODE);  // print log state to screen
 }
 
-int init_conference_win(Tox *m, uint32_t conferencenum, uint8_t type, const char *title, size_t length)
+int init_conference_win(Toxic *toxic, uint32_t conferencenum, uint8_t type, const char *title, size_t length)
 {
+    if (toxic == NULL) {
+        return -1;
+    }
+
     if (conferencenum > MAX_CONFERENCE_NUM) {
         return -1;
     }
@@ -205,7 +217,8 @@ int init_conference_win(Tox *m, uint32_t conferencenum, uint8_t type, const char
             // probably it so happens that this will (at least typically) be
             // the case, because toxic and tox maintain the indices in
             // parallel ways. But it isn't guaranteed by the API.
-            conferences[i].chatwin = add_window(m, self);
+            conferences[i].conferencenum = conferencenum;
+            conferences[i].chatwin = add_window(toxic, self);
             conferences[i].active = true;
             conferences[i].num_peers = 0;
             conferences[i].type = type;
@@ -213,15 +226,19 @@ int init_conference_win(Tox *m, uint32_t conferencenum, uint8_t type, const char
             conferences[i].audio_enabled = false;
             conferences[i].last_sent_audio = 0;
 
+            if (!tox_conference_get_id(toxic->tox, conferencenum, (uint8_t *) conferences[i].id)) {
+                fprintf(stderr, "Failed to fetch conference ID for conferencenum: %u\n", conferencenum);
+            }
+
 #ifdef AUDIO
-            conferences[i].push_to_talk_enabled = user_settings->push_to_talk;
+            conferences[i].push_to_talk_enabled = toxic->c_config->push_to_talk;
 #endif
 
             set_active_window_index(conferences[i].chatwin);
 
             conference_set_title(self, conferencenum, title, length);
 
-            init_conference_logging(self, m, conferencenum);
+            init_conference_logging(self, toxic, conferencenum);
 
             if (i == max_conference_index) {
                 ++max_conference_index;
@@ -231,7 +248,7 @@ int init_conference_win(Tox *m, uint32_t conferencenum, uint8_t type, const char
         }
     }
 
-    kill_conference_window(self);
+    kill_conference_window(self, toxic->c_config);
 
     return -1;
 }
@@ -247,7 +264,7 @@ static void free_peer(ConferencePeer *peer)
 #endif
 }
 
-void free_conference(ToxWindow *self, uint32_t conferencenum)
+void free_conference(ToxWindow *self, const Client_Config *c_config, uint32_t conferencenum)
 {
     ConferenceChat *chat = &conferences[conferencenum];
 
@@ -282,16 +299,16 @@ void free_conference(ToxWindow *self, uint32_t conferencenum)
     }
 
     max_conference_index = i;
-    kill_conference_window(self);
+    kill_conference_window(self, c_config);
 }
 
-static void delete_conference(ToxWindow *self, Tox *m, uint32_t conferencenum)
+static void delete_conference(ToxWindow *self, Toxic *toxic, uint32_t conferencenum)
 {
-    tox_conference_delete(m, conferencenum, NULL);
-    free_conference(self, conferencenum);
+    tox_conference_delete(toxic->tox, conferencenum, NULL);
+    free_conference(self, toxic->c_config, conferencenum);
 }
 
-void conference_rename_log_path(Tox *m, uint32_t conferencenum, const char *new_title)
+void conference_rename_log_path(Toxic *toxic, uint32_t conferencenum, const char *new_title)
 {
     ConferenceChat *chat = &conferences[conferencenum];
 
@@ -300,12 +317,12 @@ void conference_rename_log_path(Tox *m, uint32_t conferencenum, const char *new_
     }
 
     char myid[TOX_ADDRESS_SIZE];
-    tox_self_get_address(m, (uint8_t *) myid);
+    tox_self_get_address(toxic->tox, (uint8_t *) myid);
 
     char conference_id[TOX_CONFERENCE_ID_SIZE];
-    tox_conference_get_id(m, conferencenum, (uint8_t *) conference_id);
+    tox_conference_get_id(toxic->tox, conferencenum, (uint8_t *) conference_id);
 
-    if (rename_logfile(chat->title, new_title, myid, conference_id, chat->chatwin) != 0) {
+    if (rename_logfile(toxic->c_config, chat->title, new_title, myid, conference_id, chat->chatwin) != 0) {
         fprintf(stderr, "Failed to rename conference log to `%s`\n", new_title);
     }
 }
@@ -352,11 +369,18 @@ void redraw_conference_win(ToxWindow *self)
     wmove(self->window, y2 - CURS_Y_OFFSET, 0);
 }
 
-static void conference_onConferenceMessage(ToxWindow *self, Tox *m, uint32_t conferencenum, uint32_t peernum,
+static void conference_onConferenceMessage(ToxWindow *self, Toxic *toxic, uint32_t conferencenum, uint32_t peernum,
         Tox_Message_Type type, const char *msg, size_t len)
 {
     UNUSED_VAR(len);
 
+    if (toxic == NULL || self == NULL) {
+        return;
+    }
+
+    Tox *tox = toxic->tox;
+    const Client_Config *c_config = toxic->c_config;
+
     if (self->num != conferencenum) {
         return;
     }
@@ -364,12 +388,12 @@ static void conference_onConferenceMessage(ToxWindow *self, Tox *m, uint32_t con
     ChatContext *ctx = self->chatwin;
 
     char nick[TOX_MAX_NAME_LENGTH];
-    get_conference_nick_truncate(m, nick, peernum, conferencenum);
+    get_conference_nick_truncate(tox, nick, peernum, conferencenum);
 
     char selfnick[TOX_MAX_NAME_LENGTH];
-    tox_self_get_name(m, (uint8_t *) selfnick);
+    tox_self_get_name(tox, (uint8_t *) selfnick);
 
-    size_t sn_len = tox_self_get_name_size(m);
+    size_t sn_len = tox_self_get_name_size(tox);
     selfnick[sn_len] = '\0';
 
     int nick_clr = strcmp(nick, selfnick) == 0 ? GREEN : CYAN;
@@ -377,26 +401,34 @@ static void conference_onConferenceMessage(ToxWindow *self, Tox *m, uint32_t con
     /* Only play sound if mentioned by someone else */
     if (strcasestr(msg, selfnick) && strcmp(selfnick, nick)) {
         if (self->active_box != -1) {
-            box_notify2(self, generic_message, NT_WNDALERT_0 | NT_NOFOCUS | user_settings->bell_on_message,
+            box_notify2(self, c_config, generic_message, NT_WNDALERT_0 | NT_NOFOCUS | c_config->bell_on_message,
                         self->active_box, "%s %s", nick, msg);
         } else {
-            box_notify(self, generic_message, NT_WNDALERT_0 | NT_NOFOCUS | user_settings->bell_on_message,
+            box_notify(self, c_config, generic_message, NT_WNDALERT_0 | NT_NOFOCUS | c_config->bell_on_message,
                        &self->active_box, self->name, "%s %s", nick, msg);
         }
 
         nick_clr = RED;
     } else {
-        sound_notify(self, silent, NT_WNDALERT_1, NULL);
+        sound_notify(self, c_config, silent, NT_WNDALERT_1, NULL);
     }
 
-    line_info_add(self, true, nick, NULL, type == TOX_MESSAGE_TYPE_NORMAL ? IN_MSG : IN_ACTION, 0, nick_clr, "%s", msg);
-    write_to_log(msg, nick, ctx->log, false);
+    line_info_add(self, c_config, true, nick, NULL, type == TOX_MESSAGE_TYPE_NORMAL ? IN_MSG : IN_ACTION, 0,
+                  nick_clr, "%s", msg);
+    write_to_log(ctx->log, c_config, msg, nick, false);
 }
 
-static void conference_onConferenceTitleChange(ToxWindow *self, Tox *m, uint32_t conferencenum, uint32_t peernum,
+static void conference_onConferenceTitleChange(ToxWindow *self, Toxic *toxic, uint32_t conferencenum, uint32_t peernum,
         const char *title,
         size_t length)
 {
+    if (toxic == NULL || self == NULL) {
+        return;
+    }
+
+    Tox *tox = toxic->tox;
+    const Client_Config *c_config = toxic->c_config;
+
     ChatContext *ctx = self->chatwin;
 
     if (self->num != conferencenum) {
@@ -409,7 +441,7 @@ static void conference_onConferenceTitleChange(ToxWindow *self, Tox *m, uint32_t
         return;
     }
 
-    conference_rename_log_path(m, conferencenum, title);  // must be called first
+    conference_rename_log_path(toxic, conferencenum, title);  // must be called first
 
     conference_set_title(self, conferencenum, title, length);
 
@@ -419,12 +451,12 @@ static void conference_onConferenceTitleChange(ToxWindow *self, Tox *m, uint32_t
     }
 
     char nick[TOX_MAX_NAME_LENGTH];
-    get_conference_nick_truncate(m, nick, peernum, conferencenum);
-    line_info_add(self, true, nick, NULL, NAME_CHANGE, 0, 0, " set the conference title to: %s", title);
+    get_conference_nick_truncate(tox, nick, peernum, conferencenum);
+    line_info_add(self, c_config, true, nick, NULL, NAME_CHANGE, 0, 0, " set the conference title to: %s", title);
 
     char tmp_event[MAX_STR_SIZE];
     snprintf(tmp_event, sizeof(tmp_event), "set title to %s", title);
-    write_to_log(tmp_event, nick, ctx->log, true);
+    write_to_log(ctx->log, c_config, tmp_event, nick, true);
 }
 
 /* Puts `(NameListEntry *)`s in `entries` for each matched peer, up to a
@@ -596,7 +628,7 @@ static void conference_enable_push_to_talk(ConferenceChat *chat)
     chat->ptt_last_pushed = get_unix_time();
 }
 
-static void set_peer_audio_position(Tox *m, uint32_t conferencenum, uint32_t peernum)
+static void set_peer_audio_position(Tox *tox, uint32_t conferencenum, uint32_t peernum)
 {
     ConferenceChat *chat = &conferences[conferencenum];
     ConferencePeer *peer = &chat->peer_list[peernum];
@@ -611,7 +643,7 @@ static void set_peer_audio_position(Tox *m, uint32_t conferencenum, uint32_t pee
     uint32_t peer_posn = peernum;
 
     for (uint32_t i = 0; i < chat->num_peers; ++i) {
-        if (tox_conference_peer_number_is_ours(m, conferencenum, peernum, NULL)) {
+        if (tox_conference_peer_number_is_ours(tox, conferencenum, peernum, NULL)) {
             if (i == peernum) {
                 return;
             }
@@ -647,7 +679,7 @@ static bool find_peer_by_pubkey(const ConferencePeer *list, uint32_t num_peers,
     return false;
 }
 
-static void update_peer_list(ToxWindow *self, Tox *m, uint32_t conferencenum, uint32_t num_peers,
+static void update_peer_list(ToxWindow *self, Toxic *toxic, uint32_t conferencenum, uint32_t num_peers,
                              uint32_t old_num_peers)
 {
     ConferenceChat *chat = &conferences[conferencenum];
@@ -656,6 +688,9 @@ static void update_peer_list(ToxWindow *self, Tox *m, uint32_t conferencenum, ui
         return;
     }
 
+    Tox *tox = toxic->tox;
+    const Client_Config *c_config = toxic->c_config;
+
     ChatContext *ctx = self->chatwin;
 
     ConferencePeer *old_peer_list = malloc(old_num_peers * sizeof(ConferencePeer));
@@ -683,7 +718,7 @@ static void update_peer_list(ToxWindow *self, Tox *m, uint32_t conferencenum, ui
         };
 
         Tox_Err_Conference_Peer_Query err;
-        tox_conference_peer_get_public_key(m, conferencenum, i, peer->pubkey, &err);
+        tox_conference_peer_get_public_key(tox, conferencenum, i, peer->pubkey, &err);
 
         if (err != TOX_ERR_CONFERENCE_PEER_QUERY_OK) {
             continue;
@@ -699,14 +734,14 @@ static void update_peer_list(ToxWindow *self, Tox *m, uint32_t conferencenum, ui
             new_peer = false;
         }
 
-        size_t length = tox_conference_peer_get_name_size(m, conferencenum, i, &err);
+        size_t length = tox_conference_peer_get_name_size(tox, conferencenum, i, &err);
 
         if (err != TOX_ERR_CONFERENCE_PEER_QUERY_OK || length >= TOX_MAX_NAME_LENGTH) {
             // FIXME: length == TOX_MAX_NAME_LENGTH should not be an error!
             continue;
         }
 
-        tox_conference_peer_get_name(m, conferencenum, i, (uint8_t *) peer->name, &err);
+        tox_conference_peer_get_name(tox, conferencenum, i, (uint8_t *) peer->name, &err);
         peer->name[length] = 0;
 
         if (err != TOX_ERR_CONFERENCE_PEER_QUERY_OK) {
@@ -719,12 +754,12 @@ static void update_peer_list(ToxWindow *self, Tox *m, uint32_t conferencenum, ui
 
         if (new_peer && peer->name_length > 0 && timed_out(chat->start_time, CONFERENCE_EVENT_WAIT)) {
             const char *msg = "has joined the conference";
-            line_info_add(self, true, peer->name, NULL, CONNECTION, 0, GREEN, msg);
-            write_to_log(msg, peer->name, ctx->log, true);
+            line_info_add(self, c_config, true, peer->name, NULL, CONNECTION, 0, GREEN, msg);
+            write_to_log(ctx->log, c_config, msg, peer->name, true);
         }
 
 #ifdef AUDIO
-        set_peer_audio_position(m, conferencenum, i);
+        set_peer_audio_position(tox, conferencenum, i);
 #endif
     }
 
@@ -736,8 +771,8 @@ static void update_peer_list(ToxWindow *self, Tox *m, uint32_t conferencenum, ui
         if (old_peer->active) {
             if (old_peer->name_length > 0 && !find_peer_by_pubkey(chat->peer_list, chat->num_peers, old_peer->pubkey, NULL)) {
                 const char *msg = "has left the conference";
-                line_info_add(self, true, old_peer->name, NULL, DISCONNECTION, 0, RED, msg);
-                write_to_log(msg, old_peer->name, ctx->log, true);
+                line_info_add(self, c_config, true, old_peer->name, NULL, DISCONNECTION, 0, RED, msg);
+                write_to_log(ctx->log, c_config, msg, old_peer->name, true);
             }
 
             free_peer(old_peer);
@@ -747,8 +782,14 @@ static void update_peer_list(ToxWindow *self, Tox *m, uint32_t conferencenum, ui
     free(old_peer_list);
 }
 
-static void conference_onConferenceNameListChange(ToxWindow *self, Tox *m, uint32_t conferencenum)
+static void conference_onConferenceNameListChange(ToxWindow *self, Toxic *toxic, uint32_t conferencenum)
 {
+    if (toxic == NULL || self == NULL) {
+        return;
+    }
+
+    Tox *tox = toxic->tox;
+
     if (self->num != conferencenum) {
         return;
     }
@@ -765,7 +806,7 @@ static void conference_onConferenceNameListChange(ToxWindow *self, Tox *m, uint3
 
     Tox_Err_Conference_Peer_Query err;
 
-    uint32_t num_peers = tox_conference_peer_count(m, conferencenum, &err);
+    const uint32_t num_peers = tox_conference_peer_count(tox, conferencenum, &err);
 
     if (err != TOX_ERR_CONFERENCE_PEER_QUERY_OK) {
         fprintf(stderr, "conference_onConferenceNameListChange() failed with error: %d\n", err);
@@ -776,14 +817,21 @@ static void conference_onConferenceNameListChange(ToxWindow *self, Tox *m, uint3
 
     chat->num_peers = num_peers;
     chat->max_idx = num_peers;
-    update_peer_list(self, m, conferencenum, num_peers, old_num);
+    update_peer_list(self, toxic, conferencenum, num_peers, old_num);
 }
 
-static void conference_onConferencePeerNameChange(ToxWindow *self, Tox *m, uint32_t conferencenum, uint32_t peernum,
+static void conference_onConferencePeerNameChange(ToxWindow *self, Toxic *toxic, uint32_t conferencenum,
+        uint32_t peernum,
         const char *name, size_t length)
 {
     UNUSED_VAR(length);
 
+    if (toxic == NULL || self == NULL) {
+        return;
+    }
+
+    const Client_Config *c_config = toxic->c_config;
+
     if (self->num != conferencenum) {
         return;
     }
@@ -795,23 +843,23 @@ static void conference_onConferencePeerNameChange(ToxWindow *self, Tox *m, uint3
 
         if (peer->name_length > 0) {
             char log_event[TOXIC_MAX_NAME_LENGTH * 2 + 32];
-            line_info_add(self, true, peer->name, (const char *) name, NAME_CHANGE, 0, 0, " is now known as ");
+            line_info_add(self, c_config, true, peer->name, (const char *) name, NAME_CHANGE, 0, 0, " is now known as ");
 
             snprintf(log_event, sizeof(log_event), "is now known as %s", (const char *) name);
-            write_to_log(log_event, peer->name, ctx->log, true);
+            write_to_log(ctx->log, c_config, log_event, peer->name, true);
 
             // this is kind of a hack; peers always join a group with no name set and then set it after
         } else if (timed_out(conferences[conferencenum].start_time, CONFERENCE_EVENT_WAIT)) {
             const char *msg = "has joined the conference";
-            line_info_add(self, true, name, NULL, CONNECTION, 0, GREEN, msg);
-            write_to_log(msg, name, ctx->log, true);
+            line_info_add(self, c_config, true, name, NULL, CONNECTION, 0, GREEN, msg);
+            write_to_log(ctx->log, c_config, msg, name, true);
         }
     }
 
-    conference_onConferenceNameListChange(self, m, conferencenum);
+    conference_onConferenceNameListChange(self, toxic, conferencenum);
 }
 
-static void send_conference_action(ToxWindow *self, ChatContext *ctx, Tox *m, char *action)
+static void send_conference_action(ToxWindow *self, ChatContext *ctx, Toxic *toxic, char *action)
 {
     if (action == NULL) {
         wprintw(ctx->history, "Invalid syntax.\n");
@@ -820,8 +868,10 @@ static void send_conference_action(ToxWindow *self, ChatContext *ctx, Tox *m, ch
 
     Tox_Err_Conference_Send_Message err;
 
-    if (!tox_conference_send_message(m, self->num, TOX_MESSAGE_TYPE_ACTION, (uint8_t *) action, strlen(action), &err)) {
-        line_info_add(self, false, NULL, NULL, SYS_MSG, 0, RED, " * Failed to send action (error %d)", err);
+    if (!tox_conference_send_message(toxic->tox, self->num, TOX_MESSAGE_TYPE_ACTION, (uint8_t *) action,
+                                     strlen(action), &err)) {
+        line_info_add(self, toxic->c_config, false, NULL, NULL, SYS_MSG, 0, RED,
+                      " * Failed to send action (error %d)", err);
     }
 }
 
@@ -834,8 +884,15 @@ static int sidebar_offset(uint32_t conferencenum)
 /*
  * Return true if input is recognized by handler
  */
-static bool conference_onKey(ToxWindow *self, Tox *m, wint_t key, bool ltr)
+static bool conference_onKey(ToxWindow *self, Toxic *toxic, wint_t key, bool ltr)
 {
+    if (toxic == NULL || self == NULL) {
+        return false;
+    }
+
+    Tox *tox = toxic->tox;
+    const Client_Config *c_config = toxic->c_config;
+
     ChatContext *ctx = self->chatwin;
 
     int x, y, y2, x2;
@@ -858,15 +915,15 @@ static bool conference_onKey(ToxWindow *self, Tox *m, wint_t key, bool ltr)
     }
 
     if (ltr || key == L'\n') {    /* char is printable */
-        input_new_char(self, key, x, x2);
+        input_new_char(self, c_config, key, x, x2);
         return true;
     }
 
-    if (line_info_onKey(self, key)) {
+    if (line_info_onKey(self, c_config, key)) {
         return true;
     }
 
-    if (input_handle(self, key, x, x2)) {
+    if (input_handle(self, c_config, key, x, x2)) {
         return true;
     }
 
@@ -897,16 +954,16 @@ static bool conference_onKey(ToxWindow *self, Tox *m, wint_t key, bool ltr)
                         complete_strs[i] = (const char *) chat->name_list[i].name;
                     }
 
-                    diff = complete_line(self, complete_strs, chat->num_peers);
+                    diff = complete_line(self, toxic, complete_strs, chat->num_peers);
                     free(complete_strs);
                 }
             } else if (wcsncmp(ctx->line, L"/avatar ", wcslen(L"/avatar ")) == 0) {
-                diff = dir_match(self, m, ctx->line, L"/avatar");
+                diff = dir_match(self, toxic, ctx->line, L"/avatar");
             }
 
 #ifdef PYTHON
             else if (wcsncmp(ctx->line, L"/run ", wcslen(L"/run ")) == 0) {
-                diff = dir_match(self, m, ctx->line, L"/run");
+                diff = dir_match(self, toxic, ctx->line, L"/run");
             }
 
 #endif
@@ -918,21 +975,21 @@ static bool conference_onKey(ToxWindow *self, Tox *m, wint_t key, bool ltr)
                         complete_strs[i] = (const char *) chat->name_list[i].name;
                     }
 
-                    diff = complete_line(self, complete_strs, chat->num_peers);
+                    diff = complete_line(self, toxic, complete_strs, chat->num_peers);
 
                     if (diff == -1) {
                         for (uint32_t i = 0; i < chat->num_peers; ++i) {
                             complete_strs[i] = (const char *) chat->name_list[i].pubkey_str;
                         }
 
-                        diff = complete_line(self, complete_strs, chat->num_peers);
+                        diff = complete_line(self, toxic, complete_strs, chat->num_peers);
                     }
 
                     free(complete_strs);
                 }
 
             } else {
-                diff = complete_line(self, conference_cmd_list, sizeof(conference_cmd_list) / sizeof(char *));
+                diff = complete_line(self, toxic, conference_cmd_list, sizeof(conference_cmd_list) / sizeof(char *));
             }
 
             if (diff != -1) {
@@ -941,10 +998,10 @@ static bool conference_onKey(ToxWindow *self, Tox *m, wint_t key, bool ltr)
                     ctx->start = wlen < x2 ? 0 : wlen - x2 + 1;
                 }
             } else {
-                sound_notify(self, notif_error, 0, NULL);
+                sound_notify(self, c_config, notif_error, 0, NULL);
             }
         } else {
-            sound_notify(self, notif_error, 0, NULL);
+            sound_notify(self, c_config, notif_error, 0, NULL);
         }
     } else if (key == T_KEY_C_DOWN) {    /* Scroll peerlist up and down one position */
         input_ret = true;
@@ -976,21 +1033,21 @@ static bool conference_onKey(ToxWindow *self, Tox *m, wint_t key, bool ltr)
 
             if (line[0] == '/') {
                 if (strcmp(line, "/close") == 0) {
-                    delete_conference(self, m, self->num);
+                    delete_conference(self, toxic, self->num);
                     return true;
                 } else if (strncmp(line, "/me ", strlen("/me ")) == 0) {
-                    send_conference_action(self, ctx, m, line + strlen("/me "));
+                    send_conference_action(self, ctx, toxic, line + strlen("/me "));
                 } else {
-                    execute(ctx->history, self, m, line, CONFERENCE_COMMAND_MODE);
+                    execute(ctx->history, self, toxic, line, CONFERENCE_COMMAND_MODE);
                 }
             } else if (line[0]) {
                 Tox_Err_Conference_Send_Message err;
 
-                if (!tox_conference_send_message(m, self->num, TOX_MESSAGE_TYPE_NORMAL, (uint8_t *) line, strlen(line), &err)) {
-                    line_info_add(self, false, NULL, NULL, SYS_MSG, 0, RED, " * Failed to send message (error %d)", err);
+                if (!tox_conference_send_message(tox, self->num, TOX_MESSAGE_TYPE_NORMAL, (uint8_t *) line, strlen(line), &err)) {
+                    line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, RED, " * Failed to send message (error %d)", err);
                 }
             } else {
-                line_info_add(self, false, NULL, NULL, SYS_MSG, 0, RED, " * Failed to parse message.");
+                line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, RED, " * Failed to parse message.");
             }
         }
 
@@ -1002,12 +1059,12 @@ static bool conference_onKey(ToxWindow *self, Tox *m, wint_t key, bool ltr)
     return input_ret;
 }
 
-static void draw_peer(ToxWindow *self, Tox *m, ChatContext *ctx, uint32_t i)
+static void draw_peer(ToxWindow *self, Toxic *toxic, ChatContext *ctx, uint32_t i)
 {
     pthread_mutex_lock(&Winthread.lock);
     const uint32_t peer_idx = i + conferences[self->num].side_pos;
     const uint32_t peernum = conferences[self->num].name_list[peer_idx].peernum;
-    const bool is_self = tox_conference_peer_number_is_ours(m, self->num, peernum, NULL);
+    const bool is_self = tox_conference_peer_number_is_ours(toxic->tox, self->num, peernum, NULL);
     const bool audio = conferences[self->num].audio_enabled;
 
     if (audio) {
@@ -1034,7 +1091,7 @@ static void draw_peer(ToxWindow *self, Tox *m, ChatContext *ctx, uint32_t i)
 
     /* truncate nick to fit in side panel without modifying list */
     char tmpnick[TOX_MAX_NAME_LENGTH];
-    int maxlen = SIDEBAR_WIDTH - 2 - 2 * audio;
+    const int maxlen = SIDEBAR_WIDTH - 2 - 2 * audio;
 
     pthread_mutex_lock(&Winthread.lock);
     memcpy(tmpnick, &conferences[self->num].name_list[peer_idx].name, maxlen);
@@ -1053,9 +1110,12 @@ static void draw_peer(ToxWindow *self, Tox *m, ChatContext *ctx, uint32_t i)
     }
 }
 
-static void conference_onDraw(ToxWindow *self, Tox *m)
+static void conference_onDraw(ToxWindow *self, Toxic *toxic)
 {
-    UNUSED_VAR(m);
+    if (toxic == NULL || self == NULL) {
+        fprintf(stderr, "conference_onDraw null param\n");
+        return;
+    }
 
     int x2, y2;
     getmaxyx(self->window, y2, x2);
@@ -1073,7 +1133,7 @@ static void conference_onDraw(ToxWindow *self, Tox *m)
     ChatContext *ctx = self->chatwin;
 
     pthread_mutex_lock(&Winthread.lock);
-    line_info_print(self);
+    line_info_print(self, toxic->c_config);
     pthread_mutex_unlock(&Winthread.lock);
 
     wclear(ctx->linewin);
@@ -1087,10 +1147,10 @@ static void conference_onDraw(ToxWindow *self, Tox *m)
     wclear(ctx->sidebar);
 
     if (self->show_peerlist) {
-        wattron(ctx->sidebar, COLOR_PAIR(BLUE));
+        wattron(ctx->sidebar, COLOR_PAIR(PEERLIST_LINE));
         mvwvline(ctx->sidebar, 0, 0, ACS_VLINE, y2 - CHATBOX_HEIGHT);
         mvwaddch(ctx->sidebar, y2 - CHATBOX_HEIGHT, 0, ACS_BTEE);
-        wattroff(ctx->sidebar, COLOR_PAIR(BLUE));
+        wattroff(ctx->sidebar, COLOR_PAIR(PEERLIST_LINE));
 
         pthread_mutex_lock(&Winthread.lock);
         const uint32_t num_peers = chat->num_peers;
@@ -1150,17 +1210,17 @@ static void conference_onDraw(ToxWindow *self, Tox *m)
         wattroff(ctx->sidebar, A_BOLD);
         ++line;
 
-        wattron(ctx->sidebar, COLOR_PAIR(BLUE));
+        wattron(ctx->sidebar, COLOR_PAIR(PEERLIST_LINE));
         mvwaddch(ctx->sidebar, line, 0, ACS_LTEE);
         mvwhline(ctx->sidebar, line, 1, ACS_HLINE, SIDEBAR_WIDTH - 1);
-        wattroff(ctx->sidebar, COLOR_PAIR(BLUE));
+        wattroff(ctx->sidebar, COLOR_PAIR(PEERLIST_LINE));
         ++line;
 
         for (uint32_t i = 0;
                 i < num_peers && i < y2 - header_lines - CHATBOX_HEIGHT;
                 ++i) {
             wmove(ctx->sidebar, i + header_lines, 1);
-            draw_peer(self, m, ctx, i);
+            draw_peer(self, toxic, ctx, i);
         }
     }
 
@@ -1177,12 +1237,16 @@ static void conference_onDraw(ToxWindow *self, Tox *m)
     wnoutrefresh(self->window);
 
     if (self->help->active) {
-        help_onDraw(self);
+        help_draw_main(self);
     }
 }
 
-static void conference_onInit(ToxWindow *self, Tox *m)
+static void conference_onInit(ToxWindow *self, Toxic *toxic)
 {
+    if (toxic == NULL || self == NULL) {
+        return;
+    }
+
     int x2, y2;
     getmaxyx(self->window, y2, x2);
 
@@ -1210,6 +1274,83 @@ static void conference_onInit(ToxWindow *self, Tox *m)
     wmove(self->window, y2 - CURS_Y_OFFSET, 0);
 }
 
+/*
+ * Return the conference number associated with `public_key`.
+ * Return -1 if public_key does not designate a valid conference.
+ *
+ * `public_key` must be a string of at least TOX_PUBLIC_KEY_SIZE * 2 chars in length.
+ */
+static int get_conferencenum_by_public_key_string(const char *public_key)
+{
+    char pk_bin[TOX_PUBLIC_KEY_SIZE];
+
+    if (tox_pk_string_to_bytes(public_key, strlen(public_key), pk_bin, sizeof(pk_bin)) != 0) {
+        return -1;
+    }
+
+    for (size_t i = 0; i < max_conference_index; ++i) {
+        const ConferenceChat *chat = &conferences[i];
+
+        if (!chat->active) {
+            continue;
+        }
+
+        if (memcmp(pk_bin, chat->id, TOX_PUBLIC_KEY_SIZE) == 0) {
+            return chat->conferencenum;
+        }
+    }
+
+    return -1;
+}
+
+/*
+ * Sets the tab name colour of the ToxWindow associated with `public_key` to `colour`.
+ *
+ * Return false if conference does not exist.
+ */
+static bool conference_window_set_tab_name_colour(const char *public_key, int colour)
+{
+    const int conferencenum = get_conferencenum_by_public_key_string(public_key);
+
+    if (conferencenum < 0) {
+        return false;
+    }
+
+    ToxWindow *self = get_window_by_number_type(conferencenum, WINDOW_TYPE_CONFERENCE);
+
+    if (self == NULL) {
+        return false;
+    }
+
+    self->colour = colour;
+
+    return true;
+}
+
+bool conference_config_set_tab_name_colour(const char *public_key, const char *colour)
+{
+    const int colour_val = colour_string_to_int(colour);
+
+    if (colour_val < 0) {
+        return false;
+    }
+
+    return conference_window_set_tab_name_colour(public_key, colour_val);
+}
+
+bool conference_config_set_autolog(const char *public_key, bool autolog_enabled)
+{
+    const int conferencenum = get_conferencenum_by_public_key_string(public_key);
+
+    if (conferencenum < 0) {
+        return false;
+    }
+
+    return autolog_enabled
+           ? enable_window_log_by_number_type(conferencenum, WINDOW_TYPE_CONFERENCE)
+           : disable_window_log_by_number_type(conferencenum, WINDOW_TYPE_CONFERENCE);
+}
+
 static ToxWindow *new_conference_chat(uint32_t conferencenum)
 {
     ToxWindow *ret = calloc(1, sizeof(ToxWindow));
@@ -1256,6 +1397,12 @@ static ToxWindow *new_conference_chat(uint32_t conferencenum)
 void audio_conference_callback(void *tox, uint32_t conferencenum, uint32_t peernum, const int16_t *pcm,
                                unsigned int samples, uint8_t channels, uint32_t sample_rate, void *userdata)
 {
+    const Client_Config *c_config = (Client_Config *) userdata;
+
+    if (c_config == NULL) {
+        return;
+    }
+
     ConferencePeer *peer = peer_in_conference(conferencenum, peernum);
 
     if (peer == NULL) {
@@ -1264,7 +1411,7 @@ void audio_conference_callback(void *tox, uint32_t conferencenum, uint32_t peern
 
     if (!peer->sending_audio) {
         if (open_output_device(&peer->audio_out_idx,
-                               sample_rate, CONFAV_FRAME_DURATION, channels) != de_None) {
+                               sample_rate, CONFAV_FRAME_DURATION, channels, c_config->VAD_threshold) != de_None) {
             // TODO: error message?
             return;
         }
@@ -1295,15 +1442,14 @@ static void conference_read_device_callback(const int16_t *captured, uint32_t si
 
     chat->last_sent_audio = get_unix_time();
 
-    int channels = user_settings->conference_audio_channels;
-
     toxav_group_send_audio(audio_input_callback_data->tox,
                            audio_input_callback_data->conferencenum,
                            captured, CONFAV_SAMPLES_PER_FRAME,
-                           channels, CONFAV_SAMPLE_RATE);
+                           audio_input_callback_data->audio_channels,
+                           CONFAV_SAMPLE_RATE);
 }
 
-bool init_conference_audio_input(Tox *tox, uint32_t conferencenum)
+bool init_conference_audio_input(Toxic *toxic, uint32_t conferencenum)
 {
     ConferenceChat *chat = &conferences[conferencenum];
 
@@ -1311,15 +1457,24 @@ bool init_conference_audio_input(Tox *tox, uint32_t conferencenum)
         return false;
     }
 
-    const AudioInputCallbackData audio_input_callback_data = { tox, conferencenum };
+    const Client_Config *c_config = toxic->c_config;
+
+    const int channels = c_config->conference_audio_channels;
+
+    const AudioInputCallbackData audio_input_callback_data = {
+        toxic->tox,
+        conferencenum,
+        channels,
+    };
+
     chat->audio_input_callback_data = audio_input_callback_data;
 
-    int channels = user_settings->conference_audio_channels;
 
-    bool success = (open_input_device(&chat->audio_in_idx,
-                                      conference_read_device_callback, &chat->audio_input_callback_data,
-                                      CONFAV_SAMPLE_RATE, CONFAV_FRAME_DURATION, channels)
-                    == de_None);
+    const bool success = (open_input_device(&chat->audio_in_idx,
+                                            conference_read_device_callback, &chat->audio_input_callback_data,
+                                            CONFAV_SAMPLE_RATE, CONFAV_FRAME_DURATION, channels,
+                                            c_config->VAD_threshold)
+                          == de_None);
 
     chat->audio_enabled = success;
 
@@ -1339,10 +1494,11 @@ bool toggle_conference_push_to_talk(uint32_t conferencenum, bool enabled)
     return true;
 }
 
-bool enable_conference_audio(ToxWindow *self, Tox *tox, uint32_t conferencenum)
+bool enable_conference_audio(ToxWindow *self, Toxic *toxic, uint32_t conferencenum)
 {
-    if (!toxav_groupchat_av_enabled(tox, conferencenum)) {
-        if (toxav_groupchat_enable_av(tox, conferencenum, audio_conference_callback, NULL) != 0) {
+    if (!toxav_groupchat_av_enabled(toxic->tox, conferencenum)) {
+        if (toxav_groupchat_enable_av(toxic->tox, conferencenum, audio_conference_callback,
+                                      (void *) toxic->c_config) != 0) {
             return false;
         }
     }
@@ -1353,7 +1509,7 @@ bool enable_conference_audio(ToxWindow *self, Tox *tox, uint32_t conferencenum)
         return true;
     }
 
-    const bool success = init_conference_audio_input(tox, conferencenum);
+    const bool success = init_conference_audio_input(toxic, conferencenum);
 
     if (success) {
         self->is_call = true;
@@ -1362,7 +1518,7 @@ bool enable_conference_audio(ToxWindow *self, Tox *tox, uint32_t conferencenum)
     return success;
 }
 
-bool disable_conference_audio(ToxWindow *self, Tox *tox, uint32_t conferencenum)
+bool disable_conference_audio(ToxWindow *self, Toxic *toxic, uint32_t conferencenum)
 {
     ConferenceChat *chat = &conferences[conferencenum];
 
@@ -1377,7 +1533,7 @@ bool disable_conference_audio(ToxWindow *self, Tox *tox, uint32_t conferencenum)
         return true;
     }
 
-    const bool success = toxav_groupchat_disable_av(tox, conferencenum) == 0;
+    const bool success = toxav_groupchat_disable_av(toxic->tox, conferencenum) == 0;
 
     if (success) {
         self->is_call = false;
@@ -1399,9 +1555,9 @@ bool conference_mute_self(uint32_t conferencenum)
     return true;
 }
 
-bool conference_mute_peer(const Tox *m, uint32_t conferencenum, uint32_t peernum)
+bool conference_mute_peer(const Tox *tox, uint32_t conferencenum, uint32_t peernum)
 {
-    if (tox_conference_peer_number_is_ours(m, conferencenum, peernum, NULL)) {
+    if (tox_conference_peer_number_is_ours(tox, conferencenum, peernum, NULL)) {
         return conference_mute_self(conferencenum);
     }
 
