@@ -1,7 +1,7 @@
 /*  file_transfers.h
  *
  *
- *  Copyright (C) 2014 Toxic All Rights Reserved.
+ *  Copyright (C) 2024 Toxic All Rights Reserved.
  *
  *  This file is part of Toxic.
  *
@@ -24,6 +24,7 @@
 #define FILE_TRANSFERS_H
 
 #include <limits.h>
+#include <time.h>
 
 #include "notify.h"
 #include "toxic.h"
@@ -120,7 +121,7 @@ struct FileTransfer *new_file_transfer(ToxWindow *window, uint32_t friendnumber,
 int file_send_queue_add(uint32_t friendnumber, const char *file_path, size_t length);
 
 /* Initiates all file transfers from the file send queue for friend designated by `friendnumber`. */
-void file_send_queue_check(ToxWindow *self, Tox *m, uint32_t friendnumber);
+void file_send_queue_check(ToxWindow *self, Toxic *toxic, uint32_t friendnumber);
 
 /* Removes the `index`-th item from the file send queue for `friendnumber`.
  *
@@ -134,16 +135,16 @@ int file_send_queue_remove(uint32_t friendnumber, size_t index);
  * Set CTRL to -1 if we don't want to send a control signal.
  * Set message or self to NULL if we don't want to display a message.
  */
-void close_file_transfer(ToxWindow *self, Tox *m, struct FileTransfer *ft, int CTRL, const char *message,
+void close_file_transfer(ToxWindow *self, const Toxic *toxic, struct FileTransfer *ft, int CTRL, const char *message,
                          Notification sound_type);
 
 /* Kills active outgoing avatar file transfers for friendnumber */
-void kill_avatar_file_transfers_friend(Tox *m, uint32_t friendnumber);
+void kill_avatar_file_transfers_friend(Toxic *toxic, uint32_t friendnumber);
 
 /* Kills all active file transfers for friendnumber */
-void kill_all_file_transfers_friend(Tox *m, uint32_t friendnumber);
+void kill_all_file_transfers_friend(Toxic *toxic, uint32_t friendnumber);
 
-void kill_all_file_transfers(Tox *m);
+void kill_all_file_transfers(Toxic *toxic);
 
 /* Return true if any pending or active file receiver has the path `path`. */
 bool file_transfer_recv_path_exists(const char *path);
