@@ -1,7 +1,7 @@
 /*  friendlist.c
  *
  *
- *  Copyright (C) 2014 Toxic All Rights Reserved.
+ *  Copyright (C) 2024 Toxic All Rights Reserved.
  *
  *  This file is part of Toxic.
  *
@@ -64,6 +64,25 @@ static struct PendingDel {
     WINDOW *popup;
 } PendingDelete;
 
+typedef enum Default_Conf {
+    Default_Conf_Auto_Accept_Files = 0,
+    Default_Conf_Tab_Name_Colour = WHITE_BAR_FG,
+} Default_Conf;
+
+static void set_default_friend_config_settings(ToxicFriend *friend, const Client_Config *c_config)
+{
+    if (friend == NULL) {
+        return;
+    }
+
+    Friend_Settings *settings = &friend->settings;
+
+    settings->auto_accept_files = Default_Conf_Auto_Accept_Files != 0;
+    settings->autolog = c_config->autolog == AUTOLOG_ON;
+    settings->show_connection_msg = c_config->show_connection_msg == SHOW_CONNECTION_MSG_ON;
+    settings->tab_name_colour = Default_Conf_Tab_Name_Colour;
+}
+
 static void realloc_friends(int n)
 {
     if (n <= 0) {
@@ -106,7 +125,7 @@ static void realloc_blocklist(int n)
     Blocked.index = b_idx;
 }
 
-void kill_friendlist(ToxWindow *self)
+void kill_friendlist(ToxWindow *self, const Client_Config *c_config)
 {
     for (size_t i = 0; i < Friends.max_idx; ++i) {
         if (Friends.list[i].active) {
@@ -124,7 +143,7 @@ void kill_friendlist(ToxWindow *self)
     realloc_blocklist(0);
     realloc_friends(0);
     free(self->help);
-    del_window(self);
+    del_window(self, c_config);
 }
 
 static void clear_blocklist_index(size_t idx)
@@ -161,9 +180,9 @@ static int save_blocklist(char *path)
         return -1;
     }
 
-    int i, count = 0;
+    int count = 0;
 
-    for (i = 0; i < Blocked.max_idx; ++i) {
+    for (int i = 0; i < Blocked.max_idx; ++i) {
         if (count > Blocked.num_blocked) {
             free(data);
             return -1;
@@ -247,13 +266,18 @@ int load_blocklist(char *path)
         return -1;
     }
 
+    if (!file_exists(path)) {
+        fprintf(stderr, "block list file `%s` doesn't exist\n", path);
+        return 0;
+    }
+
     FILE *fp = fopen(path, "rb");
 
     if (fp == NULL) {
         return -1;
     }
 
-    off_t len = file_size(path);
+    const off_t len = file_size(path);
 
     if (len == 0) {
         fclose(fp);
@@ -279,7 +303,7 @@ int load_blocklist(char *path)
         return -1;
     }
 
-    int num = len / sizeof(BlockedFriend);
+    const int num = len / sizeof(BlockedFriend);
     Blocked.max_idx = num;
     realloc_blocklist(num);
 
@@ -363,24 +387,31 @@ static void sort_blocklist_index(void)
     qsort(Blocked.index, Blocked.num_blocked, sizeof(uint32_t), index_name_cmp_block);
 }
 
-static void update_friend_last_online(uint32_t num, time_t timestamp)
+static void update_friend_last_online(uint32_t num, time_t timestamp, const char *timestamp_format)
 {
     Friends.list[num].last_online.last_on = timestamp;
     Friends.list[num].last_online.tm = *localtime((const time_t *)&timestamp);
 
     /* if the format changes make sure TIME_STR_SIZE is the correct size */
-    const char *t = user_settings->timestamp_format;
-    strftime(Friends.list[num].last_online.hour_min_str, TIME_STR_SIZE, t,
+    strftime(Friends.list[num].last_online.hour_min_str, TIME_STR_SIZE, timestamp_format,
              &Friends.list[num].last_online.tm);
 }
 
-static void friendlist_onMessage(ToxWindow *self, Tox *m, uint32_t num, Tox_Message_Type type, const char *str,
+static void friendlist_onMessage(ToxWindow *self, Toxic *toxic, uint32_t num, Tox_Message_Type type, const char *str,
                                  size_t length)
 {
     UNUSED_VAR(self);
     UNUSED_VAR(type);
     UNUSED_VAR(length);
 
+    if (toxic == NULL) {
+        return;
+    }
+
+    Tox *tox = toxic->tox;
+    const Client_Config *c_config = toxic->c_config;
+    ToxWindow *home_window = toxic->home_window;
+
     if (num >= Friends.max_idx) {
         return;
     }
@@ -390,22 +421,28 @@ static void friendlist_onMessage(ToxWindow *self, Tox *m, uint32_t num, Tox_Mess
     }
 
     if (get_num_active_windows() < MAX_WINDOWS_NUM) {
-        Friends.list[num].chatwin = add_window(m, new_chat(m, Friends.list[num].num));
+        Friends.list[num].chatwin = add_window(toxic, new_chat(tox, Friends.list[num].num));
         return;
     }
 
     char nick[TOX_MAX_NAME_LENGTH];
-    get_nick_truncate(m, nick, num);
+    get_nick_truncate(tox, nick, num);
 
-    line_info_add(prompt, true, nick, NULL, IN_MSG, 0, 0, "%s", str);
-    line_info_add(prompt, false, NULL, NULL, SYS_MSG, 0, RED, "* Warning: Too many windows are open.");
-    sound_notify(prompt, notif_error, NT_WNDALERT_1, NULL);
+    line_info_add(home_window, c_config, true, nick, NULL, IN_MSG, 0, 0, "%s", str);
+    line_info_add(home_window, c_config, false, NULL, NULL, SYS_MSG, 0, RED,
+                  "* Warning: Too many windows are open.");
+
+    sound_notify(home_window, c_config, notif_error, NT_WNDALERT_1, NULL);
 }
 
-static void friendlist_onConnectionChange(ToxWindow *self, Tox *m, uint32_t num, Tox_Connection connection_status)
+static void friendlist_onConnectionChange(ToxWindow *self, Toxic *toxic, uint32_t num, Tox_Connection connection_status)
 {
     UNUSED_VAR(self);
 
+    if (toxic == NULL) {
+        return;
+    }
+
     if (num >= Friends.max_idx) {
         return;
     }
@@ -415,18 +452,18 @@ static void friendlist_onConnectionChange(ToxWindow *self, Tox *m, uint32_t num,
     } else if (Friends.list[num].connection_status == TOX_CONNECTION_NONE) {
         ++Friends.num_online;
 
-        if (avatar_send(m, num) == -1) {
+        if (avatar_send(toxic->tox, num) == -1) {
             fprintf(stderr, "avatar_send failed for friend %u\n", num);
         }
     }
 
     Friends.list[num].connection_status = connection_status;
-    update_friend_last_online(num, get_unix_time());
-    store_data(m, DATA_FILE);
+    update_friend_last_online(num, get_unix_time(), toxic->c_config->timestamp_format);
+    store_data(toxic);
     sort_friendlist_index();
 }
 
-static void friendlist_onNickChange(ToxWindow *self, Tox *m, uint32_t num, const char *nick, size_t length)
+static void friendlist_onNickChange(ToxWindow *self, Toxic *toxic, uint32_t num, const char *nick, size_t length)
 {
     UNUSED_VAR(self);
     UNUSED_VAR(length);
@@ -447,10 +484,11 @@ static void friendlist_onNickChange(ToxWindow *self, Tox *m, uint32_t num, const
     char newnamecpy[TOXIC_MAX_NAME_LENGTH + 1];
     char myid[TOX_ADDRESS_SIZE];
     strcpy(newnamecpy, Friends.list[num].name);
-    tox_self_get_address(m, (uint8_t *) myid);
+    tox_self_get_address(toxic->tox, (uint8_t *) myid);
 
     if (strcmp(oldname, newnamecpy) != 0) {
-        if (rename_logfile(oldname, newnamecpy, myid, Friends.list[num].pub_key, Friends.list[num].chatwin) != 0) {
+        if (rename_logfile(toxic->c_config, oldname, newnamecpy, myid, Friends.list[num].pub_key,
+                           Friends.list[num].chatwin) != 0) {
             fprintf(stderr, "Failed to rename friend chat log from `%s` to `%s`\n", oldname, newnamecpy);
         }
     }
@@ -458,10 +496,10 @@ static void friendlist_onNickChange(ToxWindow *self, Tox *m, uint32_t num, const
     sort_friendlist_index();
 }
 
-static void friendlist_onStatusChange(ToxWindow *self, Tox *m, uint32_t num, Tox_User_Status status)
+static void friendlist_onStatusChange(ToxWindow *self, Toxic *toxic, uint32_t num, Tox_User_Status status)
 {
     UNUSED_VAR(self);
-    UNUSED_VAR(m);
+    UNUSED_VAR(toxic);
 
     if (num >= Friends.max_idx) {
         return;
@@ -482,16 +520,22 @@ static void friendlist_onStatusMessageChange(ToxWindow *self, uint32_t num, cons
     Friends.list[num].statusmsg_len = strlen(Friends.list[num].statusmsg);
 }
 
-void friendlist_onFriendAdded(ToxWindow *self, Tox *m, uint32_t num, bool sort)
+void friendlist_onFriendAdded(ToxWindow *self, Toxic *toxic, uint32_t num, bool sort)
 {
     UNUSED_VAR(self);
 
+    if (toxic == NULL) {
+        fprintf(stderr, "friendlist_onFriendAdded null param\n");
+        return;
+    }
+
+    Tox *tox = toxic->tox;
+    const Client_Config *c_config = toxic->c_config;
+
     realloc_friends(Friends.max_idx + 1);
     clear_friendlist_index(Friends.max_idx);
 
-    uint32_t i;
-
-    for (i = 0; i <= Friends.max_idx; ++i) {
+    for (uint32_t i = 0; i <= Friends.max_idx; ++i) {
         if (Friends.list[i].active) {
             continue;
         }
@@ -504,26 +548,26 @@ void friendlist_onFriendAdded(ToxWindow *self, Tox *m, uint32_t num, bool sort)
         Friends.list[i].auto_accept_files = false;  // do not change
         Friends.list[i].connection_status = TOX_CONNECTION_NONE;
         Friends.list[i].status = TOX_USER_STATUS_NONE;
-        Friends.list[i].logging_on = (bool) user_settings->autolog == AUTOLOG_ON;
+        set_default_friend_config_settings(&Friends.list[i], c_config);
 
         Tox_Err_Friend_Get_Public_Key pkerr;
-        tox_friend_get_public_key(m, num, (uint8_t *) Friends.list[i].pub_key, &pkerr);
+        tox_friend_get_public_key(tox, num, (uint8_t *) Friends.list[i].pub_key, &pkerr);
 
         if (pkerr != TOX_ERR_FRIEND_GET_PUBLIC_KEY_OK) {
             fprintf(stderr, "tox_friend_get_public_key failed (error %d)\n", pkerr);
         }
 
         Tox_Err_Friend_Get_Last_Online loerr;
-        time_t t = tox_friend_get_last_online(m, num, &loerr);
+        time_t t = tox_friend_get_last_online(tox, num, &loerr);
 
         if (loerr != TOX_ERR_FRIEND_GET_LAST_ONLINE_OK) {
             t = 0;
         }
 
-        update_friend_last_online(i, t);
+        update_friend_last_online(i, t, c_config->timestamp_format);
 
         char tempname[TOX_MAX_NAME_LENGTH + 1];
-        int name_len = get_nick_truncate(m, tempname, num);
+        int name_len = get_nick_truncate(tox, tempname, num);
         memcpy(Friends.list[i].name, tempname, name_len);
         Friends.list[i].name[name_len] = 0;
         Friends.list[i].namelength = name_len;
@@ -545,14 +589,12 @@ void friendlist_onFriendAdded(ToxWindow *self, Tox *m, uint32_t num, bool sort)
 }
 
 /* Puts blocked friend back in friendlist. fnum is new friend number, bnum is blocked number. */
-static void friendlist_add_blocked(uint32_t fnum, uint32_t bnum)
+static void friendlist_add_blocked(const Client_Config *c_config, uint32_t fnum, uint32_t bnum)
 {
     realloc_friends(Friends.max_idx + 1);
     clear_friendlist_index(Friends.max_idx);
 
-    int i;
-
-    for (i = 0; i <= Friends.max_idx; ++i) {
+    for (int i = 0; i <= Friends.max_idx; ++i) {
         if (Friends.list[i].active) {
             continue;
         }
@@ -563,11 +605,11 @@ static void friendlist_add_blocked(uint32_t fnum, uint32_t bnum)
         Friends.list[i].active = true;
         Friends.list[i].chatwin = -1;
         Friends.list[i].status = TOX_USER_STATUS_NONE;
-        Friends.list[i].logging_on = (bool) user_settings->autolog == AUTOLOG_ON;
         Friends.list[i].namelength = Blocked.list[bnum].namelength;
-        update_friend_last_online(i, Blocked.list[bnum].last_on);
+        update_friend_last_online(i, Blocked.list[bnum].last_on, c_config->timestamp_format);
         memcpy(Friends.list[i].name, Blocked.list[bnum].name, Friends.list[i].namelength + 1);
         memcpy(Friends.list[i].pub_key, Blocked.list[bnum].pub_key, TOX_PUBLIC_KEY_SIZE);
+        set_default_friend_config_settings(&Friends.list[i], c_config);
 
         if (i == Friends.max_idx) {
             ++Friends.max_idx;
@@ -585,12 +627,21 @@ static void friendlist_add_blocked(uint32_t fnum, uint32_t bnum)
 
 #ifdef GAMES
 
-static void friendlist_onGameInvite(ToxWindow *self, Tox *m, uint32_t friend_number, const uint8_t *data, size_t length)
+static void friendlist_onGameInvite(ToxWindow *self, Toxic *toxic, uint32_t friend_number, const uint8_t *data,
+                                    size_t length)
 {
     UNUSED_VAR(self);
     UNUSED_VAR(data);
     UNUSED_VAR(length);
 
+    if (toxic == NULL) {
+        return;
+    }
+
+    Tox *tox = toxic->tox;
+    const Client_Config *c_config = toxic->c_config;
+    ToxWindow *home_window = toxic->home_window;
+
     if (friend_number >= Friends.max_idx) {
         return;
     }
@@ -600,22 +651,22 @@ static void friendlist_onGameInvite(ToxWindow *self, Tox *m, uint32_t friend_num
     }
 
     if (get_num_active_windows() < MAX_WINDOWS_NUM) {
-        Friends.list[friend_number].chatwin = add_window(m, new_chat(m, Friends.list[friend_number].num));
+        Friends.list[friend_number].chatwin = add_window(toxic, new_chat(tox, Friends.list[friend_number].num));
         return;
     }
 
     char nick[TOX_MAX_NAME_LENGTH];
-    get_nick_truncate(m, nick, friend_number);
+    get_nick_truncate(tox, nick, friend_number);
 
-    line_info_add(prompt, false, NULL, NULL, SYS_MSG, 0, RED,
+    line_info_add(home_window, c_config, false, NULL, NULL, SYS_MSG, 0, RED,
                   "* Game invite from %s failed: Too many windows are open.", nick);
 
-    sound_notify(prompt, notif_error, NT_WNDALERT_1, NULL);
+    sound_notify(home_window, c_config, notif_error, NT_WNDALERT_1, NULL);
 }
 
 #endif // GAMES
 
-static void friendlist_onFileRecv(ToxWindow *self, Tox *m, uint32_t num, uint32_t filenum,
+static void friendlist_onFileRecv(ToxWindow *self, Toxic *toxic, uint32_t num, uint32_t filenum,
                                   uint64_t file_size, const char *filename, size_t name_length)
 {
     UNUSED_VAR(self);
@@ -623,6 +674,10 @@ static void friendlist_onFileRecv(ToxWindow *self, Tox *m, uint32_t num, uint32_
     UNUSED_VAR(filename);
     UNUSED_VAR(name_length);
 
+    Tox *tox = toxic->tox;
+    const Client_Config *c_config = toxic->c_config;
+    ToxWindow *home_window = toxic->home_window;
+
     if (num >= Friends.max_idx) {
         return;
     }
@@ -632,22 +687,22 @@ static void friendlist_onFileRecv(ToxWindow *self, Tox *m, uint32_t num, uint32_
     }
 
     if (get_num_active_windows() < MAX_WINDOWS_NUM) {
-        Friends.list[num].chatwin = add_window(m, new_chat(m, Friends.list[num].num));
+        Friends.list[num].chatwin = add_window(toxic, new_chat(tox, Friends.list[num].num));
         return;
     }
 
-    tox_file_control(m, num, filenum, TOX_FILE_CONTROL_CANCEL, NULL);
+    tox_file_control(tox, num, filenum, TOX_FILE_CONTROL_CANCEL, NULL);
 
     char nick[TOX_MAX_NAME_LENGTH];
-    get_nick_truncate(m, nick, num);
+    get_nick_truncate(tox, nick, num);
 
-    line_info_add(prompt, false, NULL, NULL, SYS_MSG, 0, RED,
+    line_info_add(home_window, c_config, false, NULL, NULL, SYS_MSG, 0, RED,
                   "* File transfer from %s failed: too many windows are open.", nick);
 
-    sound_notify(prompt, notif_error, NT_WNDALERT_1, NULL);
+    sound_notify(home_window, c_config, notif_error, NT_WNDALERT_1, NULL);
 }
 
-static void friendlist_onConferenceInvite(ToxWindow *self, Tox *m, int32_t num, uint8_t type,
+static void friendlist_onConferenceInvite(ToxWindow *self, Toxic *toxic, int32_t num, uint8_t type,
         const char *conference_pub_key,
         uint16_t length)
 {
@@ -656,6 +711,14 @@ static void friendlist_onConferenceInvite(ToxWindow *self, Tox *m, int32_t num,
     UNUSED_VAR(conference_pub_key);
     UNUSED_VAR(length);
 
+    if (toxic == NULL) {
+        return;
+    }
+
+    Tox *tox = toxic->tox;
+    const Client_Config *c_config = toxic->c_config;
+    ToxWindow *home_window = toxic->home_window;
+
     if (num >= Friends.max_idx) {
         return;
     }
@@ -665,26 +728,34 @@ static void friendlist_onConferenceInvite(ToxWindow *self, Tox *m, int32_t num,
     }
 
     if (get_num_active_windows() < MAX_WINDOWS_NUM) {
-        Friends.list[num].chatwin = add_window(m, new_chat(m, Friends.list[num].num));
+        Friends.list[num].chatwin = add_window(toxic, new_chat(tox, Friends.list[num].num));
         return;
     }
 
     char nick[TOX_MAX_NAME_LENGTH];
-    get_nick_truncate(m, nick, num);
+    get_nick_truncate(tox, nick, num);
 
-    line_info_add(prompt, false, NULL, NULL, SYS_MSG, 0, RED,
+    line_info_add(home_window, c_config, false, NULL, NULL, SYS_MSG, 0, RED,
                   "* Conference chat invite from %s failed: too many windows are open.", nick);
 
-    sound_notify(prompt, notif_error, NT_WNDALERT_1, NULL);
+    sound_notify(home_window, c_config, notif_error, NT_WNDALERT_1, NULL);
 }
 
-static void friendlist_onGroupInvite(ToxWindow *self, Tox *m, uint32_t num, const char *data, size_t length,
+static void friendlist_onGroupInvite(ToxWindow *self, Toxic *toxic, uint32_t num, const char *data, size_t length,
                                      const char *group_name, size_t group_name_length)
 {
     UNUSED_VAR(self);
     UNUSED_VAR(data);
     UNUSED_VAR(length);
 
+    if (toxic == NULL) {
+        return;
+    }
+
+    Tox *tox = toxic->tox;
+    const Client_Config *c_config = toxic->c_config;
+    ToxWindow *home_window = toxic->home_window;
+
     if (num >= Friends.max_idx) {
         return;
     }
@@ -694,17 +765,17 @@ static void friendlist_onGroupInvite(ToxWindow *self, Tox *m, uint32_t num, cons
     }
 
     if (get_num_active_windows() < MAX_WINDOWS_NUM) {
-        Friends.list[num].chatwin = add_window(m, new_chat(m, Friends.list[num].num));
+        Friends.list[num].chatwin = add_window(toxic, new_chat(tox, Friends.list[num].num));
         return;
     }
 
     char nick[TOX_MAX_NAME_LENGTH];
-    get_nick_truncate(m, nick, num);
+    get_nick_truncate(tox, nick, num);
 
-    line_info_add(prompt, NULL, NULL, NULL, SYS_MSG, 0, RED,
+    line_info_add(home_window, c_config, NULL, NULL, NULL, SYS_MSG, 0, RED,
                   "* Group chat invite from %s failed: too many windows are open.", nick);
 
-    sound_notify(prompt, notif_error, NT_WNDALERT_1, NULL);
+    sound_notify(home_window, c_config, notif_error, NT_WNDALERT_1, NULL);
 }
 
 /* move friendlist/blocklist cursor up and down */
@@ -723,14 +794,20 @@ static void select_friend(wint_t key, int *selected, int num)
     }
 }
 
-static void delete_friend(Tox *m, uint32_t f_num)
+static void delete_friend(Toxic *toxic, uint32_t f_num)
 {
-    kill_all_file_transfers_friend(m, f_num);
-    kill_avatar_file_transfers_friend(m, f_num);
+    if (toxic == NULL) {
+        return;
+    }
+
+    Tox *tox = toxic->tox;
+
+    kill_all_file_transfers_friend(toxic, f_num);
+    kill_avatar_file_transfers_friend(toxic, f_num);
 
     Tox_Err_Friend_Delete err;
 
-    if (tox_friend_delete(m, f_num, &err) != true) {
+    if (tox_friend_delete(tox, f_num, &err) != true) {
         fprintf(stderr, "tox_friend_delete failed with error %d\n", err);
         return;
     }
@@ -746,7 +823,7 @@ static void delete_friend(Tox *m, uint32_t f_num)
         ToxWindow *toxwin = get_window_ptr(Friends.list[f_num].chatwin);
 
         if (toxwin != NULL) {
-            kill_chat_window(toxwin, m);
+            kill_chat_window(toxwin, toxic);
             set_active_window_index(1);   /* keep friendlist focused */
         }
     }
@@ -777,7 +854,7 @@ static void delete_friend(Tox *m, uint32_t f_num)
         --Friends.num_selected;
     }
 
-    store_data(m, DATA_FILE);
+    store_data(toxic);
 }
 
 /* activates delete friend popup */
@@ -788,17 +865,17 @@ static void del_friend_activate(uint32_t f_num)
     PendingDelete.num = f_num;
 }
 
-static void delete_blocked_friend(uint32_t bnum);
+static void delete_blocked_friend(Toxic *toxic, uint32_t bnum);
 
 /* deactivates delete friend popup and deletes friend if instructed */
-static void del_friend_deactivate(Tox *m, wint_t key)
+static void del_friend_deactivate(Toxic *toxic, wint_t key)
 {
     if (key == L'y') {
         if (blocklist_view == 0) {
-            delete_friend(m, PendingDelete.num);
+            delete_friend(toxic, PendingDelete.num);
             sort_friendlist_index();
         } else {
-            delete_blocked_friend(PendingDelete.num);
+            delete_blocked_friend(toxic, PendingDelete.num);
             sort_blocklist_index();
         }
     }
@@ -844,7 +921,7 @@ static void draw_del_popup(void)
 }
 
 /* deletes contact from blocked list */
-static void delete_blocked_friend(uint32_t bnum)
+static void delete_blocked_friend(Toxic *toxic, uint32_t bnum)
 {
     clear_blocklist_index(bnum);
 
@@ -859,7 +936,7 @@ static void delete_blocked_friend(uint32_t bnum)
     --Blocked.num_blocked;
     Blocked.max_idx = i;
     realloc_blocklist(i);
-    save_blocklist(BLOCK_FILE);
+    save_blocklist(toxic->client_data.block_path);
 
     if (Blocked.num_blocked && Blocked.num_selected == Blocked.num_blocked) {
         --Blocked.num_selected;
@@ -867,7 +944,7 @@ static void delete_blocked_friend(uint32_t bnum)
 }
 
 /* deletes contact from friendlist and puts in blocklist */
-void block_friend(Tox *m, uint32_t fnum)
+static void block_friend(Toxic *toxic, uint32_t fnum)
 {
     if (Friends.num_friends == 0) {
         return;
@@ -894,8 +971,8 @@ void block_friend(Tox *m, uint32_t fnum)
             ++Blocked.max_idx;
         }
 
-        delete_friend(m, fnum);
-        save_blocklist(BLOCK_FILE);
+        delete_friend(toxic, fnum);
+        save_blocklist(toxic->client_data.block_path);
         sort_blocklist_index();
         sort_friendlist_index();
 
@@ -904,22 +981,23 @@ void block_friend(Tox *m, uint32_t fnum)
 }
 
 /* removes friend from blocklist, puts back in friendlist */
-static void unblock_friend(Tox *m, uint32_t bnum)
+static void unblock_friend(Toxic *toxic, uint32_t bnum)
 {
     if (Blocked.num_blocked <= 0) {
         return;
     }
 
     Tox_Err_Friend_Add err;
-    uint32_t friendnum = tox_friend_add_norequest(m, (uint8_t *) Blocked.list[bnum].pub_key, &err);
+    uint32_t friendnum = tox_friend_add_norequest(toxic->tox, (uint8_t *) Blocked.list[bnum].pub_key, &err);
 
     if (err != TOX_ERR_FRIEND_ADD_OK) {
-        line_info_add(prompt, false, NULL, NULL, SYS_MSG, 0, 0, "Failed to unblock friend (error %d)", err);
+        line_info_add(toxic->home_window, toxic->c_config, false, NULL, NULL, SYS_MSG, 0, 0,
+                      "Failed to unblock friend (error %d)", err);
         return;
     }
 
-    friendlist_add_blocked(friendnum, bnum);
-    delete_blocked_friend(bnum);
+    friendlist_add_blocked(toxic->c_config, friendnum, bnum);
+    delete_blocked_friend(toxic, bnum);
     sort_blocklist_index();
     sort_friendlist_index();
 }
@@ -927,8 +1005,15 @@ static void unblock_friend(Tox *m, uint32_t bnum)
 /*
  * Return true if input is recognized by handler
  */
-static bool friendlist_onKey(ToxWindow *self, Tox *m, wint_t key, bool ltr)
+static bool friendlist_onKey(ToxWindow *self, Toxic *toxic, wint_t key, bool ltr)
 {
+    if (toxic == NULL || self == NULL) {
+        return false;
+    }
+
+    Tox *tox = toxic->tox;
+    const Client_Config *c_config = toxic->c_config;
+    ToxWindow *home_window = toxic->home_window;
 
     if (self->help->active) {
         help_onKey(self, key);
@@ -959,7 +1044,7 @@ static bool friendlist_onKey(ToxWindow *self, Tox *m, wint_t key, bool ltr)
     /* lock screen and force decision on deletion popup */
     if (PendingDelete.active) {
         if (key == L'y' || key == L'n') {
-            del_friend_deactivate(m, key);
+            del_friend_deactivate(toxic, key);
         }
 
         return true;
@@ -979,12 +1064,12 @@ static bool friendlist_onKey(ToxWindow *self, Tox *m, wint_t key, bool ltr)
             if (Friends.list[f].chatwin != -1) {
                 set_active_window_index(Friends.list[f].chatwin);
             } else if (get_num_active_windows() < MAX_WINDOWS_NUM) {
-                Friends.list[f].chatwin = add_window(m, new_chat(m, Friends.list[f].num));
+                Friends.list[f].chatwin = add_window(toxic, new_chat(tox, Friends.list[f].num));
                 set_active_window_index(Friends.list[f].chatwin);
             } else {
                 const char *msg = "* Warning: Too many windows are open.";
-                line_info_add(prompt, false, NULL, NULL, SYS_MSG, 0, RED, msg);
-                sound_notify(prompt, notif_error, NT_WNDALERT_1, NULL);
+                line_info_add(home_window, c_config, false, NULL, NULL, SYS_MSG, 0, RED, msg);
+                sound_notify(home_window, c_config, notif_error, NT_WNDALERT_1, NULL);
             }
 
             break;
@@ -995,9 +1080,9 @@ static bool friendlist_onKey(ToxWindow *self, Tox *m, wint_t key, bool ltr)
 
         case L'b':
             if (!blocklist_view) {
-                block_friend(m, f);
+                block_friend(toxic, f);
             } else {
-                unblock_friend(m, f);
+                unblock_friend(toxic, f);
             }
 
             break;
@@ -1022,9 +1107,9 @@ static bool friendlist_onKey(ToxWindow *self, Tox *m, wint_t key, bool ltr)
 
 #define FLIST_OFST 6    /* Accounts for space at top and bottom */
 
-static void blocklist_onDraw(ToxWindow *self, Tox *m, int y2, int x2)
+static void blocklist_onDraw(ToxWindow *self, Tox *tox, int y2, int x2)
 {
-    UNUSED_VAR(m);
+    UNUSED_VAR(tox);
 
     wattron(self->window, A_BOLD);
     wprintw(self->window, " Blocked: ");
@@ -1096,12 +1181,17 @@ static void blocklist_onDraw(ToxWindow *self, Tox *m, int y2, int x2)
     draw_del_popup();
 
     if (self->help->active) {
-        help_onDraw(self);
+        help_draw_main(self);
     }
 }
 
-static void friendlist_onDraw(ToxWindow *self, Tox *m)
+static void friendlist_onDraw(ToxWindow *self, Toxic *toxic)
 {
+    if (toxic == NULL || self == NULL) {
+        fprintf(stderr, "friendlist_onDraw null param\n");
+        return;
+    }
+
     curs_set(0);
     werase(self->window);
     int x2, y2;
@@ -1120,7 +1210,7 @@ static void friendlist_onDraw(ToxWindow *self, Tox *m)
     draw_window_bar(self);
 
     if (blocklist_view == 1) {
-        blocklist_onDraw(self, m, y2, x2);
+        blocklist_onDraw(self, toxic->tox, y2, x2);
         return;
     }
 
@@ -1218,8 +1308,8 @@ static void friendlist_onDraw(ToxWindow *self, Tox *m)
                     char statusmsg[TOX_MAX_STATUS_MESSAGE_LENGTH];
 
                     pthread_mutex_lock(&Winthread.lock);
-                    tox_friend_get_status_message(m, Friends.list[f].num, (uint8_t *) statusmsg, NULL);
-                    size_t s_len = tox_friend_get_status_message_size(m, Friends.list[f].num, NULL);
+                    tox_friend_get_status_message(toxic->tox, Friends.list[f].num, (uint8_t *) statusmsg, NULL);
+                    const size_t s_len = tox_friend_get_status_message_size(toxic->tox, Friends.list[f].num, NULL);
                     pthread_mutex_unlock(&Winthread.lock);
 
                     statusmsg[s_len] = '\0';
@@ -1323,13 +1413,17 @@ static void friendlist_onDraw(ToxWindow *self, Tox *m)
     draw_del_popup();
 
     if (self->help->active) {
-        help_onDraw(self);
+        help_draw_main(self);
     }
 }
 
-void friendlist_onInit(ToxWindow *self, Tox *m)
+void friendlist_onInit(ToxWindow *self, Toxic *toxic)
 {
-    UNUSED_VAR(m);
+    UNUSED_VAR(toxic);
+
+    if (self == NULL) {
+        return;
+    }
 
     int x2;
     int y2;
@@ -1352,31 +1446,37 @@ void disable_chatwin(uint32_t f_num)
 }
 
 #ifdef AUDIO
-static void friendlist_onAV(ToxWindow *self, ToxAV *av, uint32_t friend_number, int state)
+static void friendlist_onAV(ToxWindow *self, Toxic *toxic, uint32_t friend_number, int state)
 {
     UNUSED_VAR(self);
 
+    if (toxic == NULL) {
+        return;
+    }
+
     if (friend_number >= Friends.max_idx) {
         return;
     }
 
-    Tox *m = toxav_get_tox(av);
+    Tox *tox = toxic->tox;
+    const Client_Config *c_config = toxic->c_config;
+    ToxWindow *home_window = toxic->home_window;
 
     if (Friends.list[friend_number].chatwin == -1) {
         if (get_num_active_windows() < MAX_WINDOWS_NUM) {
             if (state != TOXAV_FRIEND_CALL_STATE_FINISHED) {
-                Friends.list[friend_number].chatwin = add_window(m, new_chat(m, Friends.list[friend_number].num));
+                Friends.list[friend_number].chatwin = add_window(toxic, new_chat(tox, Friends.list[friend_number].num));
                 set_active_window_index(Friends.list[friend_number].chatwin);
             }
         } else {
             char nick[TOX_MAX_NAME_LENGTH];
-            get_nick_truncate(m, nick, Friends.list[friend_number].num);
-            line_info_add(prompt, false, NULL, NULL, SYS_MSG, 0, 0, "Audio action from: %s!", nick);
+            get_nick_truncate(tox, nick, Friends.list[friend_number].num);
+            line_info_add(home_window, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "Audio action from: %s!", nick);
 
             const char *errmsg = "* Warning: Too many windows are open.";
-            line_info_add(prompt, false, NULL, NULL, SYS_MSG, 0, RED, errmsg);
+            line_info_add(home_window, c_config, false, NULL, NULL, SYS_MSG, 0, RED, errmsg);
 
-            sound_notify(prompt, notif_error, NT_WNDALERT_1, NULL);
+            sound_notify(home_window, c_config, notif_error, NT_WNDALERT_1, NULL);
         }
     }
 }
@@ -1422,13 +1522,49 @@ bool friend_is_blocked(const char *public_key)
     return false;
 }
 
+void friend_set_logging_enabled(uint32_t friendnumber, bool enable_log)
+{
+    if (friendnumber >= Friends.max_idx) {
+        return;
+    }
+
+    ToxicFriend *friend = &Friends.list[friendnumber];
+
+    if (!friend->active) {
+        return;
+    }
+
+    friend->logging_on = enable_log;
+}
+
+bool friend_get_logging_enabled(uint32_t friendnumber)
+{
+    if (friendnumber >= Friends.max_idx) {
+        return false;
+    }
+
+    const ToxicFriend *friend = &Friends.list[friendnumber];
+
+    if (!friend->active) {
+        return false;
+    }
+
+    return friend->logging_on;
+}
+
 void friend_set_auto_file_accept(uint32_t friendnumber, bool auto_accept)
 {
     if (friendnumber >= Friends.max_idx) {
         return;
     }
 
-    Friends.list[friendnumber].auto_accept_files = auto_accept;
+    ToxicFriend *friend = &Friends.list[friendnumber];
+
+    if (!friend->active) {
+        return;
+    }
+
+    friend->auto_accept_files = auto_accept;
 }
 
 bool friend_get_auto_accept_files(uint32_t friendnumber)
@@ -1437,7 +1573,162 @@ bool friend_get_auto_accept_files(uint32_t friendnumber)
         return false;
     }
 
-    return Friends.list[friendnumber].auto_accept_files;
+    const ToxicFriend *friend = &Friends.list[friendnumber];
+
+    if (!friend->active) {
+        return false;
+    }
+
+    return friend->auto_accept_files;
+}
+
+/*
+ * Returns a pointer to the Friend_Settings object associated with `public_key`.
+ * Returns NULL on failure.
+ */
+static Friend_Settings *get_friend_settings_by_key(const char *public_key)
+{
+    char pk_bin[TOX_PUBLIC_KEY_SIZE];
+
+    if (tox_pk_string_to_bytes(public_key, strlen(public_key), pk_bin, sizeof(pk_bin)) != 0) {
+        return NULL;
+    }
+
+    for (size_t i = 0; i < Friends.max_idx; ++i) {
+        ToxicFriend *friend = &Friends.list[i];
+
+        if (!friend->active) {
+            continue;
+        }
+
+        if (memcmp(pk_bin, friend->pub_key, sizeof(friend->pub_key)) == 0) {
+            return &friend->settings;
+        }
+    }
+
+    return NULL;
+}
+
+bool friend_config_set_show_connection_msg(const char *public_key, bool show_connection_msg)
+{
+    Friend_Settings *settings = get_friend_settings_by_key(public_key);
+
+    if (settings == NULL) {
+        return false;
+    }
+
+    settings->show_connection_msg = show_connection_msg;
+
+    return true;
+}
+
+bool friend_config_get_show_connection_msg(uint32_t friendnumber)
+{
+    if (friendnumber >= Friends.max_idx) {
+        fprintf(stderr, "failed to get friend tab name colour (invalid friendnumber %u)\n", friendnumber);
+        return false;
+    }
+
+    const ToxicFriend *friend = &Friends.list[friendnumber];
+
+    if (!friend->active) {
+        return false;
+    }
+
+    return friend->settings.show_connection_msg;
+}
+
+bool friend_config_set_tab_name_colour(const char *public_key, const char *colour)
+{
+    Friend_Settings *settings = get_friend_settings_by_key(public_key);
+
+    if (settings == NULL) {
+        return false;
+    }
+
+    const int colour_val = colour_string_to_int(colour);
+
+    if (colour_val < 0) {
+        return false;
+    }
+
+    settings->tab_name_colour = colour_val;
+
+    return true;
+}
+
+int friend_config_get_tab_name_colour(uint32_t friendnumber)
+{
+    if (friendnumber >= Friends.max_idx) {
+        fprintf(stderr, "failed to get friend tab name colour (invalid friendnumber %u)\n", friendnumber);
+        return -1;
+    }
+
+    const ToxicFriend *friend = &Friends.list[friendnumber];
+
+    if (!friend->active) {
+        return -1;
+    }
+
+    return friend->settings.tab_name_colour;
+}
+
+bool friend_config_set_autolog(const char *public_key, bool autolog_enabled)
+{
+    Friend_Settings *settings = get_friend_settings_by_key(public_key);
+
+    if (settings == NULL) {
+        return false;
+    }
+
+    settings->autolog = autolog_enabled;
+
+    return true;
+}
+
+bool friend_config_get_autolog(uint32_t friendnumber)
+{
+    if (friendnumber >= Friends.max_idx) {
+        fprintf(stderr, "failed to get autolog setting (invalid friendnumber %u)\n", friendnumber);
+        return false;
+    }
+
+    const ToxicFriend *friend = &Friends.list[friendnumber];
+
+    if (!friend->active) {
+        return false;
+    }
+
+    return friend->settings.autolog;
+}
+
+bool friend_config_set_auto_accept_files(const char *public_key, bool auto_accept_files)
+{
+    Friend_Settings *settings = get_friend_settings_by_key(public_key);
+
+    if (settings == NULL) {
+        return false;
+    }
+
+    settings->auto_accept_files = auto_accept_files;
+
+    return true;
+}
+
+bool friend_config_get_auto_accept_files(uint32_t friendnumber)
+{
+    if (friendnumber >= Friends.max_idx) {
+        fprintf(stderr, "failed to get auto-accept files autolog setting (invalid friendnumber %u)\n", friendnumber);
+        return false;
+    }
+
+    const ToxicFriend *friend = &Friends.list[friendnumber];
+
+    if (!friend->active) {
+        return false;
+    }
+
+    return friend->settings.auto_accept_files;
 }
 
 ToxWindow *new_friendlist(void)
@@ -1467,7 +1758,6 @@ ToxWindow *new_friendlist(void)
     ret->onInvite = &friendlist_onAV;
     ret->onRinging = &friendlist_onAV;
     ret->onStarting = &friendlist_onAV;
-    ret->onEnding = &friendlist_onAV;
     ret->onError = &friendlist_onAV;
     ret->onStart = &friendlist_onAV;
     ret->onCancel = &friendlist_onAV;
