@@ -1,7 +1,7 @@
 /*  game_base.c
  *
  *
- *  Copyright (C) 2020 Toxic All Rights Reserved.
+ *  Copyright (C) 2024 Toxic All Rights Reserved.
  *
  *  This file is part of Toxic.
  *
@@ -38,7 +38,6 @@
 #include "windows.h"
 
 extern struct Winthread Winthread;
-extern struct user_settings *user_settings;
 
 /*
  * Determines the base rate at which game objects should update their state.
@@ -68,7 +67,7 @@ extern struct user_settings *user_settings;
                                                  && ((max_x) >= (GAME_MAX_RECT_X_LARGE)))
 
 
-static ToxWindow *game_new_window(Tox *m, GameType type, uint32_t friendnumber);
+static ToxWindow *game_new_window(Tox *tox, GameType type, uint32_t friendnumber);
 
 struct GameList {
     const char *name;
@@ -112,14 +111,14 @@ const char *game_get_name_string(GameType type)
     return NULL;
 }
 
-void game_list_print(ToxWindow *self)
+void game_list_print(ToxWindow *self, const Client_Config *c_config)
 {
-    line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, "Available games:");
+    line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "Available games:");
 
     const char *name = NULL;
 
     for (size_t i = 0; (name = game_list[i].name); ++i) {
-        line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, "%zu: %s", i + 1, name);
+        line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "- %s", name);
     }
 }
 
@@ -155,11 +154,15 @@ void game_window_notify(const GameData *game, const char *message)
         return;
     }
 
+    const Client_Config *c_config = game->toxic->c_config;
+
+    const int bell_on_message = c_config->bell_on_message;
+
     if (self->active_box != -1) {
-        box_notify2(self, generic_message, NT_WNDALERT_0 | NT_NOFOCUS | user_settings->bell_on_message,
+        box_notify2(self, c_config, generic_message, NT_WNDALERT_0 | NT_NOFOCUS | bell_on_message,
                     self->active_box, "%s", message);
     } else {
-        box_notify(self, generic_message, NT_WNDALERT_0 | NT_NOFOCUS | user_settings->bell_on_message,
+        box_notify(self, c_config, generic_message, NT_WNDALERT_0 | NT_NOFOCUS | bell_on_message,
                    &self->active_box, self->name, "%s", message);
     }
 }
@@ -172,11 +175,11 @@ TIME_MS get_time_millis(void)
     return ((TIME_MS) t.tv_sec) * 1000 + ((TIME_MS) t.tv_nsec) / 1000000;
 }
 
-void game_kill(ToxWindow *self)
+void game_kill(ToxWindow *self, const Client_Config *c_config)
 {
     GameData *game = self->game;
 
-    if (game) {
+    if (game != NULL) {
         if (game->cb_game_kill) {
             game->cb_game_kill(game, game->cb_game_kill_data);
         }
@@ -187,16 +190,16 @@ void game_kill(ToxWindow *self)
     }
 
     kill_notifs(self->active_box);
-    del_window(self);
+    del_window(self, c_config);
 
     if (get_num_active_windows_type(WINDOW_TYPE_GAME) == 0) {
         set_window_refresh_rate(NCURSES_DEFAULT_REFRESH_RATE);
     }
 }
 
-static void game_init_abort(const ToxWindow *parent, ToxWindow *self)
+static void game_init_abort(const ToxWindow *parent, ToxWindow *self, const Client_Config *c_config)
 {
-    game_kill(self);
+    game_kill(self, c_config);
     set_active_window_index(parent->index);
 }
 
@@ -250,16 +253,18 @@ static int game_initialize_type(GameData *game, const uint8_t *data, size_t leng
     return ret;
 }
 
-int game_initialize(const ToxWindow *parent, Tox *m, GameType type, uint32_t id, const uint8_t *multiplayer_data,
+int game_initialize(const ToxWindow *parent, Toxic *toxic, GameType type, uint32_t id, const uint8_t *multiplayer_data,
                     size_t length, bool self_host)
 {
+    const Client_Config *c_config = toxic->c_config;
+
     int max_x;
     int max_y;
     getmaxyx(parent->window, max_y, max_x);
 
     max_y -= (CHATBOX_HEIGHT + WINDOW_BAR_HEIGHT);
 
-    ToxWindow *self = game_new_window(m, type, parent->num);
+    ToxWindow *self = game_new_window(toxic->tox, type, parent->num);
 
     if (self == NULL) {
         return -4;
@@ -267,7 +272,7 @@ int game_initialize(const ToxWindow *parent, Tox *m, GameType type, uint32_t id,
 
     GameData *game = self->game;
 
-    int window_id = add_window(m, self);
+    const int window_id = add_window(toxic, self);
 
     if (window_id == -1) {
         free(game);
@@ -279,19 +284,19 @@ int game_initialize(const ToxWindow *parent, Tox *m, GameType type, uint32_t id,
 
     if (game->is_multiplayer) {
         if (parent->type != WINDOW_TYPE_CHAT) {
-            game_init_abort(parent, self);
+            game_init_abort(parent, self, c_config);
             return -3;
         }
 
         if (get_friend_connection_status(parent->num) == TOX_CONNECTION_NONE) {
-            game_init_abort(parent, self);
+            game_init_abort(parent, self, c_config);
             return -2;
         }
 
         game->is_multiplayer = true;
     }
 
-    game->tox = m;
+    game->toxic = toxic;
     game->window_shape = GW_ShapeSquare;
     game->parent_max_x = max_x;
     game->parent_max_y = max_y;
@@ -303,14 +308,14 @@ int game_initialize(const ToxWindow *parent, Tox *m, GameType type, uint32_t id,
     game->friend_number = parent->num;
 
     if (game->window == NULL) {
-        game_init_abort(parent, self);
+        game_init_abort(parent, self, c_config);
         return -4;
     }
 
     const int init_ret = game_initialize_type(game, multiplayer_data, length, self_host);
 
     if (init_ret < 0) {
-        game_init_abort(parent, self);
+        game_init_abort(parent, self, c_config);
         return init_ret;
     }
 
@@ -708,7 +713,7 @@ static void game_draw_messages(GameData *game, bool priority)
 
 static void game_update_state(GameData *game)
 {
-    if (!game->cb_game_update_state) {
+    if (game->cb_game_update_state == NULL) {
         return;
     }
 
@@ -726,9 +731,14 @@ static void game_update_state(GameData *game)
     }
 }
 
-void game_onDraw(ToxWindow *self, Tox *m)
+static void game_onDraw(ToxWindow *self, Toxic *toxic)
 {
-    UNUSED_VAR(m);   // Note: This function is not thread safe if we ever need to use `m`
+    UNUSED_VAR(toxic);   // Note: This function is not thread safe if we ever need to use `toxic`
+
+    if (self == NULL) {
+        fprintf(stderr, "game_onDraw null param\n");
+        return;
+    }
 
     GameData *game = self->game;
 
@@ -777,16 +787,15 @@ void game_onDraw(ToxWindow *self, Tox *m)
     game_draw_messages(game, true);
 }
 
-bool game_onKey(ToxWindow *self, Tox *m, wint_t key, bool is_printable)
+static bool game_onKey(ToxWindow *self, Toxic *toxic, wint_t key, bool is_printable)
 {
     UNUSED_VAR(is_printable);
-    UNUSED_VAR(m);
 
     GameData *game = self->game;
 
     if (key == KEY_F(9)) {
         pthread_mutex_lock(&Winthread.lock);
-        game_kill(self);
+        game_kill(self, toxic->c_config);
         pthread_mutex_unlock(&Winthread.lock);
         return true;
     }
@@ -819,9 +828,13 @@ bool game_onKey(ToxWindow *self, Tox *m, wint_t key, bool is_printable)
     return true;
 }
 
-void game_onInit(ToxWindow *self, Tox *m)
+static void game_onInit(ToxWindow *self, Toxic *toxic)
 {
-    UNUSED_VAR(m);
+    UNUSED_VAR(toxic);
+
+    if (self == NULL) {
+        return;
+    }
 
     int max_x;
     int max_y;
@@ -840,9 +853,9 @@ void game_onInit(ToxWindow *self, Tox *m)
  * Byte 2-5: Game ID
  * Byte 6-*  Game data
  */
-void game_onPacket(ToxWindow *self, Tox *m, uint32_t friendnumber, const uint8_t *data, size_t length)
+static void game_onPacket(ToxWindow *self, Toxic *toxic, uint32_t friendnumber, const uint8_t *data, size_t length)
 {
-    UNUSED_VAR(m);
+    UNUSED_VAR(toxic);
 
     GameData *game = self->game;
 
@@ -885,7 +898,7 @@ void game_onPacket(ToxWindow *self, Tox *m, uint32_t friendnumber, const uint8_t
     }
 }
 
-static ToxWindow *game_new_window(Tox *m, GameType type, uint32_t friendnumber)
+static ToxWindow *game_new_window(Tox *tox, GameType type, uint32_t friendnumber)
 {
     const char *window_name = game_get_name_string(type);
 
@@ -918,7 +931,7 @@ static ToxWindow *game_new_window(Tox *m, GameType type, uint32_t friendnumber)
 
     if (game_type_is_multi_only(type)) {
         char nick[TOX_MAX_NAME_LENGTH];
-        get_nick_truncate(m, nick, friendnumber);
+        get_nick_truncate(tox, nick, friendnumber);
 
         char buf[sizeof(nick) + sizeof(ret->name) + 4];
         snprintf(buf, sizeof(buf), "%s (%s)", window_name, nick);
@@ -1180,7 +1193,7 @@ int game_packet_send(const GameData *game, const uint8_t *data, size_t length, G
 
     Tox_Err_Friend_Custom_Packet err;
 
-    if (!tox_friend_send_lossless_packet(game->tox, game->friend_number, packet, packet_length, &err)) {
+    if (!tox_friend_send_lossless_packet(game->toxic->tox, game->friend_number, packet, packet_length, &err)) {
         fprintf(stderr, "failed to send game packet: error %d\n", err);
         return -1;
     }
