@@ -1,7 +1,7 @@
 /*  game_centipede.c
  *
  *
- *  Copyright (C) 2020 Toxic All Rights Reserved.
+ *  Copyright (C) 2024 Toxic All Rights Reserved.
  *
  *  This file is part of Toxic.
  *
@@ -849,7 +849,7 @@ static void cent_bullet_move(const GameData *game, CentState *state, TIME_MS cur
     }
 }
 
-void cent_bullet_spawn(CentState *state)
+static void cent_bullet_spawn(CentState *state)
 {
     Projectile *bullet = &state->bullet;
 
@@ -1031,7 +1031,7 @@ static void cent_set_head_direction(CentState *state, Segment *head, int y_botto
         Mushroom *mush = cent_get_mushroom_at_coords(state, &next_coords);
 
         if (head->coords.x <= x_left || mush != NULL) {
-            if (mush && mush->is_poisonous) {
+            if (mush != NULL && mush->is_poisonous) {
                 cent_poison_centipede(head);
             }
 
@@ -1045,7 +1045,7 @@ static void cent_set_head_direction(CentState *state, Segment *head, int y_botto
         Mushroom *mush = cent_get_mushroom_at_coords(state, &next_coords);
 
         if (head->coords.x >= x_right || mush != NULL) {
-            if (mush && mush->is_poisonous) {
+            if (mush != NULL && mush->is_poisonous) {
                 cent_poison_centipede(head);
             }
 
@@ -1505,14 +1505,14 @@ static void cent_mushrooms_draw(WINDOW *win, const CentState *state)
     }
 }
 
-void cent_cb_update_game_state(GameData *game, void *cb_data)
+static void cent_cb_update_game_state(GameData *game, void *cb_data)
 {
-    if (!cb_data) {
+    CentState *state = (CentState *)cb_data;
+
+    if (state == NULL) {
         return;
     }
 
-    CentState *state = (CentState *)cb_data;
-
     if (state->game_over) {
         return;
     }
@@ -1529,14 +1529,14 @@ void cent_cb_update_game_state(GameData *game, void *cb_data)
     cent_do_scorpion(game, state, cur_time);
 }
 
-void cent_cb_render_window(GameData *game, WINDOW *win, void *cb_data)
+static void cent_cb_render_window(GameData *game, WINDOW *win, void *cb_data)
 {
-    if (!cb_data) {
+    CentState *state = (CentState *)cb_data;
+
+    if (state == NULL) {
         return;
     }
 
-    CentState *state = (CentState *)cb_data;
-
     cent_blaster_draw(win, state);
     cent_projectiles_draw(win, state);
     cent_mushrooms_draw(win, state);
@@ -1544,14 +1544,14 @@ void cent_cb_render_window(GameData *game, WINDOW *win, void *cb_data)
     cent_centipede_draw(game, win, state);
 }
 
-void cent_cb_on_keypress(GameData *game, int key, void *cb_data)
+static void cent_cb_on_keypress(GameData *game, int key, void *cb_data)
 {
-    if (!cb_data) {
+    CentState *state = (CentState *)cb_data;
+
+    if (state == NULL) {
         return;
     }
 
-    CentState *state = (CentState *)cb_data;
-
     if (key == CENT_KEY_FIRE) {
         cent_bullet_spawn(state);
         return;
@@ -1564,14 +1564,14 @@ void cent_cb_on_keypress(GameData *game, int key, void *cb_data)
     }
 }
 
-void cent_cb_pause(GameData *game, bool is_paused, void *cb_data)
+static void cent_cb_pause(GameData *game, bool is_paused, void *cb_data)
 {
-    if (!cb_data) {
+    CentState *state = (CentState *)cb_data;
+
+    if (state == NULL) {
         return;
     }
 
-    CentState *state = (CentState *)cb_data;
-
     TIME_S t = get_unix_time();
 
     if (is_paused) {
@@ -1581,14 +1581,14 @@ void cent_cb_pause(GameData *game, bool is_paused, void *cb_data)
     }
 }
 
-void cent_cb_kill(GameData *game, void *cb_data)
+static void cent_cb_kill(GameData *game, void *cb_data)
 {
-    if (!cb_data) {
+    CentState *state = (CentState *)cb_data;
+
+    if (state == NULL) {
         return;
     }
 
-    CentState *state = (CentState *)cb_data;
-
     cent_exterminate_centipedes(&state->centipedes);
 
     free(state->mushrooms);
@@ -1672,9 +1672,6 @@ static int cent_init_state(GameData *game, CentState *state)
 
     state->mushrooms = mushrooms;
 
-    Centipedes *centipedes = &state->centipedes;
-    memset(centipedes->heads, 0, sizeof(centipedes->heads));
-
     Direction dir = rand_range_not_secure(2) == 0 ? WEST : EAST;
 
     if (cent_birth_centipede(game, state, CENT_MAX_NUM_SEGMENTS, dir, NULL) == -1) {
