@@ -1,7 +1,7 @@
 /*  game_chess.c
  *
  *
- *  Copyright (C) 2020 Toxic All Rights Reserved.
+ *  Copyright (C) 2024 Toxic All Rights Reserved.
  *
  *  This file is part of Toxic.
  *
@@ -1484,25 +1484,25 @@ static int chess_get_display_colour(ChessColour p_colour, int tile_colour)
     }
 }
 
-void chess_move_curs_up_left(ChessState *state)
+static void chess_move_curs_up_left(ChessState *state)
 {
     chess_move_curs_left(state);
     chess_move_curs_up(state);
 }
 
-void chess_move_curs_up_right(ChessState *state)
+static void chess_move_curs_up_right(ChessState *state)
 {
     chess_move_curs_right(state);
     chess_move_curs_up(state);
 }
 
-void chess_move_curs_down_left(ChessState *state)
+static void chess_move_curs_down_left(ChessState *state)
 {
     chess_move_curs_left(state);
     chess_move_curs_down(state);
 }
 
-void chess_move_curs_down_right(ChessState *state)
+static void chess_move_curs_down_right(ChessState *state)
 {
     chess_move_curs_right(state);
     chess_move_curs_down(state);
@@ -1714,14 +1714,14 @@ static void chess_draw_interface(const GameData *game, WINDOW *win, ChessState *
     chess_print_captured(game, win, state);
 }
 
-void chess_cb_render_window(GameData *game, WINDOW *win, void *cb_data)
+static void chess_cb_render_window(GameData *game, WINDOW *win, void *cb_data)
 {
-    if (!cb_data) {
+    ChessState *state = (ChessState *)cb_data;
+
+    if (state == NULL) {
         return;
     }
 
-    ChessState *state = (ChessState *)cb_data;
-
     move(state->curs_y, state->curs_x);
 
     curs_set(1);
@@ -1730,14 +1730,14 @@ void chess_cb_render_window(GameData *game, WINDOW *win, void *cb_data)
     chess_draw_interface(game, win, state);
 }
 
-void chess_cb_on_keypress(GameData *game, int key, void *cb_data)
+static void chess_cb_on_keypress(GameData *game, int key, void *cb_data)
 {
-    if (!cb_data) {
+    ChessState *state = (ChessState *)cb_data;
+
+    if (state == NULL) {
         return;
     }
 
-    ChessState *state = (ChessState *)cb_data;
-
     switch (key) {
         case KEY_LEFT: {
             chess_move_curs_left(state);
@@ -1793,14 +1793,14 @@ void chess_cb_on_keypress(GameData *game, int key, void *cb_data)
     }
 }
 
-void chess_cb_kill(GameData *game, void *cb_data)
+static void chess_cb_kill(GameData *game, void *cb_data)
 {
-    if (!cb_data) {
+    ChessState *state = (ChessState *)cb_data;
+
+    if (state == NULL) {
         return;
     }
 
-    ChessState *state = (ChessState *)cb_data;
-
     free(state);
 
     chess_packet_send_resign(game);
@@ -1886,16 +1886,16 @@ static void chess_notify(const GameData *game, ChessPacketType type)
 
 static void chess_cb_on_packet(GameData *game, const uint8_t *data, size_t length, void *cb_data)
 {
+    ChessState *state = (ChessState *)cb_data;
+
+    if (state == NULL) {
+        return;
+    }
+
     if (length == 0 || data == NULL) {
         return;
     }
 
-    if (cb_data == NULL) {
-        return;
-    }
-
-    ChessState *state = (ChessState *)cb_data;
-
     ChessPacketType type = data[0];
 
     switch (type) {
