@@ -1,7 +1,7 @@
 /*  game_snake.c
  *
  *
- *  Copyright (C) 2020 Toxic All Rights Reserved.
+ *  Copyright (C) 2024 Toxic All Rights Reserved.
  *
  *  This file is part of Toxic.
  *
@@ -989,16 +989,16 @@ static void snake_draw_powerup(WINDOW *win, const SnakeState *state)
     }
 }
 
-void snake_cb_update_game_state(GameData *game, void *cb_data)
+static void snake_cb_update_game_state(GameData *game, void *cb_data)
 {
-    if (cb_data == NULL) {
+    SnakeState *state = (SnakeState *)cb_data;
+
+    if (state == NULL) {
         return;
     }
 
     TIME_MS cur_time = get_time_millis();
 
-    SnakeState *state = (SnakeState *)cb_data;
-
     if (!state->is_online) {
         snake_decay_points(game, state);
         snake_do_powerup(game, state);
@@ -1014,14 +1014,14 @@ void snake_cb_update_game_state(GameData *game, void *cb_data)
     }
 }
 
-void snake_cb_render_window(GameData *game, WINDOW *win, void *cb_data)
+static void snake_cb_render_window(GameData *game, WINDOW *win, void *cb_data)
 {
-    if (cb_data == NULL) {
+    SnakeState *state = (SnakeState *)cb_data;
+
+    if (state == NULL) {
         return;
     }
 
-    SnakeState *state = (SnakeState *)cb_data;
-
     snake_draw_food(win, state);
 
     if (!state->is_online) {
@@ -1034,14 +1034,14 @@ void snake_cb_render_window(GameData *game, WINDOW *win, void *cb_data)
     snake_draw_snake(win, state->snake, state->snake_length);
 }
 
-void snake_cb_kill(GameData *game, void *cb_data)
+static void snake_cb_kill(GameData *game, void *cb_data)
 {
-    if (cb_data == NULL) {
+    SnakeState *state = (SnakeState *)cb_data;
+
+    if (state == NULL) {
         return;
     }
 
-    SnakeState *state = (SnakeState *)cb_data;
-
     if (state->is_online && state->status == SnakeStatusPlaying) {
         snake_packet_abort(game);
     }
@@ -1058,26 +1058,26 @@ void snake_cb_kill(GameData *game, void *cb_data)
     game_set_cb_on_pause(game, NULL, NULL);
 }
 
-void snake_cb_on_keypress(GameData *game, int key, void *cb_data)
+static void snake_cb_on_keypress(GameData *game, int key, void *cb_data)
 {
-    if (cb_data == NULL) {
+    SnakeState *state = (SnakeState *)cb_data;
+
+    if (state == NULL) {
         return;
     }
 
-    SnakeState *state = (SnakeState *)cb_data;
-
     snake_set_key_press(state, key);
 }
 
-void snake_cb_pause(GameData *game, bool is_paused, void *cb_data)
+static void snake_cb_pause(GameData *game, bool is_paused, void *cb_data)
 {
-    UNUSED_VAR(game);
+    SnakeState *state = (SnakeState *)cb_data;
 
-    if (cb_data == NULL) {
+    if (state == NULL) {
         return;
     }
 
-    SnakeState *state = (SnakeState *)cb_data;
+    UNUSED_VAR(game);
 
     TIME_S t = get_unix_time();
 
@@ -1580,16 +1580,16 @@ static void snake_handle_abort_packet(GameData *game, SnakeState *state)
 
 static void snake_cb_on_packet(GameData *game, const uint8_t *data, size_t length, void *cb_data)
 {
+    SnakeState *state = (SnakeState *)cb_data;
+
+    if (state == NULL) {
+        return;
+    }
+
     if (length == 0 || data == NULL) {
         return;
     }
 
-    if (cb_data == NULL) {
-        return;
-    }
-
-    SnakeState *state = (SnakeState *)cb_data;
-
     if (!state->is_online) {
         return;
     }
