@@ -1,7 +1,7 @@
 /*  game_util.h
  *
  *
- *  Copyright (C) 2020 Toxic All Rights Reserved.
+ *  Copyright (C) 2024 Toxic All Rights Reserved.
  *
  *  This file is part of Toxic.
  *
@@ -24,6 +24,8 @@
 #define GAME_UTIL
 
 #include <stdbool.h>
+#include <stddef.h>
+#include <stdint.h>
 #include <time.h>
 
 typedef struct Coords {
