@@ -1,7 +1,7 @@
 /*  help.c
  *
  *
- *  Copyright (C) 2014 Toxic All Rights Reserved.
+ *  Copyright (C) 2024 Toxic All Rights Reserved.
  *
  *  This file is part of Toxic.
  *
@@ -177,6 +177,7 @@ static void help_draw_global(ToxWindow *self)
     wprintw(win, "  /add <addr> <msg>          : Add contact with optional message\n");
     wprintw(win, "  /accept <id>               : Accept friend request\n");
     wprintw(win, "  /avatar <path>             : Set an avatar (leave path empty to unset)\n");
+    wprintw(win, "  /color <color>             : Change the colour of the focused window's name\n");
     wprintw(win, "  /conference <type>         : Create a conference where type: text | audio\n");
     wprintw(win, "  /connect <ip> <port> <key> : Manually connect to a DHT node\n");
     wprintw(win, "  /decline <id>              : Decline friend request\n");
@@ -190,7 +191,7 @@ static void help_draw_global(ToxWindow *self)
     wprintw(win, "  /group <name>              : Create a new group chat\n");
     wprintw(win, "  /join <chatid>             : Join a public groupchat using a Chat ID\n");
 #ifdef GAMES
-    wprintw(win, "  /game                      : Play a game\n");
+    wprintw(win, "  /game <name>               : Play a game\n");
 #endif /* GAMES */
 
 #ifdef QRCODE
@@ -255,6 +256,10 @@ static void help_draw_chat(ToxWindow *self)
     wprintw(win, "  /savefile <id>             : Receive a file\n");
     wprintw(win, "  /cancel <type> <id>        : Cancel file transfer where type: in|out\n");
 
+#ifdef GAMES
+    wprintw(win, "  /game <name>               : Play a game with this contact\n");
+#endif /* GAMES */
+
 #ifdef AUDIO
     wattron(win, A_BOLD);
     wprintw(win, "\n Audio:\n");
@@ -279,10 +284,6 @@ static void help_draw_chat(ToxWindow *self)
     wprintw(win, "  /video                     : Toggle video in call\n");
 #endif /* VIDEO */
 
-#ifdef GAMES
-    wprintw(win, "  /game                      : Play a game with contact\n");
-#endif /* GAMES */
-
     help_draw_bottom_menu(win);
 
     box(win, ACS_VLINE, ACS_HLINE);
@@ -364,6 +365,7 @@ static void help_draw_conference(ToxWindow *self)
     wprintw(win, "Conference commands:\n");
     wattroff(win, A_BOLD | COLOR_PAIR(RED));
 
+    wprintw(win, "  /chatid                 : Print this conference's ID\n");
     wprintw(win, "  /title <msg>            : Show/set conference title\n");
 #ifdef AUDIO
     wattron(win, A_BOLD);
@@ -449,7 +451,7 @@ void help_onKey(ToxWindow *self, wint_t key)
             break;
 
         case L'g':
-            height = 24;
+            height = 25;
 #ifdef VIDEO
             height += 8;
 #elif AUDIO
@@ -466,7 +468,7 @@ void help_onKey(ToxWindow *self, wint_t key)
             break;
 
         case L'o':
-            height = 6;
+            height = 7;
 #ifdef AUDIO
             height += 7;
 #endif
@@ -504,7 +506,7 @@ void help_onKey(ToxWindow *self, wint_t key)
     }
 }
 
-void help_onDraw(ToxWindow *self)
+void help_draw_main(ToxWindow *self)
 {
     switch (self->help->type) {
         case HELP_MENU:
