@@ -1,7 +1,7 @@
 /*  help.h
  *
  *
- *  Copyright (C) 2014 Toxic All Rights Reserved.
+ *  Copyright (C) 2024 Toxic All Rights Reserved.
  *
  *  This file is part of Toxic.
  *
@@ -29,6 +29,7 @@
 typedef enum {
     HELP_MENU,
     HELP_GLOBAL,
+    HELP_GROUP,
     HELP_CHAT,
     HELP_CONFERENCE,
     HELP_KEYS,
@@ -38,7 +39,7 @@ typedef enum {
 #endif
 } HELP_TYPES;
 
-void help_onDraw(ToxWindow *self);
+void help_draw_main(ToxWindow *self);
 void help_init_menu(ToxWindow *self);
 void help_onKey(ToxWindow *self, wint_t key);
 
