@@ -1,7 +1,7 @@
 /*  misc_tools.c
  *
  *
- *  Copyright (C) 2014 Toxic All Rights Reserved.
+ *  Copyright (C) 2024 Toxic All Rights Reserved.
  *
  *  This file is part of Toxic.
  *
@@ -34,9 +34,6 @@
 #include "toxic.h"
 #include "windows.h"
 
-extern ToxWindow *prompt;
-extern struct user_settings *user_settings;
-
 void clear_screen(void)
 {
     printf("\033[2J\033[1;1H");
@@ -96,24 +93,21 @@ struct tm *get_time(void)
     return timeinfo;
 }
 
-/* Puts the current time in buf in the format of specified by the config */
-void get_time_str(char *buf, size_t bufsize)
+void get_time_str(char *buf, size_t bufsize, const char *format_string)
 {
     if (buf == NULL || bufsize == 0) {
         return;
     }
 
-    *buf = 0;
-
-    if (user_settings->timestamps == TIMESTAMPS_OFF) {
+    if (strftime(buf, bufsize, format_string, get_time()) > 0) {
         return;
     }
 
-    const char *t = user_settings->timestamp_format;
-
-    if (strftime(buf, bufsize, t, get_time()) == 0) {
-        strftime(buf, bufsize, TIMESTAMP_DEFAULT, get_time());
+    if (strftime(buf, bufsize, TIMESTAMP_DEFAULT, get_time()) > 0) {
+        return;
     }
+
+    buf[0] = '\0';
 }
 
 /* Converts seconds to string in format HH:mm:ss; truncates hours and minutes when necessary */
@@ -440,15 +434,15 @@ void str_to_lower(char *str)
 /* puts friendnum's nick in buf, truncating at TOXIC_MAX_NAME_LENGTH if necessary.
    if toxcore API call fails, put UNKNOWN_NAME in buf
    Returns nick len */
-size_t get_nick_truncate(Tox *m, char *buf, uint32_t friendnum)
+size_t get_nick_truncate(Tox *tox, char *buf, uint32_t friendnum)
 {
     Tox_Err_Friend_Query err;
-    size_t len = tox_friend_get_name_size(m, friendnum, &err);
+    size_t len = tox_friend_get_name_size(tox, friendnum, &err);
 
     if (err != TOX_ERR_FRIEND_QUERY_OK) {
         goto on_error;
     } else {
-        if (!tox_friend_get_name(m, friendnum, (uint8_t *) buf, NULL)) {
+        if (!tox_friend_get_name(tox, friendnum, (uint8_t *) buf, NULL)) {
             goto on_error;
         }
     }
@@ -466,15 +460,15 @@ on_error:
 }
 
 /* same as get_nick_truncate but for conferences */
-int get_conference_nick_truncate(Tox *m, char *buf, uint32_t peernum, uint32_t conferencenum)
+int get_conference_nick_truncate(Tox *tox, char *buf, uint32_t peernum, uint32_t conferencenum)
 {
     Tox_Err_Conference_Peer_Query err;
-    size_t len = tox_conference_peer_get_name_size(m, conferencenum, peernum, &err);
+    size_t len = tox_conference_peer_get_name_size(tox, conferencenum, peernum, &err);
 
     if (err != TOX_ERR_CONFERENCE_PEER_QUERY_OK) {
         goto on_error;
     } else {
-        if (!tox_conference_peer_get_name(m, conferencenum, peernum, (uint8_t *) buf, NULL)) {
+        if (!tox_conference_peer_get_name(tox, conferencenum, peernum, (uint8_t *) buf, NULL)) {
             goto on_error;
         }
     }
@@ -493,16 +487,16 @@ on_error:
 }
 
 /* same as get_nick_truncate but for groupchats */
-size_t get_group_nick_truncate(Tox *m, char *buf, uint32_t peer_id, uint32_t groupnum)
+size_t get_group_nick_truncate(Tox *tox, char *buf, uint32_t peer_id, uint32_t groupnum)
 {
     Tox_Err_Group_Peer_Query err;
-    size_t len = tox_group_peer_get_name_size(m, groupnum, peer_id, &err);
+    size_t len = tox_group_peer_get_name_size(tox, groupnum, peer_id, &err);
 
     if (err != TOX_ERR_GROUP_PEER_QUERY_OK || len == 0) {
         strcpy(buf, UNKNOWN_NAME);
         len = strlen(UNKNOWN_NAME);
     } else {
-        tox_group_peer_get_name(m, groupnum, peer_id, (uint8_t *) buf, &err);
+        tox_group_peer_get_name(tox, groupnum, peer_id, (uint8_t *) buf, &err);
 
         if (err != TOX_ERR_GROUP_PEER_QUERY_OK) {
             strcpy(buf, UNKNOWN_NAME);
@@ -519,16 +513,16 @@ size_t get_group_nick_truncate(Tox *m, char *buf, uint32_t peer_id, uint32_t gro
 }
 
 /* same as get_group_nick_truncate() but for self. */
-size_t get_group_self_nick_truncate(Tox *m, char *buf, uint32_t groupnum)
+size_t get_group_self_nick_truncate(Tox *tox, char *buf, uint32_t groupnum)
 {
     Tox_Err_Group_Self_Query err;
-    size_t len = tox_group_self_get_name_size(m, groupnum, &err);
+    size_t len = tox_group_self_get_name_size(tox, groupnum, &err);
 
     if (err != TOX_ERR_GROUP_SELF_QUERY_OK) {
         strcpy(buf, UNKNOWN_NAME);
         len = strlen(UNKNOWN_NAME);
     } else {
-        tox_group_self_get_name(m, groupnum, (uint8_t *) buf, &err);
+        tox_group_self_get_name(tox, groupnum, (uint8_t *) buf, &err);
 
         if (err != TOX_ERR_GROUP_SELF_QUERY_OK) {
             strcpy(buf, UNKNOWN_NAME);
@@ -565,7 +559,7 @@ size_t copy_tox_str(char *msg, size_t size, const char *data, size_t length)
    returns length of s if char not found or 0 if s is NULL. */
 int char_find(int idx, const char *s, char ch)
 {
-    if (!s) {
+    if (s == NULL) {
         return 0;
     }
 
@@ -584,7 +578,7 @@ int char_find(int idx, const char *s, char ch)
    returns 0 if char not found or s is NULL (skips 0th index). */
 int char_rfind(const char *s, char ch, int len)
 {
-    if (!s) {
+    if (s == NULL) {
         return 0;
     }
 
@@ -751,3 +745,56 @@ unsigned int rand_not_secure(void)
 
     return n;
 }
+
+int colour_string_to_int(const char *colour)
+{
+    if (strcasecmp(colour, "white") == 0) {
+        return WHITE_BAR_FG;
+    }
+
+    if (strcasecmp(colour, "red") == 0) {
+        return RED_BAR_FG;
+    }
+
+    if (strcasecmp(colour, "green") == 0) {
+        return GREEN_BAR_FG;
+    }
+
+    if (strcasecmp(colour, "yellow") == 0) {
+        return YELLOW_BAR_FG;
+    }
+
+    if (strcasecmp(colour, "cyan") == 0) {
+        return CYAN_BAR_FG;
+    }
+
+    if (strcasecmp(colour, "magenta") == 0) {
+        return MAGENTA_BAR_FG;
+    }
+
+    if (strcasecmp(colour, "black") == 0) {
+        return BLACK_BAR_FG;
+    }
+
+    if (strcasecmp(colour, "blue") == 0) {
+        return BLUE_BAR_FG;
+    }
+
+    if (strcasecmp(colour, "gray") == 0) {
+        return GRAY_BAR_FG;
+    }
+
+    if (strcasecmp(colour, "orange") == 0) {
+        return ORANGE_BAR_FG;
+    }
+
+    if (strcasecmp(colour, "pink") == 0) {
+        return PINK_BAR_FG;
+    }
+
+    if (strcasecmp(colour, "brown") == 0) {
+        return BROWN_BAR_FG;
+    }
+
+    return -1;
+}
