@@ -1,7 +1,7 @@
 /*  name_lookup.c
  *
  *
- *  Copyright (C) 2015 Toxic All Rights Reserved.
+ *  Copyright (C) 2024 Toxic All Rights Reserved.
  *
  *  This file is part of Toxic.
  *
@@ -20,6 +20,8 @@
  *
  */
 
+#include "name_lookup.h"
+
 #include <curl/curl.h>
 #include <stdarg.h>
 #include <stdlib.h>
@@ -30,6 +32,7 @@
 #include "global_commands.h"
 #include "line_info.h"
 #include "misc_tools.h"
+#include "run_options.h"
 #include "toxic.h"
 #include "windows.h"
 
@@ -46,7 +49,6 @@ static struct Nameservers {
 } Nameservers;
 
 static struct thread_data {
-    Tox       *m;
     ToxWindow *self;
     char    id_bin[TOX_ADDRESS_SIZE];
     char    addr[MAX_STR_SIZE];
@@ -67,7 +69,7 @@ static void clear_thread_data(void)
     };
 }
 
-static int lookup_error(ToxWindow *self, const char *errmsg, ...)
+static int lookup_error(ToxWindow *self, const Client_Config *c_config, const char *errmsg, ...)
 {
     char frmt_msg[MAX_STR_SIZE];
 
@@ -77,19 +79,12 @@ static int lookup_error(ToxWindow *self, const char *errmsg, ...)
     va_end(args);
 
     pthread_mutex_lock(&Winthread.lock);
-    line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, "name lookup failed: %s", frmt_msg);
+    line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "name lookup failed: %s", frmt_msg);
     pthread_mutex_unlock(&Winthread.lock);
 
     return -1;
 }
 
-static void kill_lookup_thread(void)
-{
-    clear_thread_data();
-    pthread_attr_destroy(&lookup_thread.attr);
-    pthread_exit(NULL);
-}
-
 /* Attempts to load the nameserver list pointed at by path into the Nameservers structure.
  *
  * Returns 0 on success.
@@ -238,9 +233,20 @@ static int process_response(struct Recv_Curl_Data *recv_data)
     return 0;
 }
 
-void *lookup_thread_func(void *data)
+static void *lookup_thread_func(void *data)
 {
-    UNUSED_VAR(data);
+    struct curl_slist *headers = NULL;
+    CURL *c_handle = NULL;
+    struct Recv_Curl_Data *recv_data = NULL;
+
+    Toxic *toxic = (Toxic *) data;
+
+    if (toxic == NULL) {
+        goto on_exit;
+    }
+
+    const Client_Config *c_config = toxic->c_config;
+    const Run_Options *run_opts = toxic->run_opts;
 
     ToxWindow *self = t_data.self;
 
@@ -248,43 +254,36 @@ void *lookup_thread_func(void *data)
     char name[MAX_STR_SIZE];
 
     if (parse_addr(t_data.addr, name, sizeof(name), input_domain, sizeof(input_domain)) == -1) {
-        lookup_error(self, "Input must be a 76 character Tox ID or an address in the form: username@domain");
-        kill_lookup_thread();
+        lookup_error(self, c_config, "Input must be a 76 character Tox ID or an address in the form: username@domain");
+        goto on_exit;
     }
 
     char nameserver_key[SERVER_KEY_SIZE];
     char real_domain[MAX_DOMAIN_SIZE];
 
     if (!get_domain_match(nameserver_key, real_domain, sizeof(real_domain), input_domain)) {
-        if (!strcasecmp(input_domain, "utox.org")) {
-            lookup_error(self, "utox.org uses deprecated DNS-based lookups and is no longer supported by Toxic.");
-        } else {
-            lookup_error(self, "Name server domain not found.");
-        }
-
-        kill_lookup_thread();
+        lookup_error(self, c_config, "Name server domain not found.");
+        goto on_exit;
     }
 
-    CURL *c_handle = curl_easy_init();
+    c_handle = curl_easy_init();
 
-    if (!c_handle) {
-        lookup_error(self, "curl handler error");
-        kill_lookup_thread();
+    if (c_handle == NULL) {
+        lookup_error(self, c_config, "curl handler error");
+        goto on_exit;
     }
 
-    struct Recv_Curl_Data *recv_data = calloc(1, sizeof(struct Recv_Curl_Data));
+    recv_data = calloc(1, sizeof(struct Recv_Curl_Data));
 
     if (recv_data == NULL) {
-        lookup_error(self, "memory allocation error");
-        kill_lookup_thread();
+        lookup_error(self, c_config, "memory allocation error");
+        goto on_exit;
     }
 
     char post_data[MAX_STR_SIZE + 30];
 
     snprintf(post_data, sizeof(post_data), "{\"action\": 3, \"name\": \"%s\"}", name);
 
-    struct curl_slist *headers = NULL;
-
     headers = curl_slist_append(headers, "Content-Type: application/json");
 
     headers = curl_slist_append(headers, "charsets: utf-8");
@@ -292,70 +291,70 @@ void *lookup_thread_func(void *data)
     int ret = curl_easy_setopt(c_handle, CURLOPT_HTTPHEADER, headers);
 
     if (ret != CURLE_OK) {
-        lookup_error(self, "Failed to set http headers (libcurl error %d)", ret);
+        lookup_error(self, c_config, "Failed to set http headers (libcurl error %d)", ret);
         goto on_exit;
     }
 
     ret = curl_easy_setopt(c_handle, CURLOPT_URL, real_domain);
 
     if (ret != CURLE_OK) {
-        lookup_error(self, "Failed to set url (libcurl error %d)", ret);
+        lookup_error(self, c_config, "Failed to set url (libcurl error %d)", ret);
         goto on_exit;
     }
 
     ret = curl_easy_setopt(c_handle, CURLOPT_WRITEFUNCTION, curl_cb_write_data);
 
     if (ret != CURLE_OK) {
-        lookup_error(self, "Failed to set write function callback (libcurl error %d)", ret);
+        lookup_error(self, c_config, "Failed to set write function callback (libcurl error %d)", ret);
         goto on_exit;
     }
 
     ret = curl_easy_setopt(c_handle, CURLOPT_WRITEDATA, recv_data);
 
     if (ret != CURLE_OK) {
-        lookup_error(self, "Failed to set write data (libcurl error %d)", ret);
+        lookup_error(self, c_config, "Failed to set write data (libcurl error %d)", ret);
         goto on_exit;
     }
 
     ret = curl_easy_setopt(c_handle, CURLOPT_USERAGENT, "libcurl-agent/1.0");
 
     if (ret != CURLE_OK) {
-        lookup_error(self, "Failed to set useragent (libcurl error %d)", ret);
+        lookup_error(self, c_config, "Failed to set useragent (libcurl error %d)", ret);
         goto on_exit;
     }
 
     ret = curl_easy_setopt(c_handle, CURLOPT_POSTFIELDS, post_data);
 
     if (ret != CURLE_OK) {
-        lookup_error(self, "Failed to set post data (libcurl error %d)", ret);
+        lookup_error(self, c_config, "Failed to set post data (libcurl error %d)", ret);
         goto on_exit;
     }
 
-    int proxy_ret = set_curl_proxy(c_handle, arg_opts.proxy_address, arg_opts.proxy_port, arg_opts.proxy_type);
+    int proxy_ret = set_curl_proxy(c_handle, run_opts->proxy_address, run_opts->proxy_port, run_opts->proxy_type);
 
     if (proxy_ret != 0) {
-        lookup_error(self, "Failed to set proxy (error %d)\n", proxy_ret);
+        lookup_error(self, c_config, "Failed to set proxy (error %d)\n", proxy_ret);
         goto on_exit;
     }
 
     ret = curl_easy_setopt(c_handle, CURLOPT_USE_SSL, CURLUSESSL_ALL);
 
     if (ret != CURLE_OK) {
-        lookup_error(self, "TLS could not be enabled (libcurl error %d)", ret);
+        lookup_error(self, c_config, "TLS could not be enabled (libcurl error %d)", ret);
         goto on_exit;
     }
 
     ret = curl_easy_setopt(c_handle, CURLOPT_SSLVERSION, CURL_SSLVERSION_TLSv1_2);
 
     if (ret != CURLE_OK) {
-        lookup_error(self, "TLSv1.2 could not be set (libcurl error %d)", ret);
+        lookup_error(self, c_config, "TLSv1.2 could not be set (libcurl error %d)", ret);
         goto on_exit;
     }
 
     ret = curl_easy_setopt(c_handle, CURLOPT_SSL_CIPHER_LIST, TLS_CIPHER_SUITE_LIST);
 
     if (ret != CURLE_OK) {
-        lookup_error(self, "Failed to set TLS cipher list (libcurl error %d)", ret);
+        lookup_error(self, c_config, "Failed to set TLS cipher list (libcurl error %d)", ret);
         goto on_exit;
     }
 
@@ -367,7 +366,7 @@ void *lookup_thread_func(void *data)
             ret = curl_easy_setopt(c_handle, CURLOPT_SSL_CIPHER_LIST, NULL);
 
             if (ret != CURLE_OK) {
-                lookup_error(self, "Failed to set TLS cipher list (libcurl error %d)", ret);
+                lookup_error(self, c_config, "Failed to set TLS cipher list (libcurl error %d)", ret);
                 goto on_exit;
             }
 
@@ -375,42 +374,46 @@ void *lookup_thread_func(void *data)
         }
 
         if (ret != CURLE_OK) {
-            lookup_error(self, "HTTPS lookup error (libcurl error %d)", ret);
+            lookup_error(self, c_config, "HTTPS lookup error (libcurl error %d)", ret);
             goto on_exit;
         }
     }
 
     if (process_response(recv_data) == -1) {
-        lookup_error(self, "Bad response.");
+        lookup_error(self, c_config, "Bad response.");
         goto on_exit;
     }
 
     pthread_mutex_lock(&Winthread.lock);
-    cmd_add_helper(self, t_data.m, t_data.id_bin, t_data.msg);
+    cmd_add_helper(self, toxic, t_data.id_bin, t_data.msg);
     pthread_mutex_unlock(&Winthread.lock);
 
 on_exit:
-    free(recv_data);
-    curl_slist_free_all(headers);
-    curl_easy_cleanup(c_handle);
-    kill_lookup_thread();
 
-    return 0;
+    free(recv_data);
+    curl_slist_free_all(headers);  // passing null has no effect
+    curl_easy_cleanup(c_handle);  // passing null has no effect
+    clear_thread_data();
+    pthread_attr_destroy(&lookup_thread.attr);
+    pthread_exit(NULL);
 }
 
 /* Attempts to do a tox name lookup.
  *
  * Returns true on success.
  */
-bool name_lookup(ToxWindow *self, Tox *m, const char *id_bin, const char *addr, const char *message)
+bool name_lookup(ToxWindow *self, Toxic *toxic, const char *id_bin, const char *addr, const char *message)
 {
+    const Client_Config *c_config = toxic->c_config;
+
     if (t_data.disabled) {
-        line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, "nameservers list is empty or does not exist.");
+        line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "nameservers list is empty or does not exist.");
         return false;
     }
 
     if (t_data.busy) {
-        line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, "Please wait for previous name lookup to finish.");
+        line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0,
+                      "Please wait for previous name lookup to finish.");
         return false;
     }
 
@@ -418,24 +421,23 @@ bool name_lookup(ToxWindow *self, Tox *m, const char *id_bin, const char *addr,
     snprintf(t_data.addr, sizeof(t_data.addr), "%s", addr);
     snprintf(t_data.msg, sizeof(t_data.msg), "%s", message);
     t_data.self = self;
-    t_data.m = m;
     t_data.busy = true;
 
     if (pthread_attr_init(&lookup_thread.attr) != 0) {
-        line_info_add(self, false, NULL, NULL, SYS_MSG, 0, RED, "Error: lookup thread attr failed to init");
+        line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, RED, "Error: lookup thread attr failed to init");
         clear_thread_data();
         return false;
     }
 
     if (pthread_attr_setdetachstate(&lookup_thread.attr, PTHREAD_CREATE_DETACHED) != 0) {
-        line_info_add(self, false, NULL, NULL, SYS_MSG, 0, RED, "Error: lookup thread attr failed to set");
+        line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, RED, "Error: lookup thread attr failed to set");
         pthread_attr_destroy(&lookup_thread.attr);
         clear_thread_data();
         return false;
     }
 
-    if (pthread_create(&lookup_thread.tid, &lookup_thread.attr, lookup_thread_func, NULL) != 0) {
-        line_info_add(self, false, NULL, NULL, SYS_MSG, 0, RED, "Error: lookup thread failed to init");
+    if (pthread_create(&lookup_thread.tid, &lookup_thread.attr, lookup_thread_func, (void *) toxic) != 0) {
+        line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, RED, "Error: lookup thread failed to init");
         pthread_attr_destroy(&lookup_thread.attr);
         clear_thread_data();
         return false;
@@ -444,23 +446,15 @@ bool name_lookup(ToxWindow *self, Tox *m, const char *id_bin, const char *addr,
     return true;
 }
 
-/* Initializes http based name lookups. Note: This function must be called only once before additional
- * threads are spawned.
- *
- * Returns 0 on success.
- * Returns -1 if curl failed to init.
- * Returns -2 if the nameserver list cannot be found.
- * Returns -3 if the nameserver list does not contain any valid entries.
- */
-int name_lookup_init(int curl_init_status)
+int name_lookup_init(const char *nameserver_path, int curl_init_status)
 {
     if (curl_init_status != 0) {
         t_data.disabled = true;
         return -1;
     }
 
-    const char *path = arg_opts.nameserver_path[0] ? arg_opts.nameserver_path : PACKAGE_DATADIR "/nameservers";
-    int ret = load_nameserver_list(path);
+    const char *path = !string_is_empty(nameserver_path) ? nameserver_path : PACKAGE_DATADIR "/nameservers";
+    const int ret = load_nameserver_list(path);
 
     if (ret != 0) {
         t_data.disabled = true;
