@@ -1,7 +1,7 @@
 /*  prompt.c
  *
  *
- *  Copyright (C) 2014 Toxic All Rights Reserved.
+ *  Copyright (C) 2024 Toxic All Rights Reserved.
  *
  *  This file is part of Toxic.
  *
@@ -43,8 +43,6 @@
 #include "toxic_strings.h"
 #include "windows.h"
 
-extern ToxWindow *prompt;
-extern struct user_settings *user_settings;
 extern struct Winthread Winthread;
 
 extern FriendsList Friends;
@@ -56,6 +54,7 @@ static const char *glob_cmd_list[] = {
     "/add",
     "/avatar",
     "/clear",
+    "/color",
     "/connect",
     "/decline",
     "/exit",
@@ -100,72 +99,83 @@ static const char *glob_cmd_list[] = {
 
 };
 
-void kill_prompt_window(ToxWindow *self)
+void kill_prompt_window(ToxWindow *self, const Client_Config *c_config)
 {
     ChatContext *ctx = self->chatwin;
     StatusBar *statusbar = self->stb;
 
-    log_disable(ctx->log);
-    line_info_cleanup(ctx->hst);
+    if (ctx != NULL)  {
+        log_disable(ctx->log);
+        line_info_cleanup(ctx->hst);
+
+        delwin(ctx->linewin);
+        delwin(ctx->history);
+        free(ctx->log);
+        free(ctx);
+    }
 
-    delwin(ctx->linewin);
-    delwin(ctx->history);
     delwin(statusbar->topline);
-
-    free(ctx->log);
-    free(ctx);
     free(self->help);
     free(statusbar);
 
-    del_window(self);
+    del_window(self, c_config);
 }
 
 /* callback: Updates own connection status in prompt statusbar */
-void on_self_connection_status(Tox *m, Tox_Connection connection_status, void *userdata)
+void on_self_connection_status(Tox *tox, Tox_Connection connection_status, void *userdata)
 {
-    UNUSED_VAR(m);
-    UNUSED_VAR(userdata);
-    StatusBar *statusbar = prompt->stb;
+    UNUSED_VAR(tox);
+
+    Toxic *toxic = (Toxic *) userdata;
+
+    if (toxic == NULL) {
+        return;
+    }
+
+    StatusBar *statusbar = toxic->home_window->stb;
     statusbar->connection = connection_status;
 
     flag_interface_refresh();
 }
 
 /* Updates own nick in prompt statusbar */
-void prompt_update_nick(ToxWindow *prompt, const char *nick)
+void prompt_update_nick(ToxWindow *self, const char *nick)
 {
-    StatusBar *statusbar = prompt->stb;
+    StatusBar *statusbar = self->stb;
     snprintf(statusbar->nick, sizeof(statusbar->nick), "%s", nick);
     statusbar->nick_len = strlen(statusbar->nick);
 }
 
 /* Updates own statusmessage */
-void prompt_update_statusmessage(ToxWindow *prompt, Tox *m, const char *statusmsg)
+void prompt_update_statusmessage(Toxic *toxic, const char *statusmsg)
 {
-    StatusBar *statusbar = prompt->stb;
+    ToxWindow *self = toxic->home_window;
+    StatusBar *statusbar = self->stb;
+
     snprintf(statusbar->statusmsg, sizeof(statusbar->statusmsg), "%s", statusmsg);
-    size_t len = strlen(statusbar->statusmsg);
+
+    const size_t len = strlen(statusbar->statusmsg);
     statusbar->statusmsg_len = len;
 
     Tox_Err_Set_Info err;
-    tox_self_set_status_message(m, (const uint8_t *) statusmsg, len, &err);
+    tox_self_set_status_message(toxic->tox, (const uint8_t *) statusmsg, len, &err);
 
     if (err != TOX_ERR_SET_INFO_OK) {
-        line_info_add(prompt, false, NULL, NULL, SYS_MSG, 0, 0, "Failed to set note (error %d)\n", err);
+        line_info_add(self, toxic->c_config, false, NULL, NULL, SYS_MSG, 0, 0, "Failed to set note (error %d)\n", err);
     }
 }
 
 /* Updates own status in prompt statusbar */
-void prompt_update_status(ToxWindow *prompt, Tox_User_Status status)
+void prompt_update_status(ToxWindow *self, Tox_User_Status status)
 {
-    StatusBar *statusbar = prompt->stb;
+    StatusBar *statusbar = self->stb;
     statusbar->status = status;
 }
 
 /* Returns our own connection status */
-Tox_Connection prompt_selfConnectionStatus(void)
+Tox_Connection prompt_selfConnectionStatus(Toxic *toxic)
 {
-    StatusBar *statusbar = prompt->stb;
+    StatusBar *statusbar = toxic->home_window->stb;
     return statusbar->connection;
 }
 
@@ -199,8 +209,13 @@ static int add_friend_request(const char *public_key, const char *data)
 /*
  * Return true if input is recognized by handler
  */
-static bool prompt_onKey(ToxWindow *self, Tox *m, wint_t key, bool ltr)
+static bool prompt_onKey(ToxWindow *self, Toxic *toxic, wint_t key, bool ltr)
 {
+    if (toxic == NULL || self == NULL) {
+        return false;
+    }
+
+    const Client_Config *c_config = toxic->c_config;
     ChatContext *ctx = self->chatwin;
 
     int x, y, y2, x2;
@@ -224,15 +239,15 @@ static bool prompt_onKey(ToxWindow *self, Tox *m, wint_t key, bool ltr)
     }
 
     if (ltr || key == '\n') {    /* char is printable */
-        input_new_char(self, key, x, x2);
+        input_new_char(self, c_config, key, x, x2);
         return true;
     }
 
-    if (line_info_onKey(self, key)) {
+    if (line_info_onKey(self, c_config, key)) {
         return true;
     }
 
-    if (input_handle(self, key, x, x2)) {
+    if (input_handle(self, c_config, key, x, x2)) {
         return true;
     }
 
@@ -245,25 +260,18 @@ static bool prompt_onKey(ToxWindow *self, Tox *m, wint_t key, bool ltr)
             int diff = -1;
 
             if (wcsncmp(ctx->line, L"/avatar ", wcslen(L"/avatar ")) == 0) {
-                diff = dir_match(self, m, ctx->line, L"/avatar");
+                diff = dir_match(self, toxic, ctx->line, L"/avatar");
             }
 
 #ifdef PYTHON
             else if (wcsncmp(ctx->line, L"/run ", wcslen(L"/run ")) == 0) {
-                diff = dir_match(self, m, ctx->line, L"/run");
+                diff = dir_match(self, toxic, ctx->line, L"/run");
             }
 
 #endif
 
-            else if (wcsncmp(ctx->line, L"/status ", wcslen(L"/status ")) == 0) {
-                const char *status_cmd_list[] = {
-                    "online",
-                    "away",
-                    "busy",
-                };
-                diff = complete_line(self, status_cmd_list, sizeof(status_cmd_list) / sizeof(char *));
-            } else {
-                diff = complete_line(self, glob_cmd_list, sizeof(glob_cmd_list) / sizeof(char *));
+            else {
+                diff = complete_line(self, toxic, glob_cmd_list, sizeof(glob_cmd_list) / sizeof(char *));
             }
 
             if (diff != -1) {
@@ -272,10 +280,10 @@ static bool prompt_onKey(ToxWindow *self, Tox *m, wint_t key, bool ltr)
                     ctx->start = wlen < x2 ? 0 : wlen - x2 + 1;
                 }
             } else {
-                sound_notify(self, notif_error, 0, NULL);
+                sound_notify(self, c_config, notif_error, 0, NULL);
             }
         } else {
-            sound_notify(self, notif_error, 0, NULL);
+            sound_notify(self, c_config, notif_error, 0, NULL);
         }
     } else if (key == '\r') {
         input_ret = true;
@@ -289,13 +297,13 @@ static bool prompt_onKey(ToxWindow *self, Tox *m, wint_t key, bool ltr)
             char line[MAX_STR_SIZE];
 
             if (wcs_to_mbs_buf(line, ctx->line, MAX_STR_SIZE) == -1) {
-                line_info_add(self, false, NULL, NULL, SYS_MSG, 0, RED, " * Failed to parse message.");
+                line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, RED, " * Failed to parse message.");
             } else {
                 if (strcmp(line, "/clear") != 0) {
-                    line_info_add(self, false, NULL, NULL, PROMPT, 0, 0, "%s", line);
+                    line_info_add(self, c_config, false, NULL, NULL, PROMPT, 0, 0, "%s", line);
                 }
 
-                execute(ctx->history, self, m, line, GLOBAL_COMMAND_MODE);
+                execute(ctx->history, self, toxic, line, GLOBAL_COMMAND_MODE);
             }
         }
 
@@ -307,8 +315,13 @@ static bool prompt_onKey(ToxWindow *self, Tox *m, wint_t key, bool ltr)
     return input_ret;
 }
 
-static void prompt_onDraw(ToxWindow *self, Tox *m)
+static void prompt_onDraw(ToxWindow *self, Toxic *toxic)
 {
+    if (toxic == NULL || self == NULL) {
+        fprintf(stderr, "prompt_onDraw null param\n");
+        return;
+    }
+
     int x2;
     int y2;
     getmaxyx(self->window, y2, x2);
@@ -320,7 +333,7 @@ static void prompt_onDraw(ToxWindow *self, Tox *m)
     ChatContext *ctx = self->chatwin;
 
     pthread_mutex_lock(&Winthread.lock);
-    line_info_print(self);
+    line_info_print(self, toxic->c_config);
     pthread_mutex_unlock(&Winthread.lock);
 
     wclear(ctx->linewin);
@@ -425,8 +438,8 @@ static void prompt_onDraw(ToxWindow *self, Tox *m)
 
         pthread_mutex_lock(&Winthread.lock);
 
-        size_t slen = tox_self_get_status_message_size(m);
-        tox_self_get_status_message(m, (uint8_t *) statusmsg);
+        const size_t slen = tox_self_get_status_message_size(toxic->tox);
+        tox_self_get_status_message(toxic->tox, (uint8_t *) statusmsg);
 
         statusmsg[slen] = '\0';
         snprintf(statusbar->statusmsg, sizeof(statusbar->statusmsg), "%s", statusmsg);
@@ -478,16 +491,22 @@ static void prompt_onDraw(ToxWindow *self, Tox *m)
     wnoutrefresh(self->window);
 
     if (self->help->active) {
-        help_onDraw(self);
+        help_draw_main(self);
     }
 }
 
-static void prompt_onConnectionChange(ToxWindow *self, Tox *m, uint32_t friendnum, Tox_Connection connection_status)
+static void prompt_onConnectionChange(ToxWindow *self, Toxic *toxic, uint32_t friendnum,
+                                      Tox_Connection connection_status)
 {
+    if (toxic == NULL || self == NULL) {
+        return;
+    }
+
+    const Client_Config *c_config = toxic->c_config;
     ChatContext *ctx = self->chatwin;
 
     char nick[TOX_MAX_NAME_LENGTH] = {0};    /* stop removing this initiation */
-    get_nick_truncate(m, nick, friendnum);
+    get_nick_truncate(toxic->tox, nick, friendnum);
 
     if (!nick[0]) {
         snprintf(nick, sizeof(nick), "%s", UNKNOWN_NAME);
@@ -495,32 +514,32 @@ static void prompt_onConnectionChange(ToxWindow *self, Tox *m, uint32_t friendnu
 
     const char *msg;
 
-    if (user_settings->show_connection_msg == SHOW_WELCOME_MSG_OFF) {
+    if (!friend_config_get_show_connection_msg(friendnum)) {
         return;
     }
 
     if (connection_status != TOX_CONNECTION_NONE && Friends.list[friendnum].connection_status == TOX_CONNECTION_NONE) {
         msg = "has come online";
-        line_info_add(self, true, nick, NULL, CONNECTION, 0, GREEN, msg);
-        write_to_log(msg, nick, ctx->log, true);
+        line_info_add(self, c_config, true, nick, NULL, CONNECTION, 0, GREEN, msg);
+        write_to_log(ctx->log, c_config, msg, nick, true);
 
         if (self->active_box != -1) {
-            box_notify2(self, user_log_in, NT_WNDALERT_2 | NT_NOTIFWND | NT_RESTOL, self->active_box,
+            box_notify2(self, c_config, user_log_in, NT_WNDALERT_2 | NT_NOTIFWND | NT_RESTOL, self->active_box,
                         "%s has come online", nick);
         } else {
-            box_notify(self, user_log_in, NT_WNDALERT_2 | NT_NOTIFWND | NT_RESTOL, &self->active_box,
+            box_notify(self, c_config, user_log_in, NT_WNDALERT_2 | NT_NOTIFWND | NT_RESTOL, &self->active_box,
                        "Toxic", "%s has come online", nick);
         }
     } else if (connection_status == TOX_CONNECTION_NONE) {
         msg = "has gone offline";
-        line_info_add(self, true, nick, NULL, DISCONNECTION, 0, RED, msg);
-        write_to_log(msg, nick, ctx->log, true);
+        line_info_add(self, c_config, true, nick, NULL, DISCONNECTION, 0, RED, msg);
+        write_to_log(ctx->log, c_config, msg, nick, true);
 
         if (self->active_box != -1) {
-            box_notify2(self, user_log_out, NT_WNDALERT_2 | NT_NOTIFWND | NT_RESTOL, self->active_box,
+            box_notify2(self, c_config, user_log_out, NT_WNDALERT_2 | NT_NOTIFWND | NT_RESTOL, self->active_box,
                         "%s has gone offline", nick);
         } else {
-            box_notify(self, user_log_out, NT_WNDALERT_2 | NT_NOTIFWND | NT_RESTOL, &self->active_box,
+            box_notify(self, c_config, user_log_out, NT_WNDALERT_2 | NT_NOTIFWND | NT_RESTOL, &self->active_box,
                        "Toxic", "%s has gone offline", nick);
         }
     }
@@ -546,37 +565,47 @@ static bool key_is_similar(const char *key)
     return false;
 }
 
-static void prompt_onFriendRequest(ToxWindow *self, Tox *m, const char *key, const char *data, size_t length)
+static void prompt_onFriendRequest(ToxWindow *self, Toxic *toxic, const char *key, const char *data, size_t length)
 {
-    UNUSED_VAR(m);
     UNUSED_VAR(length);
 
-    ChatContext *ctx = self->chatwin;
-
-    line_info_add(self, true, NULL, NULL, SYS_MSG, 0, 0, "Friend request with the message '%s'", data);
-    write_to_log("Friend request with the message '%s'", "", ctx->log, true);
-
-    if (key_is_similar(key)) {
-        line_info_add(self, false, NULL, NULL, SYS_MSG, 0, RED,
-                      "WARNING: This contact's public key is suspiciously similar to that of another contact ");
-        line_info_add(self, false, NULL, NULL, SYS_MSG, 0, RED,
-                      "in your list. This may be an impersonation attempt, or it may have occurred by chance.");
-    }
-
-    int n = add_friend_request(key, data);
-
-    if (n == -1) {
-        const char *errmsg = "Friend request queue is full. Discarding request.";
-        line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, errmsg);
+    if (toxic == NULL || self == NULL) {
         return;
     }
 
-    line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, "Type \"/accept %d\" or \"/decline %d\"", n, n);
-    sound_notify(self, generic_message, NT_WNDALERT_1 | NT_NOTIFWND, NULL);
+    const Client_Config *c_config = toxic->c_config;
+    ChatContext *ctx = self->chatwin;
+
+    line_info_add(self, c_config, true, NULL, NULL, SYS_MSG, 0, 0, "Friend request with the message '%s'", data);
+    write_to_log(ctx->log, c_config, "Friend request with the message '%s'", "", true);
+
+    if (key_is_similar(key)) {
+        line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, RED,
+                      "WARNING: This contact's public key is suspiciously similar to that of another contact ");
+        line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, RED,
+                      "in your list. This may be an impersonation attempt, or it may have occurred by chance.");
+    }
+
+    const int n = add_friend_request(key, data);
+
+    if (n == -1) {
+        const char *errmsg = "Friend request queue is full. Discarding request.";
+        line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, errmsg);
+        return;
+    }
+
+    line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "Type \"/accept %d\" or \"/decline %d\"", n, n);
+    sound_notify(self, c_config, generic_message, NT_WNDALERT_1 | NT_NOTIFWND, NULL);
 }
 
-void prompt_init_statusbar(ToxWindow *self, Tox *m, bool first_time_run)
+void prompt_init_statusbar(Toxic *toxic, bool first_time_run)
 {
+    ToxWindow *self = toxic->home_window;
+
+    if (self == NULL) {
+        exit_toxic_err("failed in prompt_init_statusbar", FATALERR_WININIT);
+    }
+
     int x2, y2;
     getmaxyx(self->window, y2, x2);
 
@@ -586,6 +615,8 @@ void prompt_init_statusbar(ToxWindow *self, Tox *m, bool first_time_run)
 
     UNUSED_VAR(y2);
 
+    Tox *tox = toxic->tox;
+
     /* Init statusbar info */
     StatusBar *statusbar = self->stb;
     statusbar->status = TOX_USER_STATUS_NONE;
@@ -594,13 +625,13 @@ void prompt_init_statusbar(ToxWindow *self, Tox *m, bool first_time_run)
     char nick[TOX_MAX_NAME_LENGTH];
     char statusmsg[TOX_MAX_STATUS_MESSAGE_LENGTH];
 
-    size_t n_len = tox_self_get_name_size(m);
-    tox_self_get_name(m, (uint8_t *) nick);
+    size_t n_len = tox_self_get_name_size(tox);
+    tox_self_get_name(tox, (uint8_t *) nick);
 
-    size_t s_len = tox_self_get_status_message_size(m);
-    tox_self_get_status_message(m, (uint8_t *) statusmsg);
+    size_t s_len = tox_self_get_status_message_size(tox);
+    tox_self_get_status_message(tox, (uint8_t *) statusmsg);
 
-    Tox_User_Status status = tox_self_get_status(m);
+    Tox_User_Status status = tox_self_get_status(tox);
 
     nick[n_len] = '\0';
     statusmsg[s_len] = '\0';
@@ -611,53 +642,60 @@ void prompt_init_statusbar(ToxWindow *self, Tox *m, bool first_time_run)
         statusmsg[s_len] = '\0';
     }
 
-    prompt_update_statusmessage(prompt, m, statusmsg);
-    prompt_update_status(prompt, status);
-    prompt_update_nick(prompt, nick);
+    prompt_update_statusmessage(toxic, statusmsg);
+    prompt_update_status(toxic->home_window, status);
+    prompt_update_nick(toxic->home_window, nick);
 
     /* Init statusbar subwindow */
     statusbar->topline = subwin(self->window, TOP_BAR_HEIGHT, x2, 0, 0);
 }
 
-static void print_welcome_msg(ToxWindow *self)
+static void print_welcome_msg(ToxWindow *self, const Client_Config *c_config)
 {
-    line_info_add(self, false, NULL, NULL, SYS_MSG, 1, BLUE, "    _____ _____  _____ ____ ");
-    line_info_add(self, false, NULL, NULL, SYS_MSG, 1, BLUE, "   |_   _/ _ \\ \\/ /_ _/ ___|");
-    line_info_add(self, false, NULL, NULL, SYS_MSG, 1, BLUE, "     | || | | \\  / | | |    ");
-    line_info_add(self, false, NULL, NULL, SYS_MSG, 1, BLUE, "     | || |_| /  \\ | | |___ ");
-    line_info_add(self, false, NULL, NULL, SYS_MSG, 1, BLUE, "     |_| \\___/_/\\_\\___\\____| v." TOXICVER);
-    line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, "");
+    line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 1, BLUE, "    _____ _____  _____ ____ ");
+    line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 1, BLUE, "   |_   _/ _ \\ \\/ /_ _/ ___|");
+    line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 1, BLUE, "     | || | | \\  / | | |    ");
+    line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 1, BLUE, "     | || |_| /  \\ | | |___ ");
+    line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 1, BLUE,
+                  "     |_| \\___/_/\\_\\___\\____| v." TOXICVER);
+    line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "");
 
     const char *msg = "Welcome to Toxic, a free, open source Tox-based instant messaging client.";
-    line_info_add(self, false, NULL, NULL, SYS_MSG, 1, CYAN, msg);
+    line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 1, CYAN, msg);
     msg = "Type \"/help\" for assistance. Further help may be found via the man page.";
-    line_info_add(self, false, NULL, NULL, SYS_MSG, 1, CYAN, msg);
-    line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, "");
+    line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 1, CYAN, msg);
+    line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "");
 }
 
-static void prompt_init_log(ToxWindow *self, Tox *m, const char *self_name)
+static void prompt_init_log(Toxic *toxic)
 {
+    const Client_Config *c_config = toxic->c_config;
+    ToxWindow *self = toxic->home_window;
     ChatContext *ctx = self->chatwin;
 
     char myid[TOX_ADDRESS_SIZE];
-    tox_self_get_address(m, (uint8_t *) myid);
+    tox_self_get_address(toxic->tox, (uint8_t *) myid);
 
-    if (log_init(ctx->log, self->name, myid, NULL, LOG_TYPE_PROMPT) != 0) {
-        line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, "Warning: Log failed to initialize.");
+    if (log_init(ctx->log, toxic->c_config, self->name, myid, NULL, LOG_TYPE_PROMPT) != 0) {
+        line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "Warning: Log failed to initialize.");
         return;
     }
 
-    if (user_settings->autolog == AUTOLOG_ON) {
+    if (toxic->c_config->autolog == AUTOLOG_ON) {
         if (log_enable(ctx->log) == -1) {
-            line_info_add(self, false, NULL, NULL, SYS_MSG, 0, 0, "Warning: Failed to enable log.");
+            line_info_add(self, c_config, false, NULL, NULL, SYS_MSG, 0, 0, "Warning: Failed to enable log.");
         }
     }
 }
 
-static void prompt_onInit(ToxWindow *self, Tox *m)
+static void prompt_onInit(ToxWindow *self, Toxic *toxic)
 {
     curs_set(1);
 
+    if (toxic == NULL || self == NULL) {
+        return;
+    }
+
     int y2;
     int x2;
     getmaxyx(self->window, y2, x2);
@@ -681,13 +719,13 @@ static void prompt_onInit(ToxWindow *self, Tox *m)
 
     line_info_init(ctx->hst);
 
-    prompt_init_log(self, m, self->name);
+    prompt_init_log(toxic);
 
     scrollok(ctx->history, 0);
     wmove(self->window, y2 - CURS_Y_OFFSET, 0);
 
-    if (user_settings->show_welcome_msg == SHOW_WELCOME_MSG_ON) {
-        print_welcome_msg(self);
+    if (toxic->c_config->show_welcome_msg == SHOW_WELCOME_MSG_ON) {
+        print_welcome_msg(toxic->home_window, toxic->c_config);
     }
 }
 
