@@ -20,13 +20,14 @@
  *
  */
 
+#include "api.h"
+
 #ifdef PYTHON
 #include <Python.h>
-#include "api.h"
 
 #include "execute.h"
 
-extern Tox       *user_tox;
+extern Toxic       *user_toxic;
 
 static struct python_registered_func {
     char     *name;
@@ -278,9 +279,9 @@ void terminate_python(void)
     Py_Finalize();
 }
 
-void init_python(Tox *m)
+void init_python(Toxic *toxic)
 {
-    user_tox = m;
+    user_toxic = toxic;
     PyImport_AppendInittab("toxic_api", PyInit_toxic_api);
     Py_Initialize();
 }
