@@ -1,7 +1,7 @@
 /*  settings.c
  *
  *
- *  Copyright (C) 2014 Toxic All Rights Reserved.
+ *  Copyright (C) 2024 Toxic All Rights Reserved.
  *
  *  This file is part of Toxic.
  *
@@ -20,12 +20,16 @@
  *
  */
 
+#include <assert.h>
 #include <ctype.h>
 #include <libconfig.h>
 #include <stdlib.h>
 #include <string.h>
 
+#include "conference.h"
 #include "configdir.h"
+#include "friendlist.h"
+#include "groupchats.h"
 #include "misc_tools.h"
 #include "notify.h"
 #include "toxic.h"
@@ -42,6 +46,11 @@
 #define PACKAGE_DATADIR "."
 #endif
 
+#define TOXIC_CONF_FILE_EXT ".conf"
+#define TOXIC_CONF_FILE_EXT_LENGTH (sizeof(TOXIC_CONF_FILE_EXT) - 1)
+
+static_assert(MAX_STR_SIZE > TOXIC_CONF_FILE_EXT_LENGTH, "MAX_STR_SIZE <= TOXIC_CONF_FILE_EXT_LENGTH");
+
 #define NO_SOUND "silent"
 
 static struct ui_strings {
@@ -119,7 +128,7 @@ static struct ui_strings {
     "color_bar_notify",
 };
 
-static void ui_defaults(struct user_settings *settings)
+static void ui_defaults(Client_Config *settings)
 {
     settings->timestamps = TIMESTAMPS_ON;
     snprintf(settings->timestamp_format, sizeof(settings->timestamp_format), "%s", TIMESTAMP_DEFAULT);
@@ -177,7 +186,7 @@ static const struct keys_strings {
 };
 
 /* defines from toxic.h */
-static void key_defaults(struct user_settings *settings)
+static void key_defaults(Client_Config *settings)
 {
     settings->key_next_tab = T_KEY_NEXT;
     settings->key_prev_tab = T_KEY_PREV;
@@ -206,7 +215,7 @@ static const struct tox_strings {
     "password_eval",
 };
 
-static void tox_defaults(struct user_settings *settings)
+static void tox_defaults(Client_Config *settings)
 {
     strcpy(settings->download_path, "");
     strcpy(settings->chatlogs_path, "");
@@ -234,7 +243,7 @@ static const struct audio_strings {
     "push_to_talk",
 };
 
-static void audio_defaults(struct user_settings *settings)
+static void audio_defaults(Client_Config *settings)
 {
     settings->audio_in_dev = 0;
     settings->audio_out_dev = 0;
@@ -273,6 +282,40 @@ static const struct sound_strings {
 };
 #endif
 
+static const struct friend_strings {
+    const char *self;
+    const char *auto_accept_files;
+    const char *autolog;
+    const char *show_connection_msg;
+    const char *tab_name_color;
+} friend_strings = {
+    "friends",
+    "auto_accept_files",
+    "autolog",
+    "show_connection_msg",
+    "tab_name_color",
+};
+
+static const struct groupchat_strings {
+    const char *self;
+    const char *tab_name_color;
+    const char *autolog;
+} groupchat_strings = {
+    "groupchats",
+    "tab_name_color",
+    "autolog",
+};
+
+static const struct conference_strings {
+    const char *self;
+    const char *tab_name_color;
+    const char *autolog;
+} conference_strings = {
+    "conferences",
+    "tab_name_color",
+    "autolog",
+};
+
 static int key_parse(const char **bind)
 {
     int len = strlen(*bind);
@@ -303,12 +346,287 @@ static void set_key_binding(int *key, const char **bind)
     }
 }
 
-int settings_load(struct user_settings *s, const char *patharg)
+bool settings_load_config_file(Run_Options *run_opts, const char *data_path)
+{
+    char tmp_path[MAX_STR_SIZE] = {0};
+
+    if (run_opts->use_custom_config_file) {
+        snprintf(tmp_path, sizeof(tmp_path), "%s", run_opts->config_path);
+    } else if (run_opts->use_custom_data) {
+        char tmp_data[MAX_STR_SIZE - TOXIC_CONF_FILE_EXT_LENGTH];
+
+        if (strlen(data_path) >= sizeof(tmp_data)) {
+            return false;
+        }
+
+        snprintf(tmp_data, sizeof(tmp_data), "%s", data_path);
+
+        const int dot_idx = char_rfind(tmp_data, '.', strlen(tmp_data));
+
+        if (dot_idx > 0) {
+            tmp_data[dot_idx] = '\0';  // remove .tox file extension (or any others) if it exists
+        }
+
+        snprintf(tmp_path, sizeof(tmp_path), "%s%s", tmp_data, TOXIC_CONF_FILE_EXT);
+    } else {
+        char *user_config_dir = get_user_config_dir();
+        snprintf(tmp_path, sizeof(tmp_path), "%s%stoxic%s", user_config_dir, CONFIGDIR, TOXIC_CONF_FILE_EXT);
+        free(user_config_dir);
+    }
+
+    /* make sure path exists or is created on first time running */
+    if (!file_exists(tmp_path)) {
+        FILE *fp = fopen(tmp_path, "w");
+
+        if (fp == NULL) {
+            fprintf(stderr, "failed to create config path `%s`\n", tmp_path);
+            return false;
+        }
+
+        fclose(fp);
+    }
+
+    snprintf(run_opts->config_path, sizeof(run_opts->config_path), "%s", tmp_path);
+
+    return true;
+}
+
+#define TOXIC_CONFIG_PUBLIC_KEY_PREFIX "pk_"
+
+/* Extracts a public key from `keys`.
+ *
+ * Returns the public key as a string on success.
+ * Returns NULL on error.
+ */
+static const char *extract_setting_public_key(const config_setting_t *keys)
+{
+    if (keys == NULL) {
+        return NULL;
+    }
+
+    const char *public_key = config_setting_name(keys);
+
+    if (public_key == NULL) {
+        fprintf(stderr, "config error: failed to extract public key\n");
+        return NULL;
+    }
+
+    const uint16_t prefix_len = (uint16_t) strlen(TOXIC_CONFIG_PUBLIC_KEY_PREFIX);
+
+    if (strlen(public_key) != TOX_PUBLIC_KEY_SIZE * 2 + prefix_len) {
+        fprintf(stderr, "config error: invalid public key: %s\n", public_key);
+        return NULL;
+    }
+
+    if (memcmp(public_key, TOXIC_CONFIG_PUBLIC_KEY_PREFIX, prefix_len) != 0) {
+        fprintf(stderr, "config error: invalid public key prefix\n");
+        return NULL;
+    }
+
+    return &public_key[prefix_len];
+}
+
+/*
+ * Initializes `cfg` with the contents from the toxic config file.
+ *
+ * Return 0 on success.
+ * Return -1 if the config file was not set by the client.
+ * Return -2 if the config file cannot be read or is invalid.
+ */
+static int settings_init_config(config_t *cfg, const Run_Options *run_opts)
+{
+    if (string_is_empty(run_opts->config_path)) {
+        return -1;
+    }
+
+    if (!config_read_file(cfg, run_opts->config_path)) {
+        fprintf(stderr, "config_read_file() error: %s:%d - %s\n", config_error_file(cfg),
+                config_error_line(cfg), config_error_text(cfg));
+        return -2;
+    }
+
+    return 0;
+}
+
+int settings_load_conferences(const Run_Options *run_opts)
 {
     config_t cfg[1];
-    config_setting_t *setting;
+    config_init(cfg);
+
+    const int c_ret = settings_init_config(cfg, run_opts);
+
+    if (c_ret < 0) {
+        config_destroy(cfg);
+        return c_ret;
+    }
+
     const char *str = NULL;
 
+    const config_setting_t *setting = config_lookup(cfg, conference_strings.self);
+
+    if (setting == NULL) {
+        config_destroy(cfg);
+        return 0;
+    }
+
+    const int num_conferences = config_setting_length(setting);
+
+    for (int i = 0; i < num_conferences; ++i) {
+        const config_setting_t *keys = config_setting_get_elem(setting, i);
+
+        const char *public_key = extract_setting_public_key(keys);
+
+        if (public_key == NULL) {
+            continue;
+        }
+
+        if (config_setting_lookup_string(keys, conference_strings.tab_name_color, &str)) {
+            if (!conference_config_set_tab_name_colour(public_key, str)) {
+                fprintf(stderr, "config error: failed to set conference tab name color for %s: (color: %s)\n", public_key, str);
+            }
+        }
+
+        int autolog_enabled;
+
+        if (config_setting_lookup_bool(keys, conference_strings.autolog, &autolog_enabled)) {
+            if (!conference_config_set_autolog(public_key, autolog_enabled != 0)) {
+                fprintf(stderr, "config error: failed to apply conference autolog setting for %s\n", public_key);
+            }
+        }
+    }
+
+    config_destroy(cfg);
+
+    return 0;
+}
+
+int settings_load_groups(const Run_Options *run_opts)
+{
+    config_t cfg[1];
+    config_init(cfg);
+
+    const int c_ret = settings_init_config(cfg, run_opts);
+
+    if (c_ret < 0) {
+        config_destroy(cfg);
+        return c_ret;
+    }
+
+    const char *str = NULL;
+
+    const config_setting_t *setting = config_lookup(cfg, groupchat_strings.self);
+
+    if (setting == NULL) {
+        config_destroy(cfg);
+        return 0;
+    }
+
+    const int num_groups = config_setting_length(setting);
+
+    for (int i = 0; i < num_groups; ++i) {
+        const config_setting_t *keys = config_setting_get_elem(setting, i);
+
+        const char *public_key = extract_setting_public_key(keys);
+
+        if (public_key == NULL) {
+            continue;
+        }
+
+        if (config_setting_lookup_string(keys, groupchat_strings.tab_name_color, &str)) {
+            if (!groupchat_config_set_tab_name_colour(public_key, str)) {
+                fprintf(stderr, "config error: failed to set groupchat tab name color for %s: (color: %s)\n", public_key, str);
+            }
+        }
+
+        int autolog_enabled;
+
+        if (config_setting_lookup_bool(keys, groupchat_strings.autolog, &autolog_enabled)) {
+            if (!groupchat_config_set_autolog(public_key, autolog_enabled != 0)) {
+                fprintf(stderr, "config error: failed to apply groupchat autolog setting for %s\n", public_key);
+            }
+        }
+    }
+
+    config_destroy(cfg);
+
+    return 0;
+}
+
+int settings_load_friends(const Run_Options *run_opts)
+{
+    config_t cfg[1];
+    config_init(cfg);
+
+    const int c_ret = settings_init_config(cfg, run_opts);
+
+    if (c_ret < 0) {
+        config_destroy(cfg);
+        return c_ret;
+    }
+
+    const char *str = NULL;
+
+    const config_setting_t *setting = config_lookup(cfg, friend_strings.self);
+
+    if (setting == NULL) {
+        config_destroy(cfg);
+        return 0;
+    }
+
+    const int num_friends = config_setting_length(setting);
+
+    for (int i = 0; i < num_friends; ++i) {
+        const config_setting_t *keys = config_setting_get_elem(setting, i);
+
+        const char *public_key = extract_setting_public_key(keys);
+
+        if (public_key == NULL) {
+            continue;
+        }
+
+        if (config_setting_lookup_string(keys, friend_strings.tab_name_color, &str)) {
+            if (!friend_config_set_tab_name_colour(public_key, str)) {
+                fprintf(stderr, "config error: failed to set friend tab name color for %s: (color: %s)\n", public_key, str);
+            }
+        }
+
+        int autolog_enabled;
+
+        if (config_setting_lookup_bool(keys, friend_strings.autolog, &autolog_enabled)) {
+            if (!friend_config_set_autolog(public_key, autolog_enabled != 0)) {
+                fprintf(stderr, "config error: failed to apply friend autolog setting for: %s\n", public_key);
+            }
+        }
+
+        int auto_accept_files;
+
+        if (config_setting_lookup_bool(keys, friend_strings.auto_accept_files, &auto_accept_files)) {
+            if (!friend_config_set_auto_accept_files(public_key, auto_accept_files != 0)) {
+                fprintf(stderr, "config error: failed to apply friend auto-accept filetransfers setting for: %s\n", public_key);
+            }
+        }
+
+        int show_connection_msg;
+
+        if (config_setting_lookup_bool(keys, friend_strings.show_connection_msg, &show_connection_msg)) {
+            if (!friend_config_set_show_connection_msg(public_key, show_connection_msg != 0)) {
+                fprintf(stderr, "config error: failed to apply friend show connection message setting for: %s\n", public_key);
+            }
+        }
+    }
+
+    config_destroy(cfg);
+
+    return 0;
+}
+
+int settings_load_main(Client_Config *s, const Run_Options *run_opts)
+{
+    config_t cfg[1];
+    config_init(cfg);
+
+    config_setting_t *setting = NULL;
+
     /* Load default settings */
     ui_defaults(s);
     tox_defaults(s);
@@ -318,35 +636,15 @@ int settings_load(struct user_settings *s, const char *patharg)
     audio_defaults(s);
 #endif
 
-    config_init(cfg);
+    const int c_ret = settings_init_config(cfg, run_opts);
 
-    char path[MAX_STR_SIZE];
-
-    /* use default config file path */
-    if (patharg == NULL) {
-        char *user_config_dir = get_user_config_dir();
-        snprintf(path, sizeof(path), "%s%stoxic.conf", user_config_dir, CONFIGDIR);
-        free(user_config_dir);
-
-        /* make sure path exists or is created on first time running */
-        if (!file_exists(path)) {
-            FILE *fp = fopen(path, "w");
-
-            if (fp == NULL) {
-                return -1;
-            }
-
-            fclose(fp);
-        }
-    } else {
-        snprintf(path, sizeof(path), "%s", patharg);
-    }
-
-    if (!config_read_file(cfg, path)) {
+    if (c_ret < 0) {
         config_destroy(cfg);
-        return -1;
+        return c_ret;
     }
 
+    const char *str = NULL;
+
     /* ui */
     if ((setting = config_lookup(cfg, ui_strings.self)) != NULL) {
         config_setting_lookup_bool(setting, ui_strings.timestamps, &s->timestamps);
@@ -548,6 +846,7 @@ int settings_load(struct user_settings *s, const char *patharg)
 
 #ifdef AUDIO
 
+    /* Audio */
     if ((setting = config_lookup(cfg, audio_strings.self)) != NULL) {
         config_setting_lookup_int(setting, audio_strings.input_device, &s->audio_in_dev);
         s->audio_in_dev = s->audio_in_dev < 0 || s->audio_in_dev > MAX_DEVICES ? 0 : s->audio_in_dev;
@@ -571,6 +870,7 @@ int settings_load(struct user_settings *s, const char *patharg)
 
 #ifdef SOUND_NOTIFY
 
+    /* Sound notifications */
     if ((setting = config_lookup(cfg, sound_strings.self)) != NULL) {
         if ((config_setting_lookup_string(setting, sound_strings.notif_error, &str) != CONFIG_TRUE) ||
                 !set_sound(notif_error, str)) {
