@@ -1,7 +1,7 @@
 /*  settings.h
  *
  *
- *  Copyright (C) 2014 Toxic All Rights Reserved.
+ *  Copyright (C) 2024 Toxic All Rights Reserved.
  *
  *  This file is part of Toxic.
  *
@@ -27,13 +27,16 @@
 
 #include <tox/tox.h>
 
+#include "run_options.h"
+#include "toxic_constants.h"
+
 /* Represents line_* hints max strlen */
 #define LINE_HINT_MAX 3
 
 #define PASSWORD_EVAL_MAX 512
 
 /* Holds user setting values defined in the toxic config file. */
-struct user_settings {
+typedef struct Client_Config {
     int autolog;           /* boolean */
     int alerts;            /* boolean */
 
@@ -97,9 +100,7 @@ struct user_settings {
     int chat_audio_channels;
     int push_to_talk;      /* boolean */
 #endif
-};
-
-extern struct user_settings *user_settings;
+} Client_Config;
 
 enum settings_values {
     AUTOLOG_OFF = 0,
@@ -141,6 +142,68 @@ enum settings_values {
 #define LOG_TIMESTAMP_DEFAULT  "%Y/%m/%d [%H:%M:%S]"
 #define MPLEX_AWAY_NOTE "Away from keyboard, be back soon!"
 
-int settings_load(struct user_settings *s, const char *patharg);
+/*
+ * Loads the config file into `run_opts` and creates an empty file if it does not
+ * already exist. This function must be called before any other `settings_load` function.
+ *
+ * If the client has set a custom config file via the run option it will be
+ * prioritized.
+ *
+ * If the client is using the default tox data file and has not specified a custom
+ * config file, the default config file in the user config directory is used
+ *
+ * If the client is using a custom tox data file, the config path and filename will be
+ * identical to those of the data file, but with a `.conf` file extension.
+ *
+ * `data_path` is the name of the tox profile being used.
+ *
+ * Returns true if config file is successfully loaded.
+ */
+bool settings_load_config_file(Run_Options *run_opts, const char *data_path);
+
+/*
+ * Loads general toxic settings from the toxic config file pointed to by `patharg'.
+ *
+ * Return 0 on success.
+ * Return -1 if we fail to open the file path.
+ * Return -2 if libconfig fails to read the config file.
+ */
+int settings_load_main(Client_Config *s, const Run_Options *run_opts);
+
+/*
+ * Loads friend config settings from the toxic config file pointed to by `patharg`.
+ * If `patharg` is null, the default config path is used.
+ *
+ * Return 0 on success (or if no config entry for friends exists).
+ * Return -1 if we fail to open the file path.
+ * Return -2 if libconfig fails to read the config file.
+ *
+ * This function will have no effect on friends that are added in the future.
+ */
+int settings_load_friends(const Run_Options *run_opts);
+
+/*
+ * Loads groupchat config settings from the toxic config file pointed to by `patharg`.
+ * If `patharg` is null, the default config path is used.
+ *
+ * Return 0 on success (or if no config entry for groupchats exists).
+ * Return -1 if we fail to open the file path.
+ * Return -2 if libconfig fails to read the config file.
+ *
+ * This function will have no effect on groupchat instances that are created in the future.
+ */
+int settings_load_groups(const Run_Options *run_opts);
+
+/*
+ * Loads conference config settings from the toxic config file pointed to by `patharg`.
+ * If `patharg` is null, the default config path is used.
+ *
+ * Return 0 on success (or if no config entry for conferences exists).
+ * Return -1 if we fail to open the file path.
+ * Return -2 if libconfig fails to read the config file.
+ *
+ * This function will have no effect on conference instances that are created in the future.
+ */
+int settings_load_conferences(const Run_Options *run_opts);
 
 #endif /* SETTINGS_H */
