@@ -1,7 +1,7 @@
 /*  term_mplex.c
  *
  *
- *  Copyright (C) 2015 Toxic All Rights Reserved.
+ *  Copyright (C) 2024 Toxic All Rights Reserved.
  *
  *  This file is part of Toxic.
  *
@@ -39,8 +39,6 @@
 #include "toxic.h"
 #include "windows.h"
 
-extern struct ToxWindow *prompt;
-extern struct user_settings *user_settings;
 extern struct Winthread Winthread;
 
 #if defined(PATH_MAX) && PATH_MAX > 512
@@ -377,8 +375,14 @@ static int mplex_is_detached(void)
     return gnu_screen_is_detached()  ||  tmux_is_detached();
 }
 
-static void mplex_timer_handler(Tox *m)
+static void mplex_timer_handler(Toxic *toxic)
 {
+    if (toxic == NULL) {
+        return;
+    }
+
+    ToxWindow *home_window = toxic->home_window;
+
     Tox_User_Status current_status, new_status;
     const char *new_note;
 
@@ -389,7 +393,7 @@ static void mplex_timer_handler(Tox *m)
     int detached = mplex_is_detached();
 
     pthread_mutex_lock(&Winthread.lock);
-    current_status = tox_self_get_status(m);
+    current_status = tox_self_get_status(toxic->tox);
     pthread_mutex_unlock(&Winthread.lock);
 
     if (auto_away_active && current_status == TOX_USER_STATUS_AWAY && !detached) {
@@ -401,11 +405,11 @@ static void mplex_timer_handler(Tox *m)
         prev_status = current_status;
         new_status = TOX_USER_STATUS_AWAY;
         pthread_mutex_lock(&Winthread.lock);
-        size_t slen = tox_self_get_status_message_size(m);
-        tox_self_get_status_message(m, (uint8_t *) prev_note);
+        size_t slen = tox_self_get_status_message_size(toxic->tox);
+        tox_self_get_status_message(toxic->tox, (uint8_t *) prev_note);
         prev_note[slen] = '\0';
         pthread_mutex_unlock(&Winthread.lock);
-        new_note = user_settings->mplex_away_note;
+        new_note = toxic->c_config->mplex_away_note;
     } else {
         return;
     }
@@ -418,31 +422,31 @@ static void mplex_timer_handler(Tox *m)
     snprintf(note_str, sizeof(status_str), "/note %s", new_note);
 
     pthread_mutex_lock(&Winthread.lock);
-    execute(prompt->chatwin->history, prompt, m, status_str, GLOBAL_COMMAND_MODE);
-    execute(prompt->chatwin->history, prompt, m, note_str, GLOBAL_COMMAND_MODE);
+    execute(home_window->chatwin->history, home_window, toxic, status_str, GLOBAL_COMMAND_MODE);
+    execute(home_window->chatwin->history, home_window, toxic, note_str, GLOBAL_COMMAND_MODE);
     pthread_mutex_unlock(&Winthread.lock);
 }
 
 /* Time in seconds between calls to mplex_timer_handler */
 #define MPLEX_TIMER_INTERVAL 5
 
-void *mplex_timer_thread(void *data)
+static void *mplex_timer_thread(void *data)
 {
-    Tox *m = (Tox *) data;
+    Toxic *toxic = (Toxic *) data;
 
     while (true) {
         sleep(MPLEX_TIMER_INTERVAL);
-        mplex_timer_handler(m);
+        mplex_timer_handler(toxic);
     }
 }
 
-int init_mplex_away_timer(Tox *m)
+int init_mplex_away_timer(Toxic *toxic)
 {
-    if (! detect_mplex()) {
+    if (!detect_mplex()) {
         return 0;
     }
 
-    if (! user_settings->mplex_away) {
+    if (!toxic->c_config->mplex_away) {
         return 0;
     }
 
@@ -451,7 +455,7 @@ int init_mplex_away_timer(Tox *m)
         return -1;
     }
 
-    if (pthread_create(&mplex_tid, NULL, mplex_timer_thread, (void *) m) != 0) {
+    if (pthread_create(&mplex_tid, NULL, mplex_timer_thread, (void *)toxic) != 0) {
         return -1;
     }
 
