@@ -1,7 +1,7 @@
 /*  term_mplex.h
  *
  *
- *  Copyright (C) 2015 Toxic All Rights Reserved.
+ *  Copyright (C) 2024 Toxic All Rights Reserved.
  *
  *  This file is part of Toxic.
  *
@@ -27,7 +27,7 @@
  * yes, it initializes a timer which periodically checks the attached/detached
  * state of the terminal and updates away status accordingly.
  */
-int init_mplex_away_timer(Tox *m);
+int init_mplex_away_timer(Toxic *toxic);
 
 void lock_status(void);
 void unlock_status(void);
