@@ -1,7 +1,7 @@
 /*  windows.c
  *
  *
- *  Copyright (C) 2014 Toxic All Rights Reserved.
+ *  Copyright (C) 2024 Toxic All Rights Reserved.
  *
  *  This file is part of Toxic.
  *
@@ -32,6 +32,7 @@
 #include "friendlist.h"
 #include "groupchats.h"
 #include "line_info.h"
+#include "log.h"
 #include "misc_tools.h"
 #include "prompt.h"
 #include "settings.h"
@@ -47,70 +48,76 @@ static uint8_t active_window_index;
 static int num_active_windows;
 
 /* CALLBACKS START */
-void on_friend_request(Tox *m, const uint8_t *public_key, const uint8_t *data, size_t length, void *userdata)
+void on_friend_request(Tox *tox, const uint8_t *public_key, const uint8_t *data, size_t length, void *userdata)
 {
-    UNUSED_VAR(userdata);
+    UNUSED_VAR(tox);
+    Toxic *toxic = (Toxic *) userdata;
 
     char msg[MAX_STR_SIZE + 1];
     length = copy_tox_str(msg, sizeof(msg), (const char *) data, length);
 
     for (uint8_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
         if (windows[i] != NULL && windows[i]->onFriendRequest != NULL) {
-            windows[i]->onFriendRequest(windows[i], m, (const char *) public_key, msg, length);
+            windows[i]->onFriendRequest(windows[i], toxic, (const char *) public_key, msg, length);
         }
     }
 }
 
-void on_friend_connection_status(Tox *m, uint32_t friendnumber, Tox_Connection connection_status, void *userdata)
+void on_friend_connection_status(Tox *tox, uint32_t friendnumber, Tox_Connection connection_status, void *userdata)
 {
-    UNUSED_VAR(userdata);
+    UNUSED_VAR(tox);
+    Toxic *toxic = (Toxic *) userdata;
 
-    on_avatar_friend_connection_status(m, friendnumber, connection_status);
+    on_avatar_friend_connection_status(toxic, friendnumber, connection_status);
 
     for (uint8_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
         if (windows[i] != NULL && windows[i]->onConnectionChange != NULL) {
-            windows[i]->onConnectionChange(windows[i], m, friendnumber, connection_status);
+            windows[i]->onConnectionChange(windows[i], toxic, friendnumber, connection_status);
         }
     }
 
     flag_interface_refresh();
 }
 
-void on_friend_typing(Tox *m, uint32_t friendnumber, bool is_typing, void *userdata)
+void on_friend_typing(Tox *tox, uint32_t friendnumber, bool is_typing, void *userdata)
 {
-    UNUSED_VAR(userdata);
+    UNUSED_VAR(tox);
+    Toxic *toxic = (Toxic *) userdata;
 
-    if (user_settings->show_typing_other == SHOW_TYPING_OFF) {
+    if (toxic->c_config->show_typing_other == SHOW_TYPING_OFF) {
         return;
     }
 
     for (uint8_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
         if (windows[i] != NULL && windows[i]->onTypingChange != NULL) {
-            windows[i]->onTypingChange(windows[i], m, friendnumber, is_typing);
+            windows[i]->onTypingChange(windows[i], toxic, friendnumber, is_typing);
         }
     }
 
     flag_interface_refresh();
 }
 
-void on_friend_message(Tox *m, uint32_t friendnumber, Tox_Message_Type type, const uint8_t *string, size_t length,
+void on_friend_message(Tox *tox, uint32_t friendnumber, Tox_Message_Type type, const uint8_t *string, size_t length,
                        void *userdata)
 {
-    UNUSED_VAR(userdata);
+    UNUSED_VAR(tox);
+    Toxic *toxic = (Toxic *) userdata;
 
     char msg[MAX_STR_SIZE + 1];
     length = copy_tox_str(msg, sizeof(msg), (const char *) string, length);
 
     for (uint8_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
         if (windows[i] != NULL && windows[i]->onMessage != NULL) {
-            windows[i]->onMessage(windows[i], m, friendnumber, type, msg, length);
+            windows[i]->onMessage(windows[i], toxic, friendnumber, type, msg, length);
         }
     }
 }
 
-void on_friend_name(Tox *m, uint32_t friendnumber, const uint8_t *string, size_t length, void *userdata)
+void on_friend_name(Tox *tox, uint32_t friendnumber, const uint8_t *string, size_t length, void *userdata)
 {
-    UNUSED_VAR(userdata);
+    UNUSED_VAR(tox);
+
+    Toxic *toxic = (Toxic *) userdata;
 
     char nick[TOXIC_MAX_NAME_LENGTH + 1];
     length = copy_tox_str(nick, sizeof(nick), (const char *) string, length);
@@ -118,19 +125,19 @@ void on_friend_name(Tox *m, uint32_t friendnumber, const uint8_t *string, size_t
 
     for (uint8_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
         if (windows[i] != NULL && windows[i]->onNickChange != NULL) {
-            windows[i]->onNickChange(windows[i], m, friendnumber, nick, length);
+            windows[i]->onNickChange(windows[i], toxic, friendnumber, nick, length);
         }
     }
 
     flag_interface_refresh();
 
-    store_data(m, DATA_FILE);
+    store_data(toxic);
 }
 
-void on_friend_status_message(Tox *m, uint32_t friendnumber, const uint8_t *string, size_t length, void *userdata)
+void on_friend_status_message(Tox *tox, uint32_t friendnumber, const uint8_t *string, size_t length, void *userdata)
 {
+    UNUSED_VAR(tox);
     UNUSED_VAR(userdata);
-    UNUSED_VAR(m);
 
     char msg[TOX_MAX_STATUS_MESSAGE_LENGTH + 1];
     length = copy_tox_str(msg, sizeof(msg), (const char *) string, length);
@@ -145,74 +152,80 @@ void on_friend_status_message(Tox *m, uint32_t friendnumber, const uint8_t *stri
     flag_interface_refresh();
 }
 
-void on_friend_status(Tox *m, uint32_t friendnumber, Tox_User_Status status, void *userdata)
+void on_friend_status(Tox *tox, uint32_t friendnumber, Tox_User_Status status, void *userdata)
 {
-    UNUSED_VAR(userdata);
+    UNUSED_VAR(tox);
+    Toxic *toxic = (Toxic *) userdata;
 
     for (uint8_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
         if (windows[i] != NULL && windows[i]->onStatusChange != NULL) {
-            windows[i]->onStatusChange(windows[i], m, friendnumber, status);
+            windows[i]->onStatusChange(windows[i], toxic, friendnumber, status);
         }
     }
 
     flag_interface_refresh();
 }
 
-void on_friend_added(Tox *m, uint32_t friendnumber, bool sort)
+// TODO: This isn't a proper tox callback. Refactor with friendlist.
+void on_friend_added(Toxic *toxic, uint32_t friendnumber, bool sort)
 {
     for (uint8_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
         if (windows[i] != NULL && windows[i]->onFriendAdded != NULL) {
-            windows[i]->onFriendAdded(windows[i], m, friendnumber, sort);
+            windows[i]->onFriendAdded(windows[i], toxic, friendnumber, sort);
         }
     }
 
-    store_data(m, DATA_FILE);
+    store_data(toxic);
 }
 
-void on_conference_message(Tox *m, uint32_t conferencenumber, uint32_t peernumber, Tox_Message_Type type,
+void on_conference_message(Tox *tox, uint32_t conferencenumber, uint32_t peernumber, Tox_Message_Type type,
                            const uint8_t *message, size_t length, void *userdata)
 {
-    UNUSED_VAR(userdata);
+    UNUSED_VAR(tox);
+    Toxic *toxic = (Toxic *) userdata;
 
     char msg[MAX_STR_SIZE + 1];
     length = copy_tox_str(msg, sizeof(msg), (const char *) message, length);
 
     for (uint8_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
         if (windows[i] != NULL && windows[i]->onConferenceMessage != NULL) {
-            windows[i]->onConferenceMessage(windows[i], m, conferencenumber, peernumber, type, msg, length);
+            windows[i]->onConferenceMessage(windows[i], toxic, conferencenumber, peernumber, type, msg, length);
         }
     }
 }
 
-void on_conference_invite(Tox *m, uint32_t friendnumber, Tox_Conference_Type type, const uint8_t *conference_pub_key,
+void on_conference_invite(Tox *tox, uint32_t friendnumber, Tox_Conference_Type type, const uint8_t *conference_pub_key,
                           size_t length, void *userdata)
 {
-    UNUSED_VAR(userdata);
+    UNUSED_VAR(tox);
+    Toxic *toxic = (Toxic *) userdata;
 
     for (uint8_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
         if (windows[i] != NULL && windows[i]->onConferenceInvite != NULL) {
-            windows[i]->onConferenceInvite(windows[i], m, friendnumber, type, (const char *) conference_pub_key, length);
+            windows[i]->onConferenceInvite(windows[i], toxic, friendnumber, type, (const char *) conference_pub_key, length);
         }
     }
 }
 
-void on_conference_peer_list_changed(Tox *m, uint32_t conferencenumber, void *userdata)
+void on_conference_peer_list_changed(Tox *tox, uint32_t conferencenumber, void *userdata)
 {
-    UNUSED_VAR(userdata);
+    UNUSED_VAR(tox);
+    Toxic *toxic = (Toxic *) userdata;
 
     for (uint8_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
         if (windows[i] != NULL && windows[i]->onConferenceNameListChange != NULL) {
-            windows[i]->onConferenceNameListChange(windows[i], m, conferencenumber);
+            windows[i]->onConferenceNameListChange(windows[i], toxic, conferencenumber);
         }
     }
 
     flag_interface_refresh();
 }
 
-void on_conference_peer_name(Tox *m, uint32_t conferencenumber, uint32_t peernumber, const uint8_t *name,
+void on_conference_peer_name(Tox *tox, uint32_t conferencenumber, uint32_t peernumber, const uint8_t *name,
                              size_t length, void *userdata)
 {
-    UNUSED_VAR(userdata);
+    UNUSED_VAR(tox);
+    Toxic *toxic = (Toxic *) userdata;
 
     char nick[TOXIC_MAX_NAME_LENGTH + 1];
     length = copy_tox_str(nick, sizeof(nick), (const char *) name, length);
@@ -220,129 +233,132 @@ void on_conference_peer_name(Tox *m, uint32_t conferencenumber, uint32_t peernum
 
     for (uint8_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
         if (windows[i] != NULL && windows[i]->onConferencePeerNameChange != NULL) {
-            windows[i]->onConferencePeerNameChange(windows[i], m, conferencenumber, peernumber, nick, length);
+            windows[i]->onConferencePeerNameChange(windows[i], toxic, conferencenumber, peernumber, nick, length);
         }
     }
 }
 
-void on_conference_title(Tox *m, uint32_t conferencenumber, uint32_t peernumber, const uint8_t *title, size_t length,
+void on_conference_title(Tox *tox, uint32_t conferencenumber, uint32_t peernumber, const uint8_t *title, size_t length,
                          void *userdata)
 {
-    UNUSED_VAR(userdata);
+    UNUSED_VAR(tox);
+    Toxic *toxic = (Toxic *) userdata;
 
     char data[MAX_STR_SIZE + 1];
     length = copy_tox_str(data, sizeof(data), (const char *) title, length);
 
     for (uint8_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
         if (windows[i] != NULL && windows[i]->onConferenceTitleChange != NULL) {
-            windows[i]->onConferenceTitleChange(windows[i], m, conferencenumber, peernumber, data, length);
+            windows[i]->onConferenceTitleChange(windows[i], toxic, conferencenumber, peernumber, data, length);
         }
     }
 }
 
-void on_file_chunk_request(Tox *m, uint32_t friendnumber, uint32_t filenumber, uint64_t position,
+void on_file_chunk_request(Tox *tox, uint32_t friendnumber, uint32_t filenumber, uint64_t position,
                            size_t length, void *userdata)
 {
-    UNUSED_VAR(userdata);
+    Toxic *toxic = (Toxic *) userdata;
 
     FileTransfer *ft = get_file_transfer_struct(friendnumber, filenumber);
 
-    if (!ft) {
+    if (ft == NULL) {
         return;
     }
 
     if (ft->file_type == TOX_FILE_KIND_AVATAR) {
-        on_avatar_chunk_request(m, ft, position, length);
+        on_avatar_chunk_request(toxic, ft, position, length);
         return;
     }
 
     for (uint8_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
         if (windows[i] != NULL && windows[i]->onFileChunkRequest != NULL) {
-            windows[i]->onFileChunkRequest(windows[i], m, friendnumber, filenumber, position, length);
+            windows[i]->onFileChunkRequest(windows[i], toxic, friendnumber, filenumber, position, length);
         }
     }
 }
 
-void on_file_recv_chunk(Tox *m, uint32_t friendnumber, uint32_t filenumber, uint64_t position,
+void on_file_recv_chunk(Tox *tox, uint32_t friendnumber, uint32_t filenumber, uint64_t position,
                         const uint8_t *data, size_t length, void *userdata)
 {
-    UNUSED_VAR(userdata);
+    UNUSED_VAR(tox);
+    Toxic *toxic = (Toxic *) userdata;
 
-    FileTransfer *ft = get_file_transfer_struct(friendnumber, filenumber);
+    const FileTransfer *ft = get_file_transfer_struct(friendnumber, filenumber);
 
-    if (!ft) {
+    if (ft == NULL) {
         return;
     }
 
     for (uint8_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
         if (windows[i] != NULL && windows[i]->onFileRecvChunk != NULL) {
-            windows[i]->onFileRecvChunk(windows[i], m, friendnumber, filenumber, position, (const char *) data, length);
+            windows[i]->onFileRecvChunk(windows[i], toxic, friendnumber, filenumber, position, (const char *) data, length);
         }
     }
 }
 
-void on_file_recv_control(Tox *m, uint32_t friendnumber, uint32_t filenumber, Tox_File_Control control,
+void on_file_recv_control(Tox *tox, uint32_t friendnumber, uint32_t filenumber, Tox_File_Control control,
                           void *userdata)
 {
-    UNUSED_VAR(userdata);
+    Toxic *toxic = (Toxic *) userdata;
 
     FileTransfer *ft = get_file_transfer_struct(friendnumber, filenumber);
 
-    if (!ft) {
+    if (ft == NULL) {
         return;
     }
 
     if (ft->file_type == TOX_FILE_KIND_AVATAR) {
-        on_avatar_file_control(m, ft, control);
+        on_avatar_file_control(toxic, ft, control);
         return;
     }
 
     for (uint8_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
         if (windows[i] != NULL && windows[i]->onFileControl != NULL) {
-            windows[i]->onFileControl(windows[i], m, friendnumber, filenumber, control);
+            windows[i]->onFileControl(windows[i], toxic, friendnumber, filenumber, control);
         }
     }
 }
 
-void on_file_recv(Tox *m, uint32_t friendnumber, uint32_t filenumber, uint32_t kind, uint64_t file_size,
+void on_file_recv(Tox *tox, uint32_t friendnumber, uint32_t filenumber, uint32_t kind, uint64_t file_size,
                   const uint8_t *filename, size_t filename_length, void *userdata)
 {
-    UNUSED_VAR(userdata);
+    Toxic *toxic = (Toxic *) userdata;
 
     /* We don't care about receiving avatars */
     if (kind != TOX_FILE_KIND_DATA) {
-        tox_file_control(m, friendnumber, filenumber, TOX_FILE_CONTROL_CANCEL, NULL);
+        tox_file_control(tox, friendnumber, filenumber, TOX_FILE_CONTROL_CANCEL, NULL);
         return;
     }
 
     for (uint8_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
         if (windows[i] != NULL && windows[i]->onFileRecv != NULL) {
-            windows[i]->onFileRecv(windows[i], m, friendnumber, filenumber, file_size, (const char *) filename,
+            windows[i]->onFileRecv(windows[i], toxic, friendnumber, filenumber, file_size, (const char *) filename,
                                    filename_length);
         }
     }
 }
 
-void on_friend_read_receipt(Tox *m, uint32_t friendnumber, uint32_t receipt, void *userdata)
+void on_friend_read_receipt(Tox *tox, uint32_t friendnumber, uint32_t receipt, void *userdata)
 {
-    UNUSED_VAR(userdata);
+    UNUSED_VAR(tox);
+    Toxic *toxic = (Toxic *) userdata;
 
     for (uint8_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
         if (windows[i] != NULL && windows[i]->onReadReceipt != NULL) {
-            windows[i]->onReadReceipt(windows[i], m, friendnumber, receipt);
+            windows[i]->onReadReceipt(windows[i], toxic, friendnumber, receipt);
         }
     }
 }
 
-void on_lossless_custom_packet(Tox *m, uint32_t friendnumber, const uint8_t *data, size_t length, void *userdata)
+void on_lossless_custom_packet(Tox *tox, uint32_t friendnumber, const uint8_t *data, size_t length, void *userdata)
 {
-    UNUSED_VAR(userdata);
-
     if (length == 0 || data == NULL) {
         return;
     }
 
-    uint8_t type = data[0];
+    Toxic *toxic = (Toxic *) userdata;
+
+    const uint8_t type = data[0];
 
     switch (type) {
 #ifdef GAMES
@@ -352,7 +368,7 @@ void on_lossless_custom_packet(Tox *m, uint32_t friendnumber, const uint8_t *dat
                 ToxWindow *window = windows[i];
 
                 if (window != NULL && window->onGameInvite != NULL) {
-                    window->onGameInvite(window, m, friendnumber, data + 1, length - 1);
+                    window->onGameInvite(window, toxic, friendnumber, data + 1, length - 1);
                 }
             }
 
@@ -364,13 +380,15 @@ void on_lossless_custom_packet(Tox *m, uint32_t friendnumber, const uint8_t *dat
                 ToxWindow *window = windows[i];
 
                 if (window != NULL && window->onGameData != NULL) {
-                    window->onGameData(window, m, friendnumber, data + 1, length - 1);
+                    window->onGameData(window, toxic, friendnumber, data + 1, length - 1);
                 }
             }
 
             break;
         }
 
+#else
+        UNUSED_VAR(toxic);
 #endif // GAMES
 
         default: {
@@ -380,75 +398,96 @@ void on_lossless_custom_packet(Tox *m, uint32_t friendnumber, const uint8_t *dat
     }
 }
 
-void on_group_invite(Tox *m, uint32_t friendnumber, const uint8_t *invite_data, size_t length,
+void on_group_invite(Tox *tox, uint32_t friendnumber, const uint8_t *invite_data, size_t length,
                      const uint8_t *group_name,
                      size_t group_name_length, void *userdata)
 {
+    Toxic *toxic = (Toxic *) userdata;
+
     char gname[MAX_STR_SIZE + 1];
     group_name_length = copy_tox_str(gname, sizeof(gname), (const char *) group_name, group_name_length);
 
     for (size_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
         if (windows[i] != NULL && windows[i]->onGroupInvite != NULL) {
-            windows[i]->onGroupInvite(windows[i], m, friendnumber, (char *) invite_data, length, gname, group_name_length);
+            windows[i]->onGroupInvite(windows[i], toxic, friendnumber, (char *) invite_data, length, gname, group_name_length);
         }
     }
 }
 
-void on_group_message(Tox *m, uint32_t groupnumber, uint32_t peer_id, TOX_MESSAGE_TYPE type,
+void on_group_message(Tox *tox, uint32_t groupnumber, uint32_t peer_id, TOX_MESSAGE_TYPE type,
                       const uint8_t *message, size_t length, uint32_t message_id, void *userdata)
 {
     UNUSED_VAR(message_id);
+    UNUSED_VAR(tox);
+
+    Toxic *toxic = (Toxic *) userdata;
 
     char msg[MAX_STR_SIZE + 1];
     length = copy_tox_str(msg, sizeof(msg), (const char *) message, length);
 
     for (size_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
         if (windows[i] != NULL && windows[i]->onGroupMessage != NULL) {
-            windows[i]->onGroupMessage(windows[i], m, groupnumber, peer_id, type, msg, length);
+            windows[i]->onGroupMessage(windows[i], toxic, groupnumber, peer_id, type, msg, length);
         }
     }
 }
 
-void on_group_private_message(Tox *m, uint32_t groupnumber, uint32_t peer_id, TOX_MESSAGE_TYPE type,
+void on_group_private_message(Tox *tox, uint32_t groupnumber, uint32_t peer_id, TOX_MESSAGE_TYPE type,
                               const uint8_t *message,
                               size_t length, void *userdata)
 {
+    UNUSED_VAR(tox);
+
+    Toxic *toxic = (Toxic *) userdata;
+
     char msg[MAX_STR_SIZE + 1];
     length = copy_tox_str(msg, sizeof(msg), (const char *) message, length);
 
     for (size_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
         if (windows[i] != NULL && windows[i]->onGroupPrivateMessage != NULL) {
-            windows[i]->onGroupPrivateMessage(windows[i], m, groupnumber, peer_id, msg, length);
+            windows[i]->onGroupPrivateMessage(windows[i], toxic, groupnumber, peer_id, msg, length);
         }
     }
 }
 
-void on_group_status_change(Tox *m, uint32_t groupnumber, uint32_t peer_id, TOX_USER_STATUS status, void *userdata)
+void on_group_status_change(Tox *tox, uint32_t groupnumber, uint32_t peer_id, TOX_USER_STATUS status, void *userdata)
 {
+    UNUSED_VAR(tox);
+
+    Toxic *toxic = (Toxic *) userdata;
+
     for (size_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
         if (windows[i] != NULL && windows[i]->onGroupStatusChange != NULL) {
-            windows[i]->onGroupStatusChange(windows[i], m, groupnumber, peer_id, status);
+            windows[i]->onGroupStatusChange(windows[i], toxic, groupnumber, peer_id, status);
         }
     }
 
     flag_interface_refresh();
 }
 
-void on_group_peer_join(Tox *m, uint32_t groupnumber, uint32_t peer_id, void *userdata)
+void on_group_peer_join(Tox *tox, uint32_t groupnumber, uint32_t peer_id, void *userdata)
 {
+    UNUSED_VAR(tox);
+
+    Toxic *toxic = (Toxic *) userdata;
+
     for (size_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
         if (windows[i] != NULL && windows[i]->onGroupPeerJoin != NULL) {
-            windows[i]->onGroupPeerJoin(windows[i], m, groupnumber, peer_id);
+            windows[i]->onGroupPeerJoin(windows[i], toxic, groupnumber, peer_id);
         }
     }
 
     flag_interface_refresh();
 }
 
-void on_group_peer_exit(Tox *m, uint32_t groupnumber, uint32_t peer_id, Tox_Group_Exit_Type exit_type,
+void on_group_peer_exit(Tox *tox, uint32_t groupnumber, uint32_t peer_id, Tox_Group_Exit_Type exit_type,
                         const uint8_t *nick,
                         size_t nick_len, const uint8_t *part_message, size_t length, void *userdata)
 {
+    UNUSED_VAR(tox);
+
+    Toxic *toxic = (Toxic *) userdata;
+
     char toxic_nick[TOXIC_MAX_NAME_LENGTH + 1];
     nick_len = copy_tox_str(toxic_nick, sizeof(toxic_nick), (const char *) nick, nick_len);
 
@@ -461,114 +500,154 @@ void on_group_peer_exit(Tox *m, uint32_t groupnumber, uint32_t peer_id, Tox_Grou
 
     for (size_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
         if (windows[i] != NULL && windows[i]->onGroupPeerExit != NULL) {
-            windows[i]->onGroupPeerExit(windows[i], m, groupnumber, peer_id, exit_type, toxic_nick, nick_len, buf, buf_len);
+            windows[i]->onGroupPeerExit(windows[i], toxic, groupnumber, peer_id, exit_type, toxic_nick, nick_len, buf, buf_len);
         }
     }
 }
 
-void on_group_topic_change(Tox *m, uint32_t groupnumber, uint32_t peer_id, const uint8_t *topic, size_t length,
+void on_group_topic_change(Tox *tox, uint32_t groupnumber, uint32_t peer_id, const uint8_t *topic, size_t length,
                            void *userdata)
 {
+    UNUSED_VAR(tox);
+
+    Toxic *toxic = (Toxic *) userdata;
+
     char data[MAX_STR_SIZE + 1];
     length = copy_tox_str(data, sizeof(data), (const char *) topic, length);
 
     for (size_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
         if (windows[i] != NULL && windows[i]->onGroupTopicChange != NULL) {
-            windows[i]->onGroupTopicChange(windows[i], m, groupnumber, peer_id, data, length);
+            windows[i]->onGroupTopicChange(windows[i], toxic, groupnumber, peer_id, data, length);
         }
     }
 }
 
-void on_group_peer_limit(Tox *m, uint32_t groupnumber, uint32_t peer_limit, void *userdata)
+void on_group_peer_limit(Tox *tox, uint32_t groupnumber, uint32_t peer_limit, void *userdata)
 {
+    UNUSED_VAR(tox);
+
+    Toxic *toxic = (Toxic *) userdata;
+
     for (size_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
         if (windows[i] != NULL && windows[i]->onGroupPeerLimit != NULL) {
-            windows[i]->onGroupPeerLimit(windows[i], m, groupnumber, peer_limit);
+            windows[i]->onGroupPeerLimit(windows[i], toxic, groupnumber, peer_limit);
         }
     }
 }
 
-void on_group_privacy_state(Tox *m, uint32_t groupnumber, Tox_Group_Privacy_State privacy_state, void *userdata)
+void on_group_privacy_state(Tox *tox, uint32_t groupnumber, Tox_Group_Privacy_State privacy_state, void *userdata)
 {
+    UNUSED_VAR(tox);
+
+    Toxic *toxic = (Toxic *) userdata;
+
     for (size_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
         if (windows[i] != NULL && windows[i]->onGroupPrivacyState != NULL) {
-            windows[i]->onGroupPrivacyState(windows[i], m, groupnumber, privacy_state);
+            windows[i]->onGroupPrivacyState(windows[i], toxic, groupnumber, privacy_state);
         }
     }
 }
 
-void on_group_topic_lock(Tox *m, uint32_t groupnumber, Tox_Group_Topic_Lock topic_lock, void *userdata)
+void on_group_topic_lock(Tox *tox, uint32_t groupnumber, Tox_Group_Topic_Lock topic_lock, void *userdata)
 {
+    UNUSED_VAR(tox);
+
+    Toxic *toxic = (Toxic *) userdata;
+
     for (size_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
         if (windows[i] != NULL && windows[i]->onGroupTopicLock != NULL) {
-            windows[i]->onGroupTopicLock(windows[i], m, groupnumber, topic_lock);
+            windows[i]->onGroupTopicLock(windows[i], toxic, groupnumber, topic_lock);
         }
     }
 }
 
-void on_group_password(Tox *m, uint32_t groupnumber, const uint8_t *password, size_t length, void *userdata)
+void on_group_password(Tox *tox, uint32_t groupnumber, const uint8_t *password, size_t length, void *userdata)
 {
+    UNUSED_VAR(tox);
+
+    Toxic *toxic = (Toxic *) userdata;
+
     for (size_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
         if (windows[i] != NULL && windows[i]->onGroupPassword != NULL) {
-            windows[i]->onGroupPassword(windows[i], m, groupnumber, (char *) password, length);
+            windows[i]->onGroupPassword(windows[i], toxic, groupnumber, (char *) password, length);
         }
     }
 }
 
-void on_group_nick_change(Tox *m, uint32_t groupnumber, uint32_t peer_id, const uint8_t *newname, size_t length,
+void on_group_nick_change(Tox *tox, uint32_t groupnumber, uint32_t peer_id, const uint8_t *newname, size_t length,
                           void *userdata)
 {
+    UNUSED_VAR(tox);
+
+    Toxic *toxic = (Toxic *) userdata;
+
     char name[TOXIC_MAX_NAME_LENGTH + 1];
     length = copy_tox_str(name, sizeof(name), (const char *) newname, length);
     filter_str(name, length);
 
     for (size_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
         if (windows[i] != NULL && windows[i]->onGroupNickChange != NULL) {
-            windows[i]->onGroupNickChange(windows[i], m, groupnumber, peer_id, name, length);
+            windows[i]->onGroupNickChange(windows[i], toxic, groupnumber, peer_id, name, length);
         }
     }
 }
 
-void on_group_self_join(Tox *m, uint32_t groupnumber, void *userdata)
+void on_group_self_join(Tox *tox, uint32_t groupnumber, void *userdata)
 {
+    UNUSED_VAR(tox);
+
+    Toxic *toxic = (Toxic *) userdata;
+
     for (size_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
         if (windows[i] != NULL && windows[i]->onGroupSelfJoin != NULL) {
-            windows[i]->onGroupSelfJoin(windows[i], m, groupnumber);
+            windows[i]->onGroupSelfJoin(windows[i], toxic, groupnumber);
         }
     }
 }
 
-void on_group_rejected(Tox *m, uint32_t groupnumber, Tox_Group_Join_Fail type, void *userdata)
+void on_group_rejected(Tox *tox, uint32_t groupnumber, Tox_Group_Join_Fail type, void *userdata)
 {
+    UNUSED_VAR(tox);
+
+    Toxic *toxic = (Toxic *) userdata;
+
     for (size_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
         if (windows[i] != NULL && windows[i]->onGroupRejected != NULL) {
-            windows[i]->onGroupRejected(windows[i], m, groupnumber, type);
+            windows[i]->onGroupRejected(windows[i], toxic, groupnumber, type);
         }
     }
 }
 
-void on_group_moderation(Tox *m, uint32_t groupnumber, uint32_t source_peer_id, uint32_t target_peer_id,
+void on_group_moderation(Tox *tox, uint32_t groupnumber, uint32_t source_peer_id, uint32_t target_peer_id,
                          Tox_Group_Mod_Event type, void *userdata)
 {
+    UNUSED_VAR(tox);
+
+    Toxic *toxic = (Toxic *) userdata;
+
     for (size_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
         if (windows[i] != NULL && windows[i]->onGroupModeration != NULL) {
-            windows[i]->onGroupModeration(windows[i], m, groupnumber, source_peer_id, target_peer_id, type);
+            windows[i]->onGroupModeration(windows[i], toxic, groupnumber, source_peer_id, target_peer_id, type);
         }
     }
 }
 
-void on_group_voice_state(Tox *m, uint32_t groupnumber, Tox_Group_Voice_State voice_state, void *userdata)
+void on_group_voice_state(Tox *tox, uint32_t groupnumber, Tox_Group_Voice_State voice_state, void *userdata)
 {
+    UNUSED_VAR(tox);
+
+    Toxic *toxic = (Toxic *) userdata;
+
     for (size_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
         if (windows[i] != NULL && windows[i]->onGroupVoiceState != NULL) {
-            windows[i]->onGroupVoiceState(windows[i], m, groupnumber, voice_state);
+            windows[i]->onGroupVoiceState(windows[i], toxic, groupnumber, voice_state);
         }
     }
 }
 
 /* CALLBACKS END */
 
-int add_window(Tox *m, ToxWindow *w)
+int add_window(Toxic *toxic, ToxWindow *w)
 {
     if (LINES < 2) {
         return -1;
@@ -581,6 +660,7 @@ int add_window(Tox *m, ToxWindow *w)
 
         w->index = i;
         w->window = newwin(LINES, COLS, 0, 0);
+        w->colour = BAR_TEXT;
 
         if (w->window == NULL) {
             return -1;
@@ -593,7 +673,7 @@ int add_window(Tox *m, ToxWindow *w)
         windows[i] = w;
 
         if (w->onInit) {
-            w->onInit(w, m);
+            w->onInit(w, toxic);
         }
 
         ++num_active_windows;
@@ -614,11 +694,11 @@ void set_active_window_index(uint8_t index)
 /* Displays the next window if `ch` is equal to the next window key binding.
  * Otherwise displays the previous window.
  */
-static void set_next_window(int ch)
+static void set_next_window(const Client_Config *c_config, int ch)
 {
     uint8_t index = 0;
 
-    if (ch == user_settings->key_next_tab) {
+    if (ch == c_config->key_next_tab) {
         for (uint8_t i = active_window_index + 1; i < MAX_WINDOWS_NUM; ++i) {
             if (windows[i] != NULL) {
                 index = i;
@@ -640,9 +720,9 @@ static void set_next_window(int ch)
 }
 
 /* Deletes window w and cleans up */
-void del_window(ToxWindow *w)
+void del_window(ToxWindow *w, const Client_Config *c_config)
 {
-    uint8_t idx = w->index;
+    const uint8_t idx = w->index;
     delwin(w->window_bar);
     delwin(w->window);
     free(w);
@@ -652,37 +732,35 @@ void del_window(ToxWindow *w)
     refresh();
 
     if (num_active_windows > 0) {
-        if (active_window_index == 2) {    // if closing current window would bring us to friend list
-            set_next_window(-1);           // skip back to the home window instead. FIXME: magic numbers
+        if (active_window_index == 2) { // if closing current window would bring us to friend list
+            set_next_window(c_config, -1);  // skip back to the home window instead. FIXME: magic numbers
         }
 
-        set_next_window(-1);
+        set_next_window(c_config, -1);
 
         --num_active_windows;
     }
 }
 
-ToxWindow *init_windows(Tox *m)
+void init_windows(Toxic *toxic)
 {
     if (COLS <= CHATBOX_HEIGHT + WINDOW_BAR_HEIGHT) {
         exit_toxic_err("add_window() for prompt failed in init_windows", FATALERR_WININIT);
     }
 
-    prompt = new_prompt();
+    toxic->home_window = new_prompt();
 
-    int n_prompt = add_window(m, prompt);
+    const int win_num = add_window(toxic, toxic->home_window);
 
-    if (n_prompt < 0) {
+    if (win_num != 0) {  // prompt window is always index 0
         exit_toxic_err("add_window() for prompt failed in init_windows", FATALERR_WININIT);
     }
 
-    if (add_window(m, new_friendlist()) == -1) {
+    if (add_window(toxic, new_friendlist()) != 1) {  // friendlist is always index 1
         exit_toxic_err("add_window() for friendlist failed in init_windows", FATALERR_WININIT);
     }
 
-    set_active_window_index(n_prompt);
-
-    return prompt;
+    set_active_window_index(win_num);
 }
 
 void on_window_resize(void)
@@ -795,7 +873,7 @@ static void draw_window_tab(WINDOW *win, ToxWindow *toxwin, bool active_window)
 
     pthread_mutex_unlock(&Winthread.lock);
 
-    WINDOW_TYPE type = toxwin->type;
+    Window_Type type = toxwin->type;
 
     if (active_window) {
         wattron(win, A_BOLD | COLOR_PAIR(BAR_ACCENT));
@@ -816,8 +894,18 @@ static void draw_window_tab(WINDOW *win, ToxWindow *toxwin, bool active_window)
         }
     }
 
-    if (active_window || (type == WINDOW_TYPE_PROMPT || type == WINDOW_TYPE_FRIEND_LIST)) {
+    if (type == WINDOW_TYPE_PROMPT || type == WINDOW_TYPE_FRIEND_LIST) {
+        if (!has_alert) {
+            wattron(win, COLOR_PAIR(toxwin->colour));
+            wprintw(win, "%s", toxwin->name);
+            wattroff(win, COLOR_PAIR(toxwin->colour));
+        } else {
+            wprintw(win, "%s", toxwin->name);
+        }
+    } else if (active_window) {
+        wattron(win, COLOR_PAIR(toxwin->colour));
         wprintw(win, "%s", toxwin->name);
+        wattroff(win, COLOR_PAIR(toxwin->colour));
     } else {
         if (pending_messages > 0) {
             wprintw(win, "%u", pending_messages);
@@ -968,8 +1056,13 @@ static wint_t get_input_sequence_code(void)
     return -1;
 }
 
-void draw_active_window(Tox *m)
+void draw_active_window(Toxic *toxic)
 {
+    if (toxic == NULL) {
+        return;
+    }
+
+    const Client_Config *c_config = toxic->c_config;
     ToxWindow *a = windows[active_window_index];
 
     if (a == NULL) {
@@ -979,19 +1072,19 @@ void draw_active_window(Tox *m)
     pthread_mutex_lock(&Winthread.lock);
     a->alert = WINDOW_ALERT_NONE;
     a->pending_messages = 0;
-    bool flag_refresh = Winthread.flag_refresh;
+    const bool flag_refresh = Winthread.flag_refresh;
     pthread_mutex_unlock(&Winthread.lock);
 
     if (flag_refresh) {
         touchwin(a->window);
-        a->onDraw(a, m);
+        a->onDraw(a, toxic);
         wrefresh(a->window);
     }
 
 #ifdef AUDIO
     else if (a->is_call && timed_out(a->chatwin->infobox.lastupdate, 1)) {
         touchwin(a->window);
-        a->onDraw(a, m);
+        a->onDraw(a, toxic);
         wrefresh(a->window);
     }
 
@@ -1002,7 +1095,7 @@ void draw_active_window(Tox *m)
     if (a->type == WINDOW_TYPE_GAME) {
         if (!flag_refresh) {  // we always want to be continously refreshing game windows
             touchwin(a->window);
-            a->onDraw(a, m);
+            a->onDraw(a, toxic);
             wrefresh(a->window);
         }
 
@@ -1016,11 +1109,11 @@ void draw_active_window(Tox *m)
         flag_interface_refresh();
         pthread_mutex_unlock(&Winthread.lock);
 
-        if (ch == user_settings->key_next_tab || ch == user_settings->key_prev_tab) {
-            set_next_window(ch);
+        if (ch == c_config->key_next_tab || ch == c_config->key_prev_tab) {
+            set_next_window(c_config, ch);
         }
 
-        a->onKey(a, m, ch, false);  // we lock only when necessary in the onKey callback
+        a->onKey(a, toxic, ch, false);  // we lock only when necessary in the onKey callback
 
         return;
     }
@@ -1038,12 +1131,12 @@ void draw_active_window(Tox *m)
     flag_interface_refresh();
     pthread_mutex_unlock(&Winthread.lock);
 
-    if (printable == 0 && (ch == user_settings->key_next_tab || ch == user_settings->key_prev_tab)) {
-        set_next_window((int) ch);
+    if (printable == 0 && (ch == c_config->key_next_tab || ch == c_config->key_prev_tab)) {
+        set_next_window(c_config, (int) ch);
         return;
     } else if ((printable == 0) && (a->type != WINDOW_TYPE_FRIEND_LIST)) {
         pthread_mutex_lock(&Winthread.lock);
-        bool input_ret = a->onKey(a, m, ch, (bool) printable);
+        bool input_ret = a->onKey(a, toxic, ch, (bool) printable);
         pthread_mutex_unlock(&Winthread.lock);
 
         if (input_ret) {
@@ -1059,14 +1152,14 @@ void draw_active_window(Tox *m)
     }
 
     pthread_mutex_lock(&Winthread.lock);
-    a->onKey(a, m, ch, (bool) printable);
+    a->onKey(a, toxic, ch, (bool) printable);
     pthread_mutex_unlock(&Winthread.lock);
 }
 
 /* Refresh inactive windows to prevent scrolling bugs.
  * Call at least once per second.
  */
-void refresh_inactive_windows(void)
+void refresh_inactive_windows(const Client_Config *c_config)
 {
     for (uint8_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
         ToxWindow *toxwin = windows[i];
@@ -1077,7 +1170,7 @@ void refresh_inactive_windows(void)
 
         if ((i != active_window_index) && (toxwin->type != WINDOW_TYPE_FRIEND_LIST)) {
             pthread_mutex_lock(&Winthread.lock);
-            line_info_print(toxwin);
+            line_info_print(toxwin, c_config);
             pthread_mutex_unlock(&Winthread.lock);
         }
     }
@@ -1114,7 +1207,7 @@ int get_num_active_windows(void)
 }
 
 /* Returns the number of active windows of given type. */
-size_t get_num_active_windows_type(WINDOW_TYPE type)
+size_t get_num_active_windows_type(Window_Type type)
 {
     size_t count = 0;
 
@@ -1133,9 +1226,60 @@ size_t get_num_active_windows_type(WINDOW_TYPE type)
     return count;
 }
 
-/* destroys all chat and conference windows (should only be called on shutdown) */
-void kill_all_windows(Tox *m)
+ToxWindow *get_window_by_number_type(uint32_t number, Window_Type type)
 {
+    for (size_t i = 0; i < MAX_WINDOWS_NUM; ++i) {
+        ToxWindow *win = windows[i];
+
+        if (win->type == type && win->num == number) {
+            return win;
+        }
+    }
+
+    return NULL;
+}
+
+bool disable_window_log_by_number_type(uint32_t number, Window_Type type)
+{
+    ToxWindow *win = get_window_by_number_type(number, type);
+
+    if (win == NULL) {
+        return false;
+    }
+
+    ChatContext *ctx = win->chatwin;
+
+    if (ctx == NULL) {
+        return false;
+    }
+
+    log_disable(ctx->log);
+
+    return true;
+}
+
+bool enable_window_log_by_number_type(uint32_t number, Window_Type type)
+{
+    ToxWindow *win = get_window_by_number_type(number, type);
+
+    if (win == NULL) {
+        return false;
+    }
+
+    ChatContext *ctx = win->chatwin;
+
+    if (ctx == NULL) {
+        return false;
+    }
+
+    return log_enable(ctx->log) == 0;
+}
+
+/* destroys all chat and conference windows (should only be called on shutdown) */
+void kill_all_windows(Toxic *toxic)
+{
+    const Client_Config *c_config = toxic->c_config;
+
     for (size_t i = 2; i < MAX_WINDOWS_NUM; ++i) {
         ToxWindow *w = windows[i];
 
@@ -1145,27 +1289,27 @@ void kill_all_windows(Tox *m)
 
         switch (w->type) {
             case WINDOW_TYPE_CHAT: {
-                kill_chat_window(w, m);
+                kill_chat_window(w, toxic);
                 break;
             }
 
             case WINDOW_TYPE_CONFERENCE: {
-                free_conference(w, w->num);
+                free_conference(w, c_config, w->num);
                 break;
             }
 
 #ifdef GAMES
 
             case WINDOW_TYPE_GAME: {
-                game_kill(w);
+                game_kill(w, c_config);
                 break;
             }
 
 #endif // GAMES
 
             case WINDOW_TYPE_GROUPCHAT: {
-                exit_groupchat(w, m, w->num, user_settings->group_part_message,
-                               strlen(user_settings->group_part_message));
+                exit_groupchat(w, toxic, w->num, c_config->group_part_message,
+                               strlen(c_config->group_part_message));
                 break;
             }
 
@@ -1176,6 +1320,6 @@ void kill_all_windows(Tox *m)
     }
 
     /* TODO: use enum instead of magic indices */
-    kill_friendlist(windows[1]);
-    kill_prompt_window(windows[0]);
+    kill_friendlist(windows[1], c_config);
+    kill_prompt_window(toxic->home_window, c_config);
 }
